@@ -40,6 +40,7 @@
  * Todo: check compat API with jhead
  */
 #include "gdcmFilename.h"
+#include "gdcmImageHelper.h"
 #include "gdcmDirectory.h"
 #include "gdcmMediaStorage.h"
 #include "gdcmSmartPointer.h"
@@ -69,13 +70,13 @@
 #include <getopt.h>
 #include <string.h>
 
-bool readsize(const char *str, unsigned int * size)
+static unsigned int readsize(const char *str, unsigned int * size)
 {
-  int n = sscanf( str, "%i,%i", size, size+1);
-  return n == 2;
+  int n = sscanf( str, "%i,%i,%i", size, size+1, size+2);
+  return n == EOF ? 0 : (unsigned int)n;
 }
 
-bool readgeometry(const char *geometry, unsigned int * region)
+static bool readgeometry(const char *geometry, unsigned int * region)
 {
   int n = sscanf( geometry, "%i,%i,%i,%i,%i,%i", region, region+1, region+2, region+3, region+4, region+5);
   if( n != 6 ) return false;
@@ -83,7 +84,7 @@ bool readgeometry(const char *geometry, unsigned int * region)
 }
 
 template <typename T>
-void FillRegionWithColor(T *p, const unsigned int *dims, const unsigned int * region, unsigned int color, unsigned int nsamples)
+static void FillRegionWithColor(T *p, const unsigned int *dims, const unsigned int * region, unsigned int color, unsigned int nsamples)
 {
     unsigned int xmin = region[0];
     unsigned int xmax = region[1];
@@ -107,14 +108,14 @@ void FillRegionWithColor(T *p, const unsigned int *dims, const unsigned int * re
       }
 }
 
-void PrintVersion()
+static void PrintVersion()
 {
   std::cout << "gdcmimg: gdcm " << gdcm::Version::GetVersion() << " ";
   const char date[] = "$Date$";
   std::cout << date << std::endl;
 }
 
-void PrintHelp()
+static void PrintHelp()
 {
   PrintVersion();
   std::cout << "Usage: gdcmimg [OPTION]... FILE..." << std::endl;
@@ -128,11 +129,17 @@ void PrintHelp()
   std::cout << "     --endian %s       Endianness (LSB/MSB)." << std::endl;
   std::cout << "  -d --depth %d        Depth (Either 8/16/32 or BitsAllocated eg. 12 when known)." << std::endl;
   std::cout << "     --sign %s         Pixel sign (0/1)." << std::endl;
-  std::cout << "     --spp  %d         Sample Per Pixel (1/3)." << std::endl;
-  std::cout << "  -s --size %d,%d      Size." << std::endl;
+  std::cout << "     --spp  %d         Sample Per Pixel (1/3/4)." << std::endl;
+  std::cout << "     --pc [01]         Change planar configuration." << std::endl;
+  std::cout << "     --pi [str]        Change photometric interpretation." << std::endl;
+  std::cout << "     --pf %d,%d,%d     Change pixel format: (BA,BS,HB)." << std::endl;
+  std::cout << "  -s --size %d,%d,%d   Size." << std::endl;
+  std::cout << "     --offset %ull     Start Offset." << std::endl;
   std::cout << "  -C --sop-class-uid   SOP Class UID (name or value)." << std::endl;
   std::cout << "  -T --study-uid       Study UID." << std::endl;
   std::cout << "  -S --series-uid      Series UID." << std::endl;
+  std::cout << "     --template        DICOM template." << std::endl;
+  std::cout << "     --keep-meta       Keep meta info from template file (advanced users only)." << std::endl;
   std::cout << "     --root-uid        Root UID." << std::endl;
   std::cout << "Fill Options:" << std::endl;
   std::cout << "  -R --region %d,%d    Region." << std::endl;
@@ -154,7 +161,7 @@ void PrintHelp()
  */
 }
 
-bool AddContentDateTime(gdcm::DataSet &ds, const char *filename )
+static bool AddContentDateTime(gdcm::DataSet &ds, const char *filename )
 {
   time_t studydatetime = gdcm::System::FileTime( filename );
   char date[22];
@@ -179,37 +186,40 @@ bool AddContentDateTime(gdcm::DataSet &ds, const char *filename )
   return true;
 }
 // Set Study Date/Time to the file time:
-bool AddStudyDateTime(gdcm::DataSet &ds, const char *filename )
+static bool AddStudyDateTime(gdcm::DataSet &ds, const char *filename )
 {
   // StudyDate
   char date[22];
   const size_t datelen = 8;
   int res = gdcm::System::GetCurrentDateTime(date);
   if( !res ) return false;
-    {
-    gdcm::DataElement de( gdcm::Tag(0x0008,0x0020) );
-    // Do not copy the whole cstring:
-    de.SetByteValue( date, datelen );
-    de.SetVR( gdcm::Attribute<0x0008,0x0020>::GetVR() );
-    ds.Insert( de );
-    }
+
+  {
+  gdcm::DataElement de( gdcm::Tag(0x0008,0x0020) );
+  // Do not copy the whole cstring:
+  de.SetByteValue( date, datelen );
+  de.SetVR( gdcm::Attribute<0x0008,0x0020>::GetVR() );
+  ds.Insert( de );
+  }
+
   // StudyTime
   const size_t timelen = 6; // get rid of milliseconds
-    {
-    gdcm::DataElement de( gdcm::Tag(0x0008,0x0030) );
-    // Do not copy the whole cstring:
-    de.SetByteValue( date+datelen, timelen );
-    de.SetVR( gdcm::Attribute<0x0008,0x0030>::GetVR() );
-    ds.Insert( de );
-    }
+  {
+  gdcm::DataElement de( gdcm::Tag(0x0008,0x0030) );
+  // Do not copy the whole cstring:
+  de.SetByteValue( date+datelen, timelen );
+  de.SetVR( gdcm::Attribute<0x0008,0x0030>::GetVR() );
+  ds.Insert( de );
+  }
   return AddContentDateTime(ds, filename);
 }
 
 
-bool AddUIDs(int sopclassuid, std::string const & sopclass, std::string const & study_uid, std::string const & series_uid, gdcm::PixmapWriter& writer)
+static bool AddUIDs(int sopclassuid, std::string const & sopclass, std::string const & study_uid, std::string const & series_uid, gdcm::PixmapWriter& writer)
 {
   gdcm::DataSet & ds = writer.GetFile().GetDataSet();
   gdcm::MediaStorage ms = gdcm::MediaStorage::MS_END;
+  gdcm::Pixmap &image = writer.GetPixmap();
   if( sopclassuid )
     {
     // Is it by value or by name ?
@@ -221,35 +231,36 @@ bool AddUIDs(int sopclassuid, std::string const & sopclass, std::string const &
       {
       std::cerr << "not implemented" << std::endl;
       }
-    if( !gdcm::MediaStorage::IsImage(ms) )
-      {
-      std::cerr << "invalid media storage (no pixel data): " << sopclass << std::endl;
-      return false;
-      }
-
-    const char* msstr = gdcm::MediaStorage::GetMSString(ms);
-    if( !msstr )
-      {
-      std::cerr << "problem with media storage: " << sopclass << std::endl;
-      return false;
-      }
-    gdcm::DataElement de( gdcm::Tag(0x0008, 0x0016 ) );
-    de.SetByteValue( msstr, (uint32_t)strlen(msstr) );
-    de.SetVR( gdcm::Attribute<0x0008, 0x0016>::GetVR() );
-    ds.Insert( de );
     }
   else
-    {
-    // FIXME we are copying the default behavior of gdcm.PixmapWriter here:
-    ms = gdcm::MediaStorage::SecondaryCaptureImageStorage;
+    { // guess a default
+    ms = gdcm::ImageHelper::ComputeMediaStorageFromModality(
+      "OT", image.GetNumberOfDimensions(),
+      image.GetPixelFormat(), image.GetPhotometricInterpretation() );
     }
 
-  gdcm::Pixmap &image = writer.GetPixmap();
+  if( !gdcm::MediaStorage::IsImage(ms) )
+    {
+    std::cerr << "invalid media storage (no pixel data): " << sopclass << std::endl;
+    return false;
+    }
   if( ms.GetModalityDimension() < image.GetNumberOfDimensions() )
     {
     std::cerr << "Could not find Modality" << std::endl;
     return false;
     }
+  const char* msstr = gdcm::MediaStorage::GetMSString(ms);
+  if( !msstr )
+    {
+    std::cerr << "problem with media storage: " << sopclass << std::endl;
+    return false;
+    }
+  {
+    gdcm::DataElement de( gdcm::Tag(0x0008, 0x0016 ) );
+    de.SetByteValue( msstr, (uint32_t)strlen(msstr) );
+    de.SetVR( gdcm::Attribute<0x0008, 0x0016>::GetVR() );
+    ds.Insert( de );
+  }
 
     {
     gdcm::DataElement de( gdcm::Tag(0x0020,0x000d) ); // Study
@@ -268,7 +279,10 @@ bool AddUIDs(int sopclassuid, std::string const & sopclass, std::string const &
   return true;
 }
 
-bool PopulateSingeFile( gdcm::PixmapWriter & writer, gdcm::SequenceOfFragments *sq , gdcm::ImageCodec & jpeg, const char *filename )
+// Append data to either sq or bv depending whether encapsulated or not
+static bool PopulateSingeFile( gdcm::PixmapWriter & writer,
+  gdcm::SequenceOfFragments *sq , gdcm::ByteValue * bv, gdcm::ImageCodec & jpeg,
+  const char *filename, std::streampos const pos = 0 )
 {
   /*
    * FIXME: when JPEG contains JFIF marker, we should only read them
@@ -288,6 +302,7 @@ bool PopulateSingeFile( gdcm::PixmapWriter & writer, gdcm::SequenceOfFragments *
   image.SetDimensions( jpeg.GetDimensions() );
   image.SetPixelFormat( jpeg.GetPixelFormat() );
   image.SetPhotometricInterpretation( jpeg.GetPhotometricInterpretation() );
+  image.SetPlanarConfiguration( jpeg.GetPlanarConfiguration() );
   image.SetTransferSyntax( ts );
 
   AddStudyDateTime( writer.GetFile().GetDataSet(), filename );
@@ -303,6 +318,10 @@ bool PopulateSingeFile( gdcm::PixmapWriter & writer, gdcm::SequenceOfFragments *
     // do not rewind file should be just at right offset
     }
   char *buf = new char[len];
+  if( pos )
+    {
+    is.seekg( pos, std::ios::beg );
+    }
   is.read(buf, len);
   gdcm::DataElement pixeldata( gdcm::Tag(0x7fe0,0x0010) );
 
@@ -315,32 +334,32 @@ bool PopulateSingeFile( gdcm::PixmapWriter & writer, gdcm::SequenceOfFragments *
     }
   else
     {
-    pixeldata.SetByteValue( buf, (uint32_t)len );
+    gdcm::ByteValue frame( buf, (uint32_t)len );
+    bv->Append( frame );
+    pixeldata.SetValue( *bv );
     }
-    delete[] buf;
+  delete[] buf;
   image.SetDataElement( pixeldata );
 
   return true;
 }
 
-bool Populate( gdcm::PixmapWriter & writer, gdcm::ImageCodec & jpeg, gdcm::Directory::FilenamesType const & filenames )
+static bool Populate( gdcm::PixmapWriter & writer, gdcm::ImageCodec & jpeg, gdcm::Directory::FilenamesType const & filenames, unsigned int ndim = 2, std::streampos const & pos = 0 )
 {
   std::vector<std::string>::const_iterator it = filenames.begin();
   bool b = true;
   gdcm::Pixmap &image = writer.GetPixmap();
-  image.SetNumberOfDimensions( 2 );
+  image.SetNumberOfDimensions( ndim );
+
+  gdcm::SmartPointer<gdcm::SequenceOfFragments> sq = new gdcm::SequenceOfFragments;
+  gdcm::SmartPointer<gdcm::ByteValue> bv = new gdcm::ByteValue;
+  for(; it != filenames.end(); ++it)
+    {
+    b = b && PopulateSingeFile( writer, sq, bv, jpeg, it->c_str(), pos );
+    }
   if( filenames.size() > 1 )
     {
     image.SetNumberOfDimensions( 3 );
-    }
-
-  gdcm::SmartPointer<gdcm::SequenceOfFragments> sq = new gdcm::SequenceOfFragments;
-  for(; it != filenames.end(); ++it)
-    {
-    b = b && PopulateSingeFile( writer, sq, jpeg, it->c_str() );
-    }
-  if( filenames.size() > 1 )
-    {
     image.SetDimension(2,  (unsigned int)filenames.size() );
     }
 
@@ -348,7 +367,7 @@ bool Populate( gdcm::PixmapWriter & writer, gdcm::ImageCodec & jpeg, gdcm::Direc
 }
 
 
-bool GetPixelFormat( gdcm::PixelFormat & pf, int depth, int bpp, int sign, int pixelsign, int spp = 0, int pixelspp = 1 )
+static bool GetPixelFormat( gdcm::PixelFormat & pf, int depth, int bpp, int sign, int pixelsign, int spp = 0, int pixelspp = 1 )
 {
   if( depth )
     {
@@ -369,7 +388,7 @@ bool GetPixelFormat( gdcm::PixelFormat & pf, int depth, int bpp, int sign, int p
       std::cerr << "Invalid depth: << " << bpp << std::endl;
       return false;
       }
-    pf.SetBitsStored( bpp );
+    pf.SetBitsStored( (short)bpp );
     }
   if( sign )
     {
@@ -393,15 +412,20 @@ int main (int argc, char *argv[])
   gdcm::Filename filename;
   gdcm::Directory::FilenamesType filenames;
   gdcm::Filename outfilename;
+  gdcm::Filename templatefilename;
   unsigned int region[6] = {}; // Rows & Columns are VR=US anyway...
   unsigned int color = 0;
   int bregion = 0;
   int fill = 0;
   int sign = 0;
   int spp = 0;
+  int pconf = 0; // planar configuration
   int studyuid = 0;
   int seriesuid = 0;
-  unsigned int size[2] = {};
+  int templated = 0;
+  int keepmeta = 0;
+  unsigned int size[3] = {0,0,0};
+  unsigned int ndimension = 2;
   int depth = 0;
   int endian = 0;
   int bpp = 0;
@@ -410,6 +434,12 @@ int main (int argc, char *argv[])
   std::string sopclass;
   std::string lsb_msb;
   int sopclassuid = 0;
+  int pinter = 0;
+  std::string pinterstr;
+  int pformat = 0;
+  std::string pformatstr;
+  int poffset = 0;
+  size_t start_pos = 0;
 
   int verbose = 0;
   int warning = 0;
@@ -422,15 +452,15 @@ int main (int argc, char *argv[])
   // Too early for UID Generation
   std::string series_uid; // = uid.Generate();
   std::string study_uid; // = uid.Generate();
-  while (1) {
+  while (true) {
     //int this_option_optind = optind ? optind : 1;
     int option_index = 0;
     static struct option long_options[] = {
-        {"input", 1, 0, 0},
-        {"output", 1, 0, 0},
+        {"input", 1, nullptr, 0},
+        {"output", 1, nullptr, 0},
         // provide convert-like command line args:
         {"depth", 1, &depth, 1},
-        {"size", 1, 0, 0},
+        {"size", 1, nullptr, 0},
         {"region", 1, &bregion, 1},
         {"fill", 1, &fill, 1},
         {"study-uid", 1, &studyuid, 1},
@@ -440,6 +470,12 @@ int main (int argc, char *argv[])
         {"endian", 1, &endian, 1}, //
         {"sign", 1, &sign, 1}, //
         {"spp", 1, &spp, 1}, //
+        {"pc", 1, &pconf, 1}, //
+        {"pi", 1, &pinter, 1}, //
+        {"pf", 1, &pformat, 1}, //
+        {"offset", 1, &poffset, 1}, //
+        {"template", 1, &templated, 1},
+        {"keep-meta", 0, &keepmeta, 1}, // by default we do not want to keep
 
 // General options !
         {"verbose", 0, &verbose, 1},
@@ -448,7 +484,7 @@ int main (int argc, char *argv[])
         {"error", 0, &error, 1},
         {"help", 0, &help, 1},
         {"version", 0, &version, 1},
-        {0, 0, 0, 0}
+        {nullptr, 0, nullptr, 0}
     };
 
     // i -> input file
@@ -466,7 +502,7 @@ int main (int argc, char *argv[])
       {
     case 0:
         {
-        const char *s = long_options[option_index].name;
+        const char *s = long_options[option_index].name; (void)s;
         //printf ("option %s", s);
         if (optarg)
           {
@@ -484,7 +520,7 @@ int main (int argc, char *argv[])
           else if( option_index == 3 ) /* size */
             {
             assert( strcmp(s, "size") == 0 );
-            readsize(optarg, size);
+            ndimension = readsize(optarg, size);
             }
           else if( option_index == 4 ) /* region */
             {
@@ -531,6 +567,35 @@ int main (int argc, char *argv[])
             assert( strcmp(s, "spp") == 0 );
             pixelspp = atoi(optarg);
             }
+          else if( option_index == 13 ) /* pconf */
+            {
+            assert( strcmp(s, "pc") == 0 );
+            pconf = atoi(optarg);
+            }
+          else if( option_index == 14 ) /* pinter */
+            {
+            assert( strcmp(s, "pi") == 0 );
+            pinter = 1;
+            pinterstr = optarg;
+            }
+          else if( option_index == 15 ) /* pformat */
+            {
+            assert( strcmp(s, "pf") == 0 );
+            pformat = 1;
+            pformatstr = optarg;
+            }
+          else if( option_index == 16 ) /* start_pos */
+            {
+            assert( strcmp(s, "offset") == 0 );
+            poffset = 1;
+            start_pos = (size_t)std::atoll(optarg);
+            }
+          else if( option_index == 17 ) /* template */
+            {
+            assert( strcmp(s, "template") == 0 );
+            templated = 1;
+            templatefilename = optarg;
+            }
           //printf (" with arg %s", optarg);
           }
         //printf ("\n");
@@ -555,7 +620,7 @@ int main (int argc, char *argv[])
       break;
 
     case 's': // size
-      readsize(optarg, size);
+      ndimension = readsize(optarg, size);
       break;
 
     case 'T':
@@ -623,7 +688,7 @@ int main (int argc, char *argv[])
     while (optind < argc)
       {
       //printf ("%s\n", argv[optind++]);
-      files.push_back( argv[optind++] );
+      files.emplace_back(argv[optind++] );
       }
     //printf ("\n");
     if( files.size() >= 2
@@ -737,7 +802,37 @@ int main (int argc, char *argv[])
     }
   if( spp )
     {
-    if( pixelspp != 1 && pixelspp != 3 ) return 1;
+    if( pixelspp != 1 && pixelspp != 3 && pixelspp != 4 ) return 1;
+    }
+  if( pconf != 0 && pconf != 1 ) return 1;
+  if( pconf )
+    {
+    if( pixelspp != 3 ) return 1;
+    }
+  gdcm::PixelFormat pfref = gdcm::PixelFormat::UINT8;
+  if( pformat )
+    {
+    int ba, bs, hb;
+    int n = sscanf( pformatstr.c_str(), "%d,%d,%d", &ba, &bs, &hb );
+    if( n != 3 ) return 1;
+    pfref.SetBitsAllocated( (unsigned short)ba );
+    pfref.SetBitsStored( (unsigned short)bs );
+    pfref.SetHighBit( (unsigned short)hb );
+    if( spp )
+      pfref.SetSamplesPerPixel( (unsigned short)pixelspp );
+    if( sign )
+      pfref.SetPixelRepresentation( (unsigned short)pixelsign );
+    }
+  gdcm::PhotometricInterpretation::PIType refpi = gdcm::PhotometricInterpretation::MONOCHROME2;
+  if( pinter )
+    {
+    refpi = gdcm::PhotometricInterpretation::GetPIType( pinterstr.c_str() );
+    if( refpi == gdcm::PhotometricInterpretation::UNKNOWN
+      || refpi == gdcm::PhotometricInterpretation::PI_END )
+      {
+      std::cerr << "Invalid PI: " << pinterstr << std::endl;
+      return 1;
+      }
     }
 
   const char *inputextension = filename.GetExtension();
@@ -745,10 +840,22 @@ int main (int argc, char *argv[])
   //if( !inputextension || !outputextension ) return 1;
   if( inputextension )
     {
-    if(  gdcm::System::StrCaseCmp(inputextension,".raw") == 0
-      || gdcm::System::StrCaseCmp(inputextension,".rawl") == 0 // kakadu convention for raw little endian
-      || gdcm::System::StrCaseCmp(inputextension,".gray") == 0
-      || gdcm::System::StrCaseCmp(inputextension,".rgb") == 0 )
+    gdcm::Reader reader;
+    if( templated )
+      {
+      reader.SetFileName( templatefilename );
+      if( !reader.Read() )
+        {
+        std::cerr << "Failed to read: " << templatefilename << std::endl;
+        return 1;
+        }
+      }
+    if(  gdcm::System::StrCaseCmp(inputextension,".raw") == 0   // watch out that .raw for kakadu means big-endian
+      || gdcm::System::StrCaseCmp(inputextension,".rawl") == 0  // kakadu convention for raw little endian
+      || gdcm::System::StrCaseCmp(inputextension,".gray") == 0  // imagemagick convention
+      || gdcm::System::StrCaseCmp(inputextension,".bin") == 0   // openjp3d convention for raw little endian
+      || gdcm::System::StrCaseCmp(inputextension,".rgb") == 0   // imagemagick convention
+      || gdcm::System::StrCaseCmp(inputextension,".yuv") == 0 ) // ffmpeg convention
       {
       if( !size[0] || !size[1] )
         {
@@ -757,14 +864,20 @@ int main (int argc, char *argv[])
         }
       gdcm::RAWCodec raw;
       gdcm::PixmapWriter writer;
+      writer.SetCheckFileMetaInformation( (keepmeta > 0 ? false : true) );
+      writer.SetFile( reader.GetFile() );
       // Because the RAW stream is not self sufficient, we need to pass in some extra
       // user info:
       unsigned int dims[3] = {};
       dims[0] = size[0];
       dims[1] = size[1];
+      if( ndimension == 3 )
+        {
+        dims[2] = size[2];
+        }
       raw.SetDimensions( dims );
       gdcm::PixelFormat pf = gdcm::PixelFormat::UINT8;
-      gdcm::PhotometricInterpretation pi = gdcm::PhotometricInterpretation::MONOCHROME2;
+      gdcm::PhotometricInterpretation pi = refpi;
       if( gdcm::System::StrCaseCmp(inputextension,".rgb") == 0 )
         {
         pi = gdcm::PhotometricInterpretation::RGB;
@@ -775,10 +888,11 @@ int main (int argc, char *argv[])
       raw.SetPixelFormat( pf );
       if( spp )
         {
-        if( pixelspp == 3 ) pi = gdcm::PhotometricInterpretation::RGB;
+        if( pixelspp == 3 && !pinter ) pi = gdcm::PhotometricInterpretation::RGB;
         }
       raw.SetPhotometricInterpretation( pi );
       raw.SetNeedByteSwap( false );
+      raw.SetPlanarConfiguration( pconf );
       if( endian )
         {
         if( lsb_msb == "LSB" || lsb_msb == "MSB" )
@@ -795,7 +909,7 @@ int main (int argc, char *argv[])
           }
         }
 
-      if( !Populate( writer, raw, filenames ) ) return 1;
+      if( !Populate( writer, raw, filenames, ndimension, start_pos ) ) return 1;
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer ) ) return 1;
 
       writer.SetFileName( outfilename );
@@ -817,6 +931,8 @@ int main (int argc, char *argv[])
         }
       gdcm::RLECodec rle;
       gdcm::PixmapWriter writer;
+      writer.SetCheckFileMetaInformation( (keepmeta > 0 ? false : true) );
+      writer.SetFile( reader.GetFile() );
       // Because the RLE stream is not self sufficient, we need to pass in some extra
       // user info:
       unsigned int dims[3] = {};
@@ -824,9 +940,13 @@ int main (int argc, char *argv[])
       dims[1] = size[1];
       rle.SetDimensions( dims );
       gdcm::PixelFormat pf = gdcm::PixelFormat::UINT8;
-      if( !GetPixelFormat( pf, depth, bpp, sign, pixelsign ) ) return 1;
+      if( !GetPixelFormat( pf, depth, bpp, sign, pixelsign, spp, pixelspp ) ) return 1;
       rle.SetPixelFormat( pf );
-      gdcm::PhotometricInterpretation pi = gdcm::PhotometricInterpretation::MONOCHROME2;
+      gdcm::PhotometricInterpretation pi = refpi;
+      if( spp )
+        {
+        if( pixelspp == 3 && !pinter ) pi = gdcm::PhotometricInterpretation::RGB;
+        }
       rle.SetPhotometricInterpretation( pi );
 
       if( !Populate( writer, rle, filenames ) ) return 1;
@@ -843,11 +963,32 @@ int main (int argc, char *argv[])
 
     if(  gdcm::System::StrCaseCmp(inputextension,".pgm") == 0
       || gdcm::System::StrCaseCmp(inputextension,".pnm") == 0
+      || gdcm::System::StrCaseCmp(inputextension,".pbm") == 0
       || gdcm::System::StrCaseCmp(inputextension,".ppm") == 0 )
       {
       gdcm::PNMCodec pnm;
+      // Let's handle the case where user really wants to specify the data:
+      gdcm::PixelFormat pf = gdcm::PixelFormat::UINT8;
+      if( !GetPixelFormat( pf, depth, bpp, sign, pixelsign ) ) return 1;
+      pnm.SetPixelFormat( pf );
       gdcm::PixmapWriter writer;
+      writer.SetCheckFileMetaInformation( (keepmeta > 0 ? false : true) );
+      writer.SetFile( reader.GetFile() );
       if( !Populate( writer, pnm, filenames ) ) return 1;
+      // populate will guess pixel format and photometric inter from file, need
+      // to override after calling Populate:
+      if( pformat )
+        {
+        writer.GetPixmap().SetPixelFormat( pfref );
+        }
+      if( pinter )
+        {
+        writer.GetPixmap().SetPhotometricInterpretation( refpi );
+        }
+      // HACK
+      if( endian && lsb_msb == "LSB" )
+        writer.GetPixmap().SetTransferSyntax( gdcm::TransferSyntax::ImplicitVRLittleEndian );
+
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer ) ) return 1;
 
       writer.SetFileName( outfilename );
@@ -863,6 +1004,8 @@ int main (int argc, char *argv[])
       {
       gdcm::PGXCodec pnm;
       gdcm::PixmapWriter writer;
+      writer.SetCheckFileMetaInformation( (keepmeta > 0 ? false : true) );
+      writer.SetFile( reader.GetFile() );
       if( !Populate( writer, pnm, filenames ) ) return 1;
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer ) ) return 1;
 
@@ -879,6 +1022,8 @@ int main (int argc, char *argv[])
       {
       gdcm::JPEGLSCodec jpeg;
       gdcm::PixmapWriter writer;
+      writer.SetCheckFileMetaInformation( (keepmeta > 0 ? false : true) );
+      writer.SetFile( reader.GetFile() );
       if( !Populate( writer, jpeg, filenames ) ) return 1;
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer ) ) return 1;
 
@@ -895,6 +1040,7 @@ int main (int argc, char *argv[])
       || gdcm::System::StrCaseCmp(inputextension,".j2k") == 0
       || gdcm::System::StrCaseCmp(inputextension,".j2c") == 0
       || gdcm::System::StrCaseCmp(inputextension,".jpx") == 0
+      || gdcm::System::StrCaseCmp(inputextension,".jpf") == 0
       || gdcm::System::StrCaseCmp(inputextension,".jpc") == 0 )
       {
       /*
@@ -903,7 +1049,13 @@ int main (int argc, char *argv[])
        */
       gdcm::JPEG2000Codec jpeg;
       gdcm::PixmapWriter writer;
+      writer.SetCheckFileMetaInformation( (keepmeta > 0 ? false : true) );
+      writer.SetFile( reader.GetFile() );
       if( !Populate( writer, jpeg, filenames ) ) return 1;
+      if( pinter )
+        {
+        writer.GetPixmap().SetPhotometricInterpretation( refpi );
+        }
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer ) ) return 1;
 
       writer.SetFileName( outfilename );
@@ -927,6 +1079,8 @@ int main (int argc, char *argv[])
       if( !GetPixelFormat( pf, depth, bpp, sign, pixelsign ) ) return 1;
       jpeg.SetPixelFormat( pf );
       gdcm::PixmapWriter writer;
+      writer.SetCheckFileMetaInformation( (keepmeta > 0 ? false : true) );
+      writer.SetFile( reader.GetFile() );
       if( !Populate( writer, jpeg, filenames ) ) return 1;
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer ) ) return 1;
 
@@ -957,12 +1111,14 @@ int main (int argc, char *argv[])
     {
     if(  gdcm::System::StrCaseCmp(outputextension,".pgm") == 0
       || gdcm::System::StrCaseCmp(outputextension,".pnm") == 0
+      || gdcm::System::StrCaseCmp(outputextension,".pbm") == 0
       || gdcm::System::StrCaseCmp(outputextension,".ppm") == 0 )
       {
       gdcm::PNMCodec pnm;
       pnm.SetDimensions( imageori.GetDimensions() );
       pnm.SetPixelFormat( imageori.GetPixelFormat() );
       pnm.SetPhotometricInterpretation( imageori.GetPhotometricInterpretation() );
+      pnm.SetPlanarConfiguration( imageori.GetPlanarConfiguration() );
       pnm.SetLUT( imageori.GetLUT() );
       const gdcm::DataElement& in = imageori.GetDataElement();
       bool b = pnm.Write( outfilename, in );
@@ -980,6 +1136,7 @@ int main (int argc, char *argv[])
       pnm.SetDimensions( imageori.GetDimensions() );
       pnm.SetPixelFormat( imageori.GetPixelFormat() );
       pnm.SetPhotometricInterpretation( imageori.GetPhotometricInterpretation() );
+      pnm.SetPlanarConfiguration( imageori.GetPlanarConfiguration() );
       pnm.SetLUT( imageori.GetLUT() );
       const gdcm::DataElement& in = imageori.GetDataElement();
       bool b = pnm.Write( outfilename, in );
