@@ -20,11 +20,13 @@
  */
 
 #include "gdcmReader.h"
+#include "gdcmCSAHeader.h"
 #include "gdcmVersion.h"
 #include "gdcmImageReader.h"
 #include "gdcmDataElement.h"
 #include "gdcmImageWriter.h"
 #include "gdcmSplitMosaicFilter.h"
+#include "gdcmFilename.h"
 #include "gdcmFilenameGenerator.h"
 #include "gdcmDirectionCosines.h"
 #include "gdcmImageHelper.h"
@@ -35,6 +37,9 @@
 #include "gdcmScanner.h"
 #include "gdcmIPPSorter.h"
 #include "gdcmAttribute.h"
+#include "gdcmAnonymizer.h"
+#include "gdcmTagKeywords.h"
+#include "gdcmMrProtocol.h"
 
 #include <string>
 #include <iostream>
@@ -44,14 +49,14 @@
 #include <getopt.h>
 #include <string.h>
 
-void PrintVersion()
+static void PrintVersion()
 {
   std::cout << "gdcmtar: gdcm " << gdcm::Version::GetVersion() << " ";
   const char date[] = "$Date$";
   std::cout << date << std::endl;
 }
 
-void PrintHelp()
+static void PrintHelp()
 {
   PrintVersion();
   std::cout << "Usage: gdcmtar [OPTION] [FILE]" << std::endl;
@@ -60,11 +65,12 @@ void PrintHelp()
   std::cout << "  -i --input     DICOM filename" << std::endl;
   std::cout << "  -o --output    DICOM filename" << std::endl;
   std::cout << "Options:" << std::endl;
-  std::cout << "     --enhance    enhance (default)" << std::endl;
-  std::cout << "  -U --unenhance  unenhance" << std::endl;
-  std::cout << "  -M --mosaic     Split SIEMENS Mosaic image into multiple frames." << std::endl;
-  std::cout << "  -p --pattern    Specify trailing file pattern." << std::endl;
-  std::cout << "     --root-uid        Root UID." << std::endl;
+  std::cout << "     --enhance        Enhance (default)" << std::endl;
+  std::cout << "  -U --unenhance      Unenhance" << std::endl;
+  std::cout << "  -M --mosaic         Split SIEMENS Mosaic image into multiple frames." << std::endl;
+  std::cout << "     --mosaic-private When splitting SIEMENS Mosaic image into multiple frames, preserve private attributes (advanced user only)." << std::endl;
+  std::cout << "  -p --pattern        Specify trailing file pattern." << std::endl;
+  std::cout << "     --root-uid       Root UID." << std::endl;
   //std::cout << "     --resources-path     Resources path." << std::endl;
   std::cout << "General Options:" << std::endl;
   std::cout << "  -V --verbose    more verbose (warning+error)." << std::endl;
@@ -94,6 +100,7 @@ namespace gdcm {
   const Tag T2(0x0020,0x000e); // Series Instance UID
   const Tag T3(0x0020,0x0052); // Frame of Reference UID
   const Tag T4(0x0020,0x0037); // Image Orientation (Patient)
+  const Tag T5(0x0008,0x0008); // Image Type
 
 class DiscriminateVolume
 {
@@ -123,34 +130,120 @@ private:
   return theReturn;
 }
 
-void ProcessAIOP(Scanner const & , Directory::FilenamesType const & subset, const char *iopval)
+class frame_diff /* */
+{
+public:
+  bool operator<(const frame_diff &rhs) const
+    {
+    (void)rhs;
+    return true;
+    }
+};
+
+static frame_diff get_frame_diff( const char* filename )
+{
+  frame_diff ret;
+  gdcm::Reader reader;
+  reader.SetFileName( filename );
+
+  gdcm::Tag dummy(0x7fe0,0x0000);
+  std::set<gdcm::Tag> skiptags;
+  skiptags.insert( dummy );
+  if ( !reader.ReadUpToTag( dummy, skiptags) )
+    {
+    assert(0);
+    }
+
+  gdcm::CSAHeader csa;
+  const gdcm::DataSet& ds = reader.GetFile().GetDataSet();
+
+  const gdcm::PrivateTag &t1 = csa.GetCSAImageHeaderInfoTag();
+  //const gdcm::PrivateTag &t2 = csa.GetCSASeriesHeaderInfoTag();
+  //const gdcm::PrivateTag &t3 = csa.GetCSADataInfo();
+
+  //bool found = false;
+  if( ds.FindDataElement( t1 ) )
+    {
+    csa.LoadFromDataElement( ds.GetDataElement( t1 ) );
+    assert( csa.GetFormat() == gdcm::CSAHeader::SV10
+      || csa.GetFormat() == gdcm::CSAHeader::NOMAGIC );
+    // SIEMENS / Diffusion
+    const CSAElement &bvalue  = csa.GetCSAElementByName( "B_value" );
+    (void)bvalue;
+    const CSAElement &bmatrix = csa.GetCSAElementByName( "B_matrix" );
+    (void)bmatrix;
+    const CSAElement &dgd = csa.GetCSAElementByName( "DiffusionGradientDirection" );
+    (void)dgd;
+    //assert(0);
+    }
+  else
+    {
+    assert(0);
+    }
+
+  return ret;
+}
+
+void ProcessAIOP(Scanner const & s, Directory::FilenamesType const & subset, const char *iopval)
 {
   if( debuggdcmtar )
-  std::cout << "IOP: " << iopval << std::endl;
-  IPPSorter ipp;
-  ipp.SetComputeZSpacing( true );
-  ipp.SetZSpacingTolerance( 1e-3 ); // ??
-  bool b = ipp.Sort( subset );
-  if( !b )
+    std::cout << "IOP: " << iopval << std::endl;
+
+  bool is_diffusion = false;
+  gdcm::Scanner::ValuesType imagetypes = s.GetValues(gdcm::T5);
+  if( imagetypes.size() == 1 )
     {
-    // If you reach here this means you need one more parameter to discriminiat this
-    // series. Eg. T1 / T2 intertwinted. Multiple Echo (0018,0081)
-    if( debuggdcmtar )
+    const std::string & imagetype = *imagetypes.begin();
+    //gdcm::Attribute<0x0008, 0x0008> at;
+    //at.SetValue( imagetype );
+    std::string::size_type pos = imagetype.find( "DIFFUSION" );
+    is_diffusion = (pos != std::string::npos);
+    }
+
+  if( is_diffusion )
+    {
+    std::multimap<frame_diff, std::string> mm;
+    for( Directory::FilenamesType::const_iterator file = subset.begin();
+      file != subset.end(); ++file)
+      {
+      //std::cerr << *file << std::endl;
+      mm.insert( std::make_pair(get_frame_diff( file->c_str() ), file->c_str()) );
+      }
+    //assert(0);
+    for( std::multimap<frame_diff, std::string>::const_iterator it = mm.begin();
+      it != mm.end(); ++it )
+      {
+      std::cerr << it->second << std::endl;
+      }
+      SortedFiles.push_back( subset );
+    }
+  else
+    {
+    IPPSorter ipp;
+    ipp.SetComputeZSpacing( true );
+    ipp.SetZSpacingTolerance( 1e-3 ); // ??
+    bool b = ipp.Sort( subset );
+    if( !b )
       {
-      std::cerr << "Failed to sort: " << subset.begin()->c_str() << std::endl;
-      for(
-        Directory::FilenamesType::const_iterator file = subset.begin();
-        file != subset.end(); ++file)
+      // If you reach here this means you need one more parameter to discriminate this
+      // series. Eg. T1 / T2 intertwined. Multiple Echo (0018,0081)
+      if( debuggdcmtar )
         {
-        std::cerr << *file << std::endl;
+        std::cerr << "Failed to sort: " << subset.begin()->c_str() << std::endl;
+        for(
+          Directory::FilenamesType::const_iterator file = subset.begin();
+          file != subset.end(); ++file)
+          {
+          std::cerr << *file << std::endl;
+          }
         }
+      UnsortedFiles.push_back( subset );
+      return ;
       }
-    UnsortedFiles.push_back( subset );
-    return ;
+    if( debuggdcmtar )
+      ipp.Print( std::cout );
+    SortedFiles.push_back( ipp.GetFilenames() );
     }
-  if( debuggdcmtar )
-  ipp.Print( std::cout );
-  SortedFiles.push_back( ipp.GetFilenames() );
 }
 
 void ProcessAFrameOfRef(Scanner const & s, Directory::FilenamesType const & subset, const char * frameuid)
@@ -179,7 +272,7 @@ void ProcessAFrameOfRef(Scanner const & s, Directory::FilenamesType const & subs
     }
 
   if( debuggdcmtar )
-  std::cout << "Frame of Ref: " << frameuid << std::endl;
+    std::cout << "Frame of Ref: " << frameuid << std::endl;
   if ( n == 1 )
     {
     ProcessAIOP(s, files, iopset.begin()->c_str() );
@@ -188,7 +281,7 @@ void ProcessAFrameOfRef(Scanner const & s, Directory::FilenamesType const & subs
     {
     const char *f = files.begin()->c_str();
     if( debuggdcmtar )
-    std::cerr << "More than one IOP: " << f << std::endl;
+      std::cerr << "More than one IOP: " << f << std::endl;
     // Make sure that there is actually 'n' different IOP
     gdcm::DirectionCosines ref;
     gdcm::DirectionCosines dc;
@@ -232,7 +325,7 @@ void ProcessAFrameOfRef(Scanner const & s, Directory::FilenamesType const & subs
 void ProcessASeries(Scanner const & s, const char * seriesuid)
 {
   if( debuggdcmtar )
-  std::cout << "Series: " << seriesuid << std::endl;
+    std::cout << "Series: " << seriesuid << std::endl;
   // let's find all files belonging to this series:
   Directory::FilenamesType seriesfiles = GetAllFilenamesFromTagToValue(
     s, s.GetFilenames(), T2, seriesuid);
@@ -249,8 +342,10 @@ void ProcessASeries(Scanner const & s, const char * seriesuid)
 void ProcessAStudy(Scanner const & s, const char * studyuid)
 {
   if( debuggdcmtar )
-  std::cout << "Study: " << studyuid << std::endl;
+    std::cout << "Study: " << studyuid << std::endl;
   gdcm::Scanner::ValuesType vt2 = s.GetValues(T2);
+  if( vt2.empty() )
+    std::cerr << "No Series Found" << std::endl;
   for(
     gdcm::Scanner::ValuesType::const_iterator it = vt2.begin()
     ; it != vt2.end(); ++it )
@@ -288,7 +383,6 @@ void Print( std::ostream & os )
       os << *file << std::endl;
       }
     }
-
 }
 
   std::vector< Directory::FilenamesType > const & GetSortedFiles() const { return SortedFiles; }
@@ -303,12 +397,11 @@ void ProcessIntoVolume( Scanner const & s )
     {
     ProcessAStudy( s, it->c_str() );
     }
-
 }
 
 };
 
-bool ConcatenateImages(Image &im1, Image const &im2)
+static bool ConcatenateImages(Image &im1, Image const &im2)
 {
   DataElement& de1 = im1.GetDataElement();
   if( de1.GetByteValue() )
@@ -324,7 +417,14 @@ bool ConcatenateImages(Image &im1, Image const &im2)
     }
   else if( de1.GetSequenceOfFragments() )
     {
-    assert( 0 );
+    SequenceOfFragments *sqf1 = de1.GetSequenceOfFragments();
+    assert( sqf1 );
+    const DataElement& de2 = im2.GetDataElement();
+    const SequenceOfFragments *sqf2 = de2.GetSequenceOfFragments();
+    assert( sqf2 );
+    assert( sqf2->GetNumberOfFragments() == 1 );
+    const Fragment& frag = sqf2->GetFragment(0);
+    sqf1->AddFragment(frag);
     }
   else
     {
@@ -337,10 +437,20 @@ bool ConcatenateImages(Image &im1, Image const &im2)
   return true;
 }
 
+static void InsertIfExist( DataSet &enhds, const DataSet &ds, const Tag & t )
+{
+  if( ds.FindDataElement( t ) )
+    {
+    assert( !enhds.FindDataElement( t ) );
+    enhds.Insert( ds.GetDataElement( t ) );
+    }
+}
+
+
 } // namespace gdcm
 
 
-int MakeImageEnhanced( std::string const & filename, std::string const &outfilename )
+static int MakeImageEnhanced( std::string const & filename, std::string const &outfilename )
 {
   if( !gdcm::System::FileIsDirectory(filename.c_str()) )
     {
@@ -357,6 +467,7 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
   s.AddTag( gdcm::T2 );
   s.AddTag( gdcm::T3 );
   s.AddTag( gdcm::T4 );
+  s.AddTag( gdcm::T5 );
   bool b = s.Scan( d.GetFilenames() );
   if( !b )
     {
@@ -369,13 +480,13 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
   if( vt.size() != 1 ) return 1;
 
   const char *sop = vt.begin()->c_str();
-{
-  gdcm::MediaStorage ms = gdcm::MediaStorage::GetMSType( sop );
-  if( ms != gdcm::MediaStorage::MRImageStorage )
+  gdcm::MediaStorage msorig = gdcm::MediaStorage::GetMSType( sop );
+  if( msorig != gdcm::MediaStorage::MRImageStorage
+   && msorig != gdcm::MediaStorage::CTImageStorage )
     {
+    std::cerr << "Sorry MediaStorage not supported: [" << sop << "]" << std::endl;
     return 1;
     }
-}
 
   gdcm::DiscriminateVolume dv;
   dv.ProcessIntoVolume( s );
@@ -412,7 +523,7 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
     // construct the target name:
     std::string targetname = targetdir;
 
-    targetdir += "/old/";
+    targetdir += "/old";
 
     // make sure the dir exist first:
     if( !gdcm::System::MakeDirectory( targetdir.c_str() ) )
@@ -453,6 +564,150 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
     ++file;
     ++count;
 
+    // insert Enhanced MR Image Storage
+    if( false )
+      {
+      gdcm::DataSet & enhds = im0.GetFile().GetDataSet();
+      gdcm::DataSet & ds = reader0.GetFile().GetDataSet();
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0005) ); // SpecificCharacterSet
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0008) ); // ImageType
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0012) ); // InstanceCreationDate
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0013) ); // InstanceCreationTime
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0020) ); // StudyDate
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0021) ); // StudyTime
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0022) ); // AcquisitionDate
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0023) ); // ContentDate
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0030) ); // StudyTime
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0031) ); // SeriesTime
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0032) ); // AcquisitionTime
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0033) ); // ContentTime
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0050) ); // AccessionNumber
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0070) ); // Manufacturer
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0080) ); // InstitutionName
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0081) ); // InstitutionAddress
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x0090) ); // ReferringPhysicianName
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x1010) ); // StationName
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x1030) ); // StudyDescription
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x103e) ); // SeriesDescription
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x1040) ); // InstitutionalDepartme
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x1050) ); // PerformingPhysicianNa
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x8,0x1090) ); // ManufacturerModelName
+
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0010,0x0010)); // PatientName
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0010,0x0020)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0010,0x0030)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0010,0x0040)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0010,0x1010)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0010,0x1030)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0020)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0021)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0022)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0023)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0024)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0025)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0050)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0080)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0081)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0083)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0084)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0085)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0086)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0087)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0088)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0089)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0091)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0093)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0094)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x0095)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1000)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1020)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1030)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1251)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1310)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1312)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1314)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1315)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1316)); //
+      //gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x1318)); //
+      gdcm::InsertIfExist( enhds, ds, gdcm::Tag(0x0018,0x5100)); //
+
+        {
+        gdcm::SmartPointer<gdcm::SequenceOfItems> sqi1 = new gdcm::SequenceOfItems;
+
+/*
+(5200,9229) SQ (Sequence with undefined length)                   # u/l,1 Shared Functional Groups Sequence
+  (fffe,e000) na (Item with undefined length)
+    (0018,9006) SQ (Sequence with undefined length)               # u/l,1 MR Imaging Modifier Sequence
+      (fffe,e000) na (Item with undefined length)
+        (0018,0095) DS [109 ]                                     # 4,1 Pixel Bandwidth
+        (0018,9020) CS [NONE]                                     # 4,1 Magnetization Transfer
+        (0018,9022) CS [NO]                                       # 2,1 Blood Signal Nulling
+        (0018,9028) CS [NONE]                                     # 4,1 Tagging
+        (0018,9098) FD 63.8799                                    # 8,1-2 Transmitter Frequency
+*/
+        gdcm::Item item0;
+        gdcm::DataSet &subds0 = item0.GetNestedDataSet();
+
+          {
+        gdcm::SmartPointer<gdcm::SequenceOfItems> sqi0 = new gdcm::SequenceOfItems;
+          const gdcm::Tag sisq0(0x0018,0x9006);
+          gdcm::DataElement de0( sisq0 );
+          de0.SetVR( gdcm::VR::SQ );
+          de0.SetValue( *sqi0 );
+          subds0.Insert( de0 );
+
+          gdcm::Item item1;
+          gdcm::DataSet &subds1 = item1.GetNestedDataSet();
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x0095)); //
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x9020)); //
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x9022)); //
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x9028)); //
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x9098)); //
+          sqi0->AddItem( item1 );
+          }
+
+/*
+    (0018,9042) SQ (Sequence with undefined length)               # u/l,1 MR Receive Coil Sequence
+      (fffe,e000) na (Item with undefined length)
+        (0018,1250) SH [MULTI COIL]                               # 10,1 Receive Coil Name
+        (0018,9041) LO (no value)                                 # 0,1 Receive Coil Manufacturer Name
+        (0018,9043) CS [MULTICOIL ]                               # 10,1 Receive Coil Type
+        (0018,9044) CS [NO]                                       # 2,1 Quadrature Receive Coil
+        (0018,9045) SQ (Sequence with undefined length)           # u/l,1 Multi-Coil Definition Sequence
+          (fffe,e000) na (Item with undefined length)
+            (0018,9047) SH [MULTI ELEMENT ]                       # 14,1 Multi-Coil Element Name
+            (0018,9048) CS [YES ]                                 # 4,1 Multi-Coil Element Used
+          (fffe,e00d)
+        (fffe,e0dd)
+      (fffe,e00d)
+*/
+          {
+        gdcm::SmartPointer<gdcm::SequenceOfItems> sqi0 = new gdcm::SequenceOfItems;
+          const gdcm::Tag sisq0(0x0018,0x9042);
+          gdcm::DataElement de0( sisq0 );
+          de0.SetVR( gdcm::VR::SQ );
+          de0.SetValue( *sqi0 );
+          subds0.Insert( de0 );
+
+          gdcm::Item item1;
+          gdcm::DataSet &subds1 = item1.GetNestedDataSet();
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x1250)); //
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x9041)); //
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x9043)); //
+          gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x9044)); //
+          //gdcm::InsertIfExist( subds1, ds, gdcm::Tag(0x0018,0x9045)); //
+          sqi0->AddItem( item1 );
+          }
+
+        sqi1->AddItem( item0 );
+        const gdcm::Tag sisq1(0x5200,0x9229);
+        gdcm::DataElement de1( sisq1 );
+        de1.SetVR( gdcm::VR::SQ );
+        de1.SetValue( *sqi1 );
+        enhds.Insert( de1 );
+        }
+      }
+
     for( ; file != files.end(); ++file, ++count )
       {
       gdcm::ImageReader reader;
@@ -477,6 +732,14 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
         {
         assert( 0 );
         }
+
+      // check consistency 
+      if( false )
+        {
+        //gdcm::DataSet & enhds = im0.GetFile().GetDataSet();
+        //gdcm::DataSet & ds = reader.GetFile().GetDataSet();
+        // TODO
+        }
       }
 
     im0.SetFileName( (targetname + "/new.dcm").c_str() );
@@ -484,7 +747,18 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
 
     gdcm::DataSet &ds = im0.GetFile().GetDataSet();
 
-    gdcm::MediaStorage ms = gdcm::MediaStorage::EnhancedMRImageStorage;
+    gdcm::MediaStorage ms;
+    switch( msorig )
+      {
+    case gdcm::MediaStorage::CTImageStorage:
+      ms = gdcm::MediaStorage::EnhancedCTImageStorage;
+      break;
+    case gdcm::MediaStorage::MRImageStorage:
+      ms = gdcm::MediaStorage::EnhancedMRImageStorage;
+      break;
+    default:
+      return 1;
+      }
 
     gdcm::DataElement de( gdcm::Tag(0x0008, 0x0016) );
     const char* msstr = gdcm::MediaStorage::GetMSString(ms);
@@ -502,8 +776,16 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
     }
 
   std::vector< gdcm::Directory::FilenamesType > const &unsorted = dv.GetUnsortedFiles();
-  std::cerr << "Could not process the following files (please report): " << std::endl;
+  if( !unsorted.empty() )
     {
+    std::string targetdir3 = outfilename;
+    targetdir3 += "/unhandled";
+    if( !gdcm::System::MakeDirectory( targetdir3.c_str() ) )
+      {
+      std::cerr << "Could not create dir: " << targetdir3 << std::endl;
+      return 1;
+      }
+    std::cerr << "Could not process the following files (please report): " << std::endl;
     std::vector< gdcm::Directory::FilenamesType >::const_iterator it = unsorted.begin();
     for( ; it != unsorted.end(); ++it )
       {
@@ -511,9 +793,17 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
       gdcm::Directory::FilenamesType::const_iterator file = files.begin();
       for( ; file != files.end(); ++file )
         {
-        std::cerr << *file << std::endl;
+        const char *f = file->c_str();
+        std::string targetdir2 = outfilename;
+        targetdir2 += "/unhandled/";
+        gdcm::Filename fn2( f );
+        const char *outfn2 = fn2.GetName();
+        targetdir2 += outfn2;
+        //std::cerr << f << " -> " << targetdir2 << std::endl;
+        std::ifstream in( f, std::ios::binary );
+        std::ofstream out( targetdir2.c_str() , std::ios::binary );
+        out << in.rdbuf();
         }
-
       }
     }
 
@@ -523,7 +813,7 @@ int MakeImageEnhanced( std::string const & filename, std::string const &outfilen
 namespace gdcm
 {
 
-const DataElement &GetNestedDataElement( const DataSet &ds, const Tag & t1, const Tag & t2 )
+static const DataElement &GetNestedDataElement( const DataSet &ds, const Tag & t1, const Tag & t2 )
 {
   assert( ds.FindDataElement( t1 ) );
   SmartPointer<SequenceOfItems> sqi1 = ds.GetDataElement( t1 ).GetValueAsSQ();
@@ -671,6 +961,7 @@ int main (int argc, char *argv[])
   std::string root;
   int resourcespath = 0;
   int mosaic = 0;
+  int mosaic_private = 0;
   int enhance = 1;
   int unenhance = 0;
   std::string xmlpath;
@@ -683,18 +974,19 @@ int main (int argc, char *argv[])
   int version = 0;
 
   std::string pattern;
-  while (1) {
+  while (true) {
     //int this_option_optind = optind ? optind : 1;
     int option_index = 0;
     static struct option long_options[] = {
-        {"input", 1, 0, 0},
-        {"output", 1, 0, 0},
+        {"input", 1, nullptr, 0},
+        {"output", 1, nullptr, 0},
         {"mosaic", 0, &mosaic, 1}, // split siemens mosaic into multiple frames
-        {"pattern", 1, 0, 0},               // p
+        {"pattern", 1, nullptr, 0},               // p
         {"enhance", 0, &enhance, 1},               // unenhance
         {"unenhance", 0, &unenhance, 1},               // unenhance
         {"root-uid", 1, &rootuid, 1}, // specific Root (not GDCM)
         //{"resources-path", 0, &resourcespath, 1},
+        {"mosaic-private", 0, &mosaic_private, 1}, // keep private attributes
 
 // General options !
         {"verbose", 0, &verbose, 1},
@@ -704,7 +996,7 @@ int main (int argc, char *argv[])
         {"help", 0, &help, 1},
         {"version", 0, &version, 1},
 
-        {0, 0, 0, 0}
+        {nullptr, 0, nullptr, 0}
     };
 
     c = getopt_long (argc, argv, "i:o:MUp:VWDEhv",
@@ -718,7 +1010,7 @@ int main (int argc, char *argv[])
       {
     case 0:
         {
-        const char *s = long_options[option_index].name;
+        const char *s = long_options[option_index].name; (void)s;
         //printf ("option %s", s);
         if (optarg)
           {
@@ -823,7 +1115,7 @@ int main (int argc, char *argv[])
     while (optind < argc)
       {
       //printf ("%s\n", argv[optind++]);
-      files.push_back( argv[optind++] );
+      files.emplace_back(argv[optind++] );
       }
     //printf ("\n");
     if( files.size() == 2
@@ -960,8 +1252,7 @@ int main (int argc, char *argv[])
     const unsigned int *dims = image.GetDimensions();
     const gdcm::DataElement &pixeldata = image.GetDataElement();
     const gdcm::ByteValue *bv = pixeldata.GetByteValue();
-    unsigned long slice_len = image.GetBufferLength() / dims[2];
-    //assert( image.GetBufferLength() == bv->GetLength() );
+    size_t slice_len = image.GetBufferLength() / dims[2];
 
     gdcm::FilenameGenerator fg;
     fg.SetNumberOfFilenames( dims[2] );
@@ -979,17 +1270,78 @@ int main (int argc, char *argv[])
     const double *origin = image.GetOrigin();
     double zspacing = image.GetSpacing(2);
 
+    gdcm::CSAHeader csa;
+    gdcm::DataSet & ds = reader.GetFile().GetDataSet();
+
+    gdcm::MrProtocol mrprot;
+    if( !csa.GetMrProtocol(ds, mrprot) ) return 1;
+
+    gdcm::MrProtocol::SliceArray sa;
+    b = mrprot.GetSliceArray(sa);
+    if( !b ) return 1;
+
+    size_t size = sa.Slices.size();
+    if( !size ) return 1;
+
+    if( !mosaic_private )
+    {
+      gdcm::Anonymizer ano;
+      ano.SetFile( reader.GetFile() );
+      // Remove CSA header
+      ano.RemovePrivateTags();
+    }
+
+    double slicePos[3];
+    double sliceNor[3];
+    namespace kwd = gdcm::Keywords;
+    gdcm::UIDGenerator ug;
+
+    kwd::InstanceNumber instart;
+    instart.Set(ds);
+    int istart = instart.GetValue();
+
     for(unsigned int i = 0; i < dims[2]; ++i)
       {
+      gdcm::MrProtocol::Slice & protSlice = sa.Slices[i];
+      gdcm::MrProtocol::Vector3 & protV = protSlice.Position;
+      gdcm::MrProtocol::Vector3 & protN = protSlice.Normal;
+      slicePos[0] = protV.dSag;
+      slicePos[1] = protV.dCor;
+      slicePos[2] = protV.dTra;
+      sliceNor[0] = protN.dSag;
+      sliceNor[1] = protN.dCor;
+      sliceNor[2] = protN.dTra;
+
       double new_origin[3];
       for (int j = 0; j < 3; j++)
         {
         // the n'th slice is n * z-spacing aloung the IOP-derived
         // z-axis
         new_origin[j] = origin[j] + normal[j] * i * zspacing;
+        if( std::fabs(slicePos[j] - new_origin[j]) > 1e-3 )
+          {
+          gdcmErrorMacro("Invalid position found");
+          return 1;
+          }
+        const double snv_dot = gdcm::DirectionCosines::Dot( normal, sliceNor );
+        if( std::fabs(1. - snv_dot) > 1e-6 )
+          {
+          gdcmErrorMacro("Invalid direction found");
+          return 1;
+          }
         }
 
+      kwd::SOPInstanceUID sid;
+      sid.SetValue( ug.Generate() );
+      ds.Replace( sid.GetAsDataElement() );
+
       const char *outfilenamei = fg.GetFilename(i);
+      kwd::SliceLocation sl;
+      sl.SetValue( new_origin[2] );
+      ds.Replace( sl.GetAsDataElement() );
+      kwd::InstanceNumber in;
+      in.SetValue( istart + i ); // Start at mosaic instance number
+      ds.Replace( in.GetAsDataElement() );
       gdcm::ImageWriter writer;
       writer.SetFileName( outfilenamei );
       //writer.SetFile( filter.GetFile() );
@@ -1081,8 +1433,8 @@ int main (int argc, char *argv[])
     gdcm::DirectionCosines dc( cosines );
     double normal[3];
     dc.Cross( normal );
-    const double *origin = image.GetOrigin();
-    double zspacing = image.GetSpacing(2);
+    //const double *origin = image.GetOrigin();
+    //double zspacing = image.GetSpacing(2);
 
     // Remove SharedFunctionalGroupsSequence
     gdcm::SmartPointer<gdcm::SequenceOfItems> sfgs =
@@ -1128,6 +1480,7 @@ int main (int argc, char *argv[])
 
     for(unsigned int i = 0; i < dims[2]; ++i)
       {
+#if 0
       double new_origin[3];
       for (int j = 0; j < 3; j++)
         {
@@ -1135,6 +1488,7 @@ int main (int argc, char *argv[])
         // z-axis
         new_origin[j] = origin[j] + normal[j] * i * zspacing;
         }
+#endif
 
       const char *outfilenamei = fg.GetFilename(i);
       //gdcm::ImageWriter writer;
@@ -1258,6 +1612,4 @@ int main (int argc, char *argv[])
     return 0;
 #endif
     }
-
-  return 0;
 }
