@@ -46,9 +46,9 @@ public:
     {
     assert( RessourcePaths.empty() );
     const char filename2[] = GDCM_CMAKE_INSTALL_PREFIX "/" GDCM_INSTALL_DATA_DIR "/XML/";
-    RessourcePaths.push_back( filename2 );
+    RessourcePaths.emplace_back(filename2 );
     const char filename3[] = GDCM_CMAKE_INSTALL_PREFIX " " GDCM_API_VERSION "/" GDCM_INSTALL_DATA_DIR "/XML/";
-    RessourcePaths.push_back( filename3 );
+    RessourcePaths.emplace_back(filename3 );
     const char *curprocfn = System::GetCurrentProcessFileName();
     if( curprocfn )
       {
@@ -60,12 +60,12 @@ public:
     const char *respath = System::GetCurrentResourcesDirectory();
     if( respath )
       {
-      RessourcePaths.push_back( respath );
+      RessourcePaths.emplace_back(respath );
       }
 #ifdef GDCM_BUILD_TESTING
     // Needed for backward compat and dashboard
     const char src_path[] = GDCM_SOURCE_DIR "/Source/InformationObjectDefinition/";
-    RessourcePaths.push_back( src_path );
+    RessourcePaths.emplace_back(src_path );
 #endif
     }
   std::vector<std::string> RessourcePaths;
@@ -95,7 +95,7 @@ Global::~Global()
     {
     //Internals->GlobalDicts.Unload();
     delete Internals;
-    Internals = NULL; // paranoid
+    Internals = nullptr; // paranoid
     }
 }
 
@@ -119,7 +119,7 @@ bool Global::Append(const char *path)
     {
     return false;
     }
-  Internals->RessourcePaths.push_back( path );
+  Internals->RessourcePaths.emplace_back(path );
   return true;
 }
 
@@ -133,6 +133,10 @@ bool Global::Prepend(const char *path)
   return true;
 }
 
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
 const char *Global::Locate(const char *resfile) const
 {
 #ifdef _WIN32
@@ -154,14 +158,14 @@ const char *Global::Locate(const char *resfile) const
       if( fullpath.size() >= sizeof(path) )
         {
         gdcmDebugMacro( "Impossible happen: path is too long" );
-        return NULL;
+        return nullptr;
         }
       strcpy(path, fullpath.c_str() );
       return path;
       }
     }
   // no match sorry  :(
-  return NULL;
+  return nullptr;
 }
 
 Dicts const &Global::GetDicts() const
