@@ -23,6 +23,10 @@
 #include "gdcmExplicitDataElement.h"
 #include "gdcmImplicitDataElement.h"
 
+#ifdef _MSC_VER
+#include <windows.h> // MultiByteToWideChar 
+#endif
+
 #ifdef GDCM_SUPPORT_BROKEN_IMPLEMENTATION
 #include "gdcmUNExplicitDataElement.h"
 #include "gdcmCP246ExplicitDataElement.h"
@@ -32,17 +36,23 @@
 #endif
 
 
-namespace gdcm
+namespace gdcm_ns
 {
 
+Reader::Reader():F(new File)
+{
+  Stream = nullptr;
+  Ifstream = nullptr;
+}
+
 Reader::~Reader()
 {
   if (Ifstream)
     {
     Ifstream->close();
     delete Ifstream;
-    Ifstream = NULL;
-    Stream = NULL;
+    Ifstream = nullptr;
+    Stream = nullptr;
     }
 }
 
@@ -198,9 +208,9 @@ namespace details
   class DefaultCaller
   {
   private:
-    gdcm::DataSet & m_dataSet;
+    DataSet & m_dataSet;
   public:
-    DefaultCaller(gdcm::DataSet &ds): m_dataSet(ds){}
+    DefaultCaller(DataSet &ds): m_dataSet(ds){}
     template<class T1, class T2>
       void ReadCommon(std::istream & is) const
         {
@@ -210,6 +220,9 @@ namespace details
       void ReadCommonWithLength(std::istream & is, VL & length) const
         {
         m_dataSet.template ReadWithLength<T1,T2>(is,length);
+        // manually set eofbit:
+        // https://groups.google.com/forum/?fromgroups#!topic/comp.lang.c++/yTW4ESh1IL8
+        is.setstate( std::ios::eofbit );
         }
     static void Check(bool b, std::istream &stream)
       {
@@ -221,11 +234,11 @@ namespace details
   class ReadUpToTagCaller
   {
   private:
-    gdcm::DataSet & m_dataSet;
-    const gdcm::Tag & m_tag;
-    std::set<gdcm::Tag> const & m_skipTags;
+    DataSet & m_dataSet;
+    const Tag & m_tag;
+    std::set<Tag> const & m_skipTags;
   public:
-    ReadUpToTagCaller(gdcm::DataSet &ds,const gdcm::Tag & tag, std::set<gdcm::Tag> const & skiptags)
+    ReadUpToTagCaller(DataSet &ds,const Tag & tag, std::set<Tag> const & skiptags)
     :
     m_dataSet(ds),m_tag(tag),m_skipTags(skiptags)
     {
@@ -239,7 +252,7 @@ namespace details
     template<class T1, class T2>
       void ReadCommonWithLength(std::istream & is, VL & length) const
         {
-        m_dataSet.template ReadUpToTagWithLength<T1,T2>(is,m_tag,length);
+        m_dataSet.template ReadUpToTagWithLength<T1,T2>(is,m_tag,m_skipTags,length);
         }
     static void Check(bool , std::istream &)  {}
   };
@@ -249,22 +262,49 @@ namespace details
   private:
     DataSet & m_dataSet;
     std::set<Tag> const & m_tags;
+    bool m_readvalues;
   public:
-    ReadSelectedTagsCaller(DataSet &ds, std::set<Tag> const & tags)
+    ReadSelectedTagsCaller(DataSet &ds, std::set<Tag> const & tags, const bool readvalues)
       :
-    m_dataSet(ds),m_tags(tags)
+    m_dataSet(ds),m_tags(tags),m_readvalues(readvalues)
     {
     }
 
     template<class T1, class T2>
     void ReadCommon(std::istream & is) const
     {
-      m_dataSet.template ReadSelectedTags<T1,T2>(is,m_tags);
+      m_dataSet.template ReadSelectedTags<T1,T2>(is,m_tags,m_readvalues);
     }
     template<class T1, class T2>
     void ReadCommonWithLength(std::istream & is, VL & length) const
     {
-      m_dataSet.template ReadSelectedTagsWithLength<T1,T2>(is,m_tags,length);
+      m_dataSet.template ReadSelectedTagsWithLength<T1,T2>(is,m_tags,length,m_readvalues);
+    }
+    static void Check(bool , std::istream &)  {}
+  };
+
+  class ReadSelectedPrivateTagsCaller
+  {
+  private:
+    DataSet & m_dataSet;
+    std::set<PrivateTag> const & m_groups;
+    bool m_readvalues;
+  public:
+    ReadSelectedPrivateTagsCaller(DataSet &ds, std::set<PrivateTag> const & groups, const bool readvalues)
+      :
+    m_dataSet(ds),m_groups(groups),m_readvalues(readvalues)
+    {
+    }
+
+    template<class T1, class T2>
+    void ReadCommon(std::istream & is) const
+    {
+      m_dataSet.template ReadSelectedPrivateTags<T1,T2>(is,m_groups,m_readvalues);
+    }
+    template<class T1, class T2>
+    void ReadCommonWithLength(std::istream & is, VL & length) const
+    {
+      m_dataSet.template ReadSelectedPrivateTagsWithLength<T1,T2>(is,m_groups,length,m_readvalues);
     }
     static void Check(bool , std::istream &)  {}
   };
@@ -282,9 +322,15 @@ bool Reader::ReadUpToTag(const Tag & tag, std::set<Tag> const & skiptags)
   return InternalReadCommon(caller);
 }
 
-bool Reader::ReadSelectedTags( std::set<Tag> const & selectedTags )
+bool Reader::ReadSelectedTags( std::set<Tag> const & selectedTags, bool readvalues )
 {
-  details::ReadSelectedTagsCaller caller(F->GetDataSet(), selectedTags);
+  details::ReadSelectedTagsCaller caller(F->GetDataSet(), selectedTags,readvalues);
+  return InternalReadCommon(caller);
+}
+
+bool Reader::ReadSelectedPrivateTags( std::set<PrivateTag> const & selectedPTags, bool readvalues )
+{
+  details::ReadSelectedPrivateTagsCaller caller(F->GetDataSet(), selectedPTags,readvalues);
   return InternalReadCommon(caller);
 }
 
@@ -343,10 +389,10 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
             }
           catch( std::exception &ex2 )
             {
-			(void)ex2;  //to avoid unreferenced variable warning on release
+            (void)ex2;  //to avoid unreferenced variable warning on release
             // Ok I get it now... there is absolutely no meta header, giving up
             //hasmetaheader = false;
-			gdcmErrorMacro(ex2.what());
+            gdcmErrorMacro(ex2.what());
             }
           }
         }
@@ -378,17 +424,11 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
       throw Exception( "Meta Header issue" );
       }
 
-  //std::cerr << ts.GetNegociatedType() << std::endl;
-  //std::cerr << TransferSyntax::GetTSString(ts) << std::endl;
   // Special case where the dataset was compressed using the deflate
   // algorithm
   if( ts == TransferSyntax::DeflatedExplicitVRLittleEndian )
     {
-#if 0
-  std::ofstream out( "/tmp/deflate.raw");
-  out << is.rdbuf();
-  out.close();
-#endif
+
     zlib_stream::zip_istream gzis( is );
     // FIXME: we also know in this case that we are dealing with Explicit:
     assert( ts.GetNegociatedType() == TransferSyntax::Explicit );
@@ -412,6 +452,7 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
         // LIBIDO-16-ACR_NEMA-Volume.dcm
         //F->GetDataSet().ReadUpToTag<ImplicitDataElement,SwapperDoOp>(is,tag, skiptags);
         //caller.template ReadCommon<ImplicitDataElement,SwapperDoOp>(is);
+        gdcmErrorMacro( "VirtualBigEndianNotHandled" );
         throw "Virtual Big Endian Implicit is not defined by DICOM";
         }
       else
@@ -434,18 +475,19 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
           std::streampos start = is.tellg();
           is.seekg( 0, std::ios::end);
           std::streampos end = is.tellg();
-		  std::streamoff theOffset = end-start;
-		  assert (theOffset > 0 || theOffset < std::numeric_limits<uint32_t>::max());
+          assert( !is.eof() );
+          assert( is.good() );
+          std::streamoff theOffset = end-start;
+          assert (theOffset > 0 || (uint32_t)theOffset < std::numeric_limits<uint32_t>::max());
           VL l = (uint32_t)(theOffset);
           is.seekg( start, std::ios::beg );
-          //F->GetDataSet().ReadUpToTagWithLength<ImplicitDataElement,SwapperNoOp>(is, tag, l);
+          assert( is.good() );
+          assert( !is.eof() );
           caller.template ReadCommonWithLength<ImplicitDataElement,SwapperNoOp>(is,l);
-          is.peek();
           }
         }
       else
         {
-        //F->GetDataSet().ReadUpToTag<ExplicitDataElement,SwapperNoOp>(is,tag, skiptags);
         caller.template ReadCommon<ExplicitDataElement,SwapperNoOp>(is);
         }
       }
@@ -595,30 +637,6 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
             // ExplicitImplicitDataElement class instead.
             // Simply rethrow the exception for now.
             throw;
-#if 0
-            is.clear();
-            if( haspreamble )
-              {
-              is.seekg(128+4, std::ios::beg);
-              }
-            else
-              {
-              is.seekg(0, std::ios::beg);
-              }
-            if( hasmetaheader )
-              {
-              // FIXME: we are reading twice the same meta-header, we succedeed the first time...
-              // We should be able to seek to proper place instead of re-reading
-              FileMetaInformation header;
-              header.Read(is);
-              }
-
-            // Explicit/Implicit
-            gdcmWarningMacro( "Attempt to read file with explicit/implicit" );
-            F->GetDataSet().Clear(); // remove garbage from 1st attempt...
-            //F->GetDataSet().template Read<UNExplicitImplicitDataElement,SwapperNoOp>(is);
-            caller.template ReadCommon<UNExplicitImplicitDataElement,SwapperNoOp>(is);
-#endif
             }
           }
         }
@@ -653,15 +671,20 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
         caller.template ReadCommon<ExplicitImplicitDataElement,SwapperNoOp>(is);
         // This file can only be rewritten as implicit...
         }
+      else
+        {
+        gdcmDebugMacro( "No way this is DICOM" );
+        success = false;
+        }
       }
 #else
-    gdcmDebugMacro( ex.what() );
+    gdcmDebugMacro( ex.what() ); (void)ex;
     success = false;
 #endif /* GDCM_SUPPORT_BROKEN_IMPLEMENTATION */
     }
   catch( Exception &ex )
     {
-	(void)ex;  //to avoid unreferenced variable warning on release
+    (void)ex;  //to avoid unreferenced variable warning on release
     gdcmDebugMacro( ex.what() );
     success = false;
     }
@@ -676,7 +699,7 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
     }
   catch( Exception &ex )
     {
-	(void)ex;  //to avoid unreferenced variable warning on release
+    (void)ex;  //to avoid unreferenced variable warning on release
     gdcmDebugMacro( ex.what() );
     success = false;
     }
@@ -685,11 +708,6 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
     gdcmWarningMacro( "Unknown exception" );
     success = false;
     }
-  //  if( !success )
-  //    {
-  //    F->GetHeader().Clear();
-  //    F->GetDataSet().Clear();
-  //    }
 
   // FIXME : call this function twice...
   if (Ifstream && Ifstream->is_open())
@@ -703,19 +721,24 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
   return success;
 }
 
+static inline bool isasciiupper( char c ) {
+  return c >= 'A' && c <= 'Z';
+}
+
 // This function re-implements code from:
 // http://www.dclunie.com/medical-image-faq/html/part2.html#DICOMTransferSyntaxDetermination
-// The above code does not work well for random file. It implicitly assumes we are trying
-// to read a DICOM file in the first place, while our goal is indeed to detect whether or
-// not the file can be assimilated as DICOM. So we extended it.
-// Of course this function only returns a 'maybe DICOM', since we are not guaranteed that
-// the stream is not truncated, but this is outside the scope of this function.
+// The above code does not work well for random file. It implicitly assumes we
+// are trying to read a DICOM file in the first place, while our goal is indeed
+// to detect whether or not the file can be assimilated as DICOM. So we
+// extended it.  Of course this function only returns a 'maybe DICOM', since we
+// are not guaranteed that the stream is not truncated, but this is outside the
+// scope of this function.
 bool Reader::CanRead() const
 {
   // fastpath
   std::istream &is = *Stream;
-  assert( is.good() );
-  assert( is.tellg() == std::streampos(0) );
+  if( is.bad() ) return false;
+  if( is.tellg() != std::streampos(0) ) return false;
     {
     is.seekg( 128, std::ios::beg ); // we ignore return value as we test is.good()
     char b[4];
@@ -728,10 +751,8 @@ bool Reader::CanRead() const
 
   // Start overhead for backward compatibility
   bool bigendian = false;
-  bool explicitvr	= false;
+  bool explicitvr = false;
   is.clear();
-  //is.seekg(0, std::ios::end);
-  //std::streampos filelen = is.tellg();
   is.seekg(0, std::ios::beg);
 
   char b[8];
@@ -748,7 +769,7 @@ bool Reader::CanRead() const
       if (b[4] < b[7]) bigendian=true;
       }
     // else littleendian
-    if (isupper(b[4]) && isupper(b[5])) explicitvr=true;
+    if (isasciiupper(b[4]) && isasciiupper(b[5])) explicitvr=true;
     }
   SwapCode sc = SwapCode::Unknown;
   TransferSyntax::NegociatedType nts = TransferSyntax::Unknown;
@@ -793,137 +814,6 @@ bool Reader::CanRead() const
       nts = TransferSyntax::Implicit;
     }
 
-#if 0
-  is.clear();
-  is.seekg(0, std::ios::end);
-  std::streampos filelen = is.tellg();
-  is.seekg(0, std::ios::beg);
-  Tag t;
-  VL gl; // group length
-  if( bigendian )
-    {
-    if( !t.Read<SwapperDoOp>(is) )
-      {
-      is.clear();
-      is.seekg(0, std::ios::beg);
-      return false;
-      }
-    }
-  else
-    {
-    if( !t.Read<SwapperNoOp>(is) )
-      {
-      is.clear();
-      is.seekg(0, std::ios::beg);
-      return false;
-      }
-    }
-  if( t.GetGroup() % 2 == 0 )
-    {
-    switch( t.GetGroup() )
-      {
-    case 0x0002:
-    //case 0x0004: // DICOMDIR is for media, thus FMI is compulsory
-    case 0x0008:
-      sc = SwapCode::LittleEndian;
-      break;
-    //case 0x0200: // FMI is Explicit VR Little Endian...
-    case 0x0800:
-      sc = SwapCode::BigEndian;
-      break;
-    default:
-      ;
-      }
-    if( sc != SwapCode::Unknown )
-      {
-      // Purposely not Re-use ReadVR since we can read VR_END
-      char vr_str[3];
-      is.read(vr_str, 2);
-      vr_str[2] = '\0';
-      // Cannot use GetVRTypeFromFile since is assert ...
-      VR::VRType vr = VR::GetVRType(vr_str);
-      if( vr != VR::VR_END )
-        {
-        nts = TransferSyntax::Explicit;
-        }
-      else
-        {
-        assert( !(VR::IsSwap(vr_str)));
-        is.seekg(-2, std::ios::cur); // Seek back
-        gl.Read<SwapperNoOp>(is);
-
-        if( t.GetElement() == 0x0000 )
-          {
-          switch(gl)
-            {
-          case 0x00000004 :
-            assert( sc == SwapCode::LittleEndian);    // 1234
-            sc = SwapCode::LittleEndian;    // 1234
-            break;
-          case 0x04000000 :
-            assert( sc == SwapCode::BigEndian);    // 1234
-            sc = SwapCode::BigEndian;       // 4321
-            break;
-          case 0x00040000 :
-            sc = SwapCode::BadLittleEndian; // 3412
-            gdcmWarningMacro( "Bad Little Endian" );
-            break;
-          case 0x00000400 :
-            sc = SwapCode::BadBigEndian;    // 2143
-            gdcmWarningMacro( "Bad Big Endian" );
-            break;
-          default:
-            ;
-            }
-          }
-        if( gl && gl < filelen )
-          nts = TransferSyntax::Implicit;
-        }
-      }
-    }
-  else
-    {
-    // US-IRAD-NoPreambleStartWith0003.dcm
-    gdcmDebugMacro( "Start with a private tag creator" );
-    if( t.GetGroup() > 0x0002 && t.GetGroup() < 0x8 )
-      {
-      switch( t.GetElement() )
-        {
-      case 0x0010:
-        sc = SwapCode::LittleEndian;
-        break;
-      default:
-        ;
-        }
-      }
-    if( sc != SwapCode::Unknown )
-      {
-      // Purposely not Re-use ReadVR since we can read VR_END
-      char vr_str[3];
-      is.read(vr_str, 2);
-      vr_str[2] = '\0';
-      // Cannot use GetVRTypeFromFile since is assert ...
-      VR::VRType vr = VR::GetVRType(vr_str);
-      if( vr != VR::VR_END )
-        {
-        nts = TransferSyntax::Explicit;
-        }
-      else
-        {
-        assert( !(VR::IsSwap(vr_str)));
-        is.seekg(-2, std::ios::cur); // Seek back
-        gl.Read<SwapperNoOp>(is);
-        if( t.GetElement() == 0x0000 )
-          {
-          assert( gl == 0x4 || gl == 0x04000000 );
-          }
-        if( gl && gl < filelen )
-          nts = TransferSyntax::Implicit;
-        }
-      }
-    }
-
-#endif
   // reset in all other cases:
   is.clear();
   is.seekg(0, std::ios::beg);
@@ -934,27 +824,51 @@ bool Reader::CanRead() const
   if( nts == TransferSyntax::Explicit && sc == SwapCode::LittleEndian ) return true;
   if( nts == TransferSyntax::Explicit && sc == SwapCode::BigEndian ) return true;
 
-//  assert( nts == TransferSyntax::Unknown );
-//  if( sc != SwapCode::Unknown )
-//    {
-//    gdcm::Reader r;
-//    r.SetStream( is );
-//    is.clear();
-//    is.seekg(0, std::ios::beg);
-//    return r.Read();
-//    }
-
   return false;
 }
 
-void Reader::SetFileName(const char *filename)
+#ifdef _MSC_VER
+namespace {
+static inline std::wstring ToUtf16(std::string str) {
+  std::wstring ret;
+  int len = MultiByteToWideChar(CP_UTF8, 0, str.c_str(), str.length(), NULL, 0);
+  if (len > 0) {
+    ret.resize(len);
+    MultiByteToWideChar(CP_UTF8, 0, str.c_str(), str.length(), &ret[0], len);
+  }
+  return ret;
+}
+}
+#endif
+
+void Reader::SetFileName(const char *uft8path)
 {
   if(Ifstream) delete Ifstream;
   Ifstream = new std::ifstream();
-  Ifstream->open(filename, std::ios::binary);
-  Stream = Ifstream;
-  assert( Stream && *Stream );
+  Ifstream->open(
+#ifdef _MSC_VER
+      ToUtf16(uft8path).c_str()
+#else
+      uft8path
+#endif
+	  , std::ios::binary);
+  if( Ifstream->is_open() )
+    {
+    Stream = Ifstream;
+    assert( Stream && *Stream );
+    }
+  else
+    {
+    delete Ifstream;
+    Ifstream = nullptr;
+    Stream = nullptr;
+    }
+}
+
+size_t Reader::GetStreamCurrentPosition() const
+{
+  return static_cast<size_t>(GetStreamPtr()->tellg());
 }
 
 
-} // end namespace gdcm
+} // end namespace gdcm_ns
