@@ -87,8 +87,7 @@ static Tag BasicApplicationLevelConfidentialityProfileAttributes[] = {
 
 
 Anonymizer::~Anonymizer()
-{
-}
+= default;
 
 bool Anonymizer::Empty( Tag const &t)
 {
@@ -99,7 +98,10 @@ bool Anonymizer::Empty( Tag const &t)
 bool Anonymizer::Remove( Tag const &t )
 {
   DataSet &ds = F->GetDataSet();
-  return ds.Remove( t ) == 1;
+  if(ds.FindDataElement(t))
+    return ds.Remove( t ) == 1;
+  else
+    return true;
 }
 
 bool Anonymizer::Replace( Tag const &t, const char *value )
@@ -121,7 +123,7 @@ bool Anonymizer::Replace( Tag const &t, const char *value, VL const & vl )
   DataSet &ds = F->GetDataSet();
   // Let's do the private tag:
   bool ret = false;
-  if ( t.IsPrivate() )
+  if ( t.IsPrivate() && !t.IsPrivateCreator() )
     {
     // Only one operation is allowed: making a private tag empty ...
     if ( vl == 0 )
@@ -159,7 +161,7 @@ bool Anonymizer::Replace( Tag const &t, const char *value, VL const & vl )
   else
     {
     // Ok this is a public element
-    assert( t.IsPublic() );
+    assert( t.IsPublic() || t.IsPrivateCreator() );
     const DictEntry &dictentry = dicts.GetDictEntry(t);
     if ( dictentry.GetVR() == VR::INVALID
       || dictentry.GetVR() == VR::UN
@@ -200,7 +202,7 @@ bool Anonymizer::Replace( Tag const &t, const char *value, VL const & vl )
         }
       else
         {
-        gdcmWarningMacro( "You need to explicitely specify the length for this type of vr: " << dictentry.GetVR() );
+        gdcmWarningMacro( "You need to explicitly specify the length for this type of vr: " << dictentry.GetVR() );
         ret = false;
         }
 #if 0
@@ -290,8 +292,8 @@ static bool Anonymizer_RemoveRetired(File const &file, DataSet &ds)
         SmartPointer<SequenceOfItems> sq = de.GetValueAsSQ();
         if( sq )
           {
-          gdcm::SequenceOfItems::SizeType n = sq->GetNumberOfItems();
-          for( gdcm::SequenceOfItems::SizeType i = 1; i <= n; i++) // item starts at 1, not 0
+          SequenceOfItems::SizeType n = sq->GetNumberOfItems();
+          for( SequenceOfItems::SizeType i = 1; i <= n; i++) // item starts at 1, not 0
             {
             Item &item = sq->GetItem( i );
             DataSet &nested = item.GetNestedDataSet();
@@ -336,8 +338,8 @@ static bool Anonymizer_RemoveGroupLength(File const &file, DataSet &ds)
         SmartPointer<SequenceOfItems> sq = de.GetValueAsSQ();
         if( sq )
           {
-          gdcm::SequenceOfItems::SizeType n = sq->GetNumberOfItems();
-          for( gdcm::SequenceOfItems::SizeType i = 1; i <= n; i++) // item starts at 1, not 0
+          SequenceOfItems::SizeType n = sq->GetNumberOfItems();
+          for( SequenceOfItems::SizeType i = 1; i <= n; i++) // item starts at 1, not 0
             {
             Item &item = sq->GetItem( i );
             DataSet &nested = item.GetNestedDataSet();
@@ -382,8 +384,8 @@ static bool Anonymizer_RemovePrivateTags(File const &file, DataSet &ds)
         SmartPointer<SequenceOfItems> sq = de.GetValueAsSQ();
         if( sq )
           {
-          gdcm::SequenceOfItems::SizeType n = sq->GetNumberOfItems();
-          for( gdcm::SequenceOfItems::SizeType i = 1; i <= n; i++) // item starts at 1, not 0
+          SequenceOfItems::SizeType n = sq->GetNumberOfItems();
+          for( SequenceOfItems::SizeType i = 1; i <= n; i++) // item starts at 1, not 0
             {
             Item &item = sq->GetItem( i );
             DataSet &nested = item.GetNestedDataSet();
@@ -411,7 +413,7 @@ bool Anonymizer::RemovePrivateTags()
  * In order to implement the dummy 'memory' we use a static std::map
  * this works great but we cannot be thread safe.
  * In order to be thread safe, we would need to externalize this map generation
- * maybe using a gdcm::Scanner do the operation once (Scanner is doing) the merging
+ * maybe using a Scanner do the operation once (Scanner is doing) the merging
  * automatically...
  * this is left as an exercise for the reader :)
  */
@@ -453,8 +455,8 @@ bool Anonymizer::CheckIfSequenceContainsAttributeToAnonymize(File const &file, S
   if( found ) return true;
 
   // now look into sub-sequence:
-  gdcm::SequenceOfItems::SizeType n = sqi->GetNumberOfItems();
-  for( gdcm::SequenceOfItems::SizeType i = 1; i <= n; i++) // item starts at 1, not 0
+  SequenceOfItems::SizeType n = sqi->GetNumberOfItems();
+  for( SequenceOfItems::SizeType i = 1; i <= n; i++) // item starts at 1, not 0
     {
     Item &item = sqi->GetItem( i );
     DataSet &nested = item.GetNestedDataSet();
@@ -463,7 +465,7 @@ bool Anonymizer::CheckIfSequenceContainsAttributeToAnonymize(File const &file, S
       {
       const DataElement &de = *it;
       VR vr = DataSetHelper::ComputeVR(file, nested, de.GetTag() );
-      SmartPointer<SequenceOfItems> sqi2 = 0;
+      SmartPointer<SequenceOfItems> sqi2 = nullptr;
       if( vr == VR::SQ )
         {
         sqi2 = de.GetValueAsSQ();
@@ -555,7 +557,7 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
     {
     const DataElement &de = *it;
     //const SequenceOfItems *sqi = de.GetSequenceOfItems();
-    SmartPointer<SequenceOfItems> sqi = 0;
+    SmartPointer<SequenceOfItems> sqi = nullptr;
     VR vr = DataSetHelper::ComputeVR(*F, ds, de.GetTag() );
     if( vr == VR::SQ )
       {
@@ -649,21 +651,26 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
 
     sq->AddItem(item2);
 
-    // 4. All instances of the Encrypted Attributes Data Set shall be encoded with a DICOM Transfer Syntax,
-    // encrypted, and stored in the dataset to be protected as an Item of the Encrypted Attributes Sequence
-    // (0400,0500). The encryption shall be done using RSA [RFC 2313] for the key transport of the
-    // content-encryption keys. A de-identifier conforming to this security profile may use either AES or
-    // Triple-DES for content-encryption. The AES key length may be any length allowed by the RFCs. The
-    // Triple-DES key length is 168 bits as defined by ANSI X9.52. Encoding shall be performed according
-    // to the specifications for RSA Key Transport and Triple DES Content Encryption in RFC-3370 and for
-    // AES Content Encryption in RFC-3565.
+    // 4. All instances of the Encrypted Attributes Data Set shall be encoded
+    // with a DICOM Transfer Syntax, encrypted, and stored in the dataset to be
+    // protected as an Item of the Encrypted Attributes Sequence (0400,0500).
+    // The encryption shall be done using RSA [RFC 2313] for the key transport
+    // of the content-encryption keys. A de-identifier conforming to this
+    // security profile may use either AES or Triple-DES for
+    // content-encryption. The AES key length may be any length allowed by the
+    // RFCs. The Triple-DES key length is 168 bits as defined by ANSI X9.52.
+    // Encoding shall be performed according to the specifications for RSA Key
+    // Transport and Triple DES Content Encryption in RFC-3370 and for AES
+    // Content Encryption in RFC-3565.
 
-    // 5. No requirements on the size of the asymmetric key pairs used for RSA key transport are defined in
-    // this confidentiality scheme. Implementations claiming conformance to the Basic Application Level
-    // Confidentiality Profile as a de-identifier shall always protect (e.g. encrypt and replace) the SOP
-    // Instance UID (0008,0018) Attribute as well as all references to other SOP Instances, whether
-    // contained in the main dataset or embedded in an Item of a Sequence of Items, that could potentially
-    // be used by unauthorized entities to identify the patient.
+    // 5. No requirements on the size of the asymmetric key pairs used for RSA
+    // key transport are defined in this confidentiality scheme.
+    // Implementations claiming conformance to the Basic Application Level
+    // Confidentiality Profile as a de-identifier shall always protect (e.g.
+    // encrypt and replace) the SOP Instance UID (0008,0018) Attribute as well
+    // as all references to other SOP Instances, whether contained in the main
+    // dataset or embedded in an Item of a Sequence of Items, that could
+    // potentially be used by unauthorized entities to identify the patient.
 
     // Insert sequence into data set
     DataElement subdes( Tag(0x0400,0x0500) );
@@ -675,8 +682,9 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile1()
     }
   this->InvokeEvent( IterationEvent() );
 
-  // 2. Each Attribute to be protected shall then either be removed from the dataset, or have its value
-  // replaced by a different "replacement value" which does not allow identification of the patient.
+  // 2. Each Attribute to be protected shall then either be removed from the
+  // dataset, or have its value replaced by a different "replacement value"
+  // which does not allow identification of the patient.
 
   //for(const Tag *ptr = start ; ptr != end ; ++ptr)
   //  {
@@ -706,12 +714,13 @@ catch(...)
   // Group Length are removed since PS 3.3-2008
   RemoveGroupLength();
 
-  // 3. At the discretion of the de-identifier, Attributes may be added to the dataset to be protected.
-  // ...
+  // 3. At the discretion of the de-identifier, Attributes may be added to the
+  // dataset to be protected.  ...
 
-  // 6. The attribute Patient Identity Removed (0012,0062) shall be replaced or added to the dataset with a
-  // value of YES, and a value inserted in De-identification Method (0012,0063) or De-identification
-  // Method Code Sequence (0012,0064).
+  // 6. The attribute Patient Identity Removed (0012,0062) shall be replaced or
+  // added to the dataset with a value of YES, and a value inserted in
+  // De-identification Method (0012,0063) or De-identification Method Code
+  // Sequence (0012,0064).
   Replace( Tag(0x0012,0x0062), "YES");
   Replace( Tag(0x0012,0x0063), "BASIC APPLICATION LEVEL CONFIDENTIALITY PROFILE");
 
@@ -812,6 +821,15 @@ generate a DICOMDIR
   return !b;
 }
 
+Anonymizer::DummyMapNonUIDTags Anonymizer::dummyMapNonUIDTags;
+Anonymizer::DummyMapUIDTags Anonymizer::dummyMapUIDTags;
+
+void Anonymizer::ClearInternalUIDs()
+{
+  dummyMapNonUIDTags.clear();
+  dummyMapUIDTags.clear();
+}
+
 bool Anonymizer::BALCPProtect(DataSet &ds, Tag const & tag, IOD const & iod)
 {
   // \precondition
@@ -821,11 +839,6 @@ bool Anonymizer::BALCPProtect(DataSet &ds, Tag const & tag, IOD const & iod)
   ae.SetTag( tag );
   this->InvokeEvent( ae );
 
-  typedef std::pair< Tag, std::string > TagValueKey;
-  typedef std::map< TagValueKey, std::string > DummyMapNonUIDTags;
-  typedef std::map< std::string, std::string > DummyMapUIDTags;
-  static DummyMapNonUIDTags dummyMapNonUIDTags;
-  static DummyMapUIDTags dummyMapUIDTags;
 
   bool canempty = CanEmptyTag( tag, iod );
   if( !canempty )
@@ -836,7 +849,7 @@ bool Anonymizer::BALCPProtect(DataSet &ds, Tag const & tag, IOD const & iod)
     if ( IsVRUI( tag ) )
       {
       std::string UIDToAnonymize = "";
-      gdcm::UIDGenerator uid;
+      UIDGenerator uid;
 
       if( !copy.IsEmpty() )
         {
@@ -931,7 +944,7 @@ void Anonymizer::RecurseDataSet( DataSet & ds )
     DataElement de = *it; ++it;
     //const SequenceOfItems *sqi = de.GetSequenceOfItems();
     VR vr = DataSetHelper::ComputeVR(*F, ds, de.GetTag() );
-    SmartPointer<SequenceOfItems> sqi = 0;
+    SmartPointer<SequenceOfItems> sqi = nullptr;
     if( vr == VR::SQ )
       {
       sqi = de.GetValueAsSQ();
@@ -943,8 +956,8 @@ void Anonymizer::RecurseDataSet( DataSet & ds )
       assert( sqi->IsUndefinedLength() );
       //de.GetVL().SetToUndefined();
       //sqi->SetLengthToUndefined();
-      gdcm::SequenceOfItems::SizeType n = sqi->GetNumberOfItems();
-      for( gdcm::SequenceOfItems::SizeType i = 1; i <= n; ++i )
+      SequenceOfItems::SizeType n = sqi->GetNumberOfItems();
+      for( SequenceOfItems::SizeType i = 1; i <= n; ++i )
         {
         Item &item = sqi->GetItem( i );
         DataSet &nested = item.GetNestedDataSet();
@@ -1090,7 +1103,7 @@ bool Anonymizer::BasicApplicationLevelConfidentialityProfile2()
   // of the encrypted one ?
   if( !nds2.FindDataElement( Tag(0x8,0x18) ) )
     {
-    gdcm::MediaStorage ms;
+    MediaStorage ms;
     ms.SetFromFile( *F );
     if( ms != MediaStorage::MediaStorageDirectoryStorage )
       {
