@@ -78,7 +78,7 @@ public:
     os << "TypeOfData                         :" << TypeOfData << std::endl;
     os << "CurveDescription                   :" << CurveDescription << std::endl;
     os << "DataValueRepresentation            :" << DataValueRepresentation << std::endl;
-    unsigned short * p = (unsigned short*)&Data[0];
+    const unsigned short * p = (const unsigned short*)(void*)&Data[0];
     for(int i = 0; i < NumberOfPoints; i+=2)
       {
       os << p[i] << "," << p[i+1] << std::endl;
@@ -291,7 +291,7 @@ static const char * const TypeOfDataDescription[][2] = {
 { "FLOW" , "flow data" },
 { "PHYSIO" , "physio data" },
 { "RESP" , "Respiration trace" },
-{ 0 , 0 }
+{ nullptr , nullptr }
 };
 const char *Curve::GetTypeOfDataDescription() const
 {
@@ -431,7 +431,7 @@ void Curve::GetAsPoints(float *array) const
     }
   assert( Internal->Dimensions == 1 || Internal->Dimensions == 2 );
 
-  int mult = Internal->Dimensions;
+  const int mult = Internal->Dimensions;
   int genidx = -1;
   if( !Internal->CurveDataDescriptor.empty() )
     {
@@ -452,31 +452,51 @@ void Curve::GetAsPoints(float *array) const
       assert( 0 && "TODO" );
       }
     }
+  const char * beg = &Internal->Data[0];
+  const char * end = beg + Internal->Data.size();
+  if( genidx == -1 )
+    {
+    assert( end == beg + 2 * Internal->NumberOfPoints ); (void)beg;(void)end;
+    }
+  else
+    {
+    assert( end == beg + mult * Internal->NumberOfPoints ); (void)beg;(void)end;
+    }
+
+
   if( Internal->DataValueRepresentation == 0 )
     {
-    uint16_t * p = (uint16_t*)&Internal->Data[0];
+    // PS 3.3 - 2004
+    // C.10.2.1.5 Curve data descriptor, coordinate start value, coordinate step value
+    uint16_t * p = (uint16_t*)(void*)&Internal->Data[0];
     // X
     if( genidx == 0 )
       for(int i = 0; i < Internal->NumberOfPoints; i++ )
-        array[3*i+0] = ComputeValueFromStartAndStep( i );
+        array[3*i+0] = (float)ComputeValueFromStartAndStep( i );
     else
       for(int i = 0; i < Internal->NumberOfPoints; i++ )
-        array[3*i+0] = p[mult*i + 0];
+        array[3*i+0] = p[i + 0];
     // Y
     if( genidx == 1 )
       for(int i = 0; i < Internal->NumberOfPoints; i++ )
-        array[3*i+1] = ComputeValueFromStartAndStep( i );
+        array[3*i+1] = (float)ComputeValueFromStartAndStep( i );
     else
       {
-      if( mult > 1 && genidx == -1 )
+      if( mult == 2 && genidx == -1 )
+        {
         for(int i = 0; i < Internal->NumberOfPoints; i++ )
-          array[3*i+1] = p[mult*i + 1];
-      else if( mult > 1 )
+          array[3*i+1] = p[i + 1];
+        }
+      else if( mult == 2 && genidx == 0 )
+        {
         for(int i = 0; i < Internal->NumberOfPoints; i++ )
-          array[3*i+1] = p[mult*i + 0];
+          array[3*i+1] = p[i + 0];
+        }
       else
+        {
         for(int i = 0; i < Internal->NumberOfPoints; i++ )
           array[3*i+1] = 0;
+        }
       }
     // Z
     for(int i = 0; i < Internal->NumberOfPoints; i++ )
@@ -484,7 +504,7 @@ void Curve::GetAsPoints(float *array) const
     }
   else if( Internal->DataValueRepresentation == 1 )
     {
-    int16_t * p = (int16_t*)&Internal->Data[0];
+    int16_t * p = (int16_t*)(void*)&Internal->Data[0];
     for(int i = 0; i < Internal->NumberOfPoints; i++ )
       {
       array[3*i+0] = p[mult*i + 0];
@@ -497,7 +517,7 @@ void Curve::GetAsPoints(float *array) const
     }
   else if( Internal->DataValueRepresentation == 2 )
     {
-    float * p = (float*)&Internal->Data[0];
+    float * p = (float*)(void*)&Internal->Data[0];
     for(int i = 0; i < Internal->NumberOfPoints; i++ )
       {
       array[3*i+0] = p[mult*i + 0];
@@ -510,7 +530,7 @@ void Curve::GetAsPoints(float *array) const
     }
   else if( Internal->DataValueRepresentation == 3 )
     {
-    double * p = (double*)&Internal->Data[0];
+    double * p = (double*)(void*)&Internal->Data[0];
     for(int i = 0; i < Internal->NumberOfPoints; i++ )
       {
       array[3*i+0] = (float)p[mult*i + 0];
@@ -523,7 +543,7 @@ void Curve::GetAsPoints(float *array) const
     }
   else if( Internal->DataValueRepresentation == 4 )
     {
-    int32_t * p = (int32_t*)&Internal->Data[0];
+    int32_t * p = (int32_t*)(void*)&Internal->Data[0];
     for(int i = 0; i < Internal->NumberOfPoints; i++ )
       {
       array[3*i+0] = (float)p[mult*i + 0];
