@@ -1,9 +1,8 @@
 /*=========================================================================
 
   Program: GDCM (Grassroots DICOM). A DICOM library
-  Module:  $URL$
 
-  Copyright (c) 2006-2010 Mathieu Malaterre
+  Copyright (c) 2006-2011 Mathieu Malaterre
   All rights reserved.
   See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.
 
@@ -84,7 +83,7 @@ VR DataSetHelper::ComputeVR(File const &file, DataSet const &ds, const Tag& tag)
   //const Dict &d = dicts.GetPublicDict();
 
   std::string strowner;
-  const char *owner = 0;
+  const char *owner = nullptr;
   const Tag& t = tag;
   if( t.IsPrivate() && !t.IsPrivateCreator() )
     {
@@ -123,7 +122,7 @@ VR DataSetHelper::ComputeVR(File const &file, DataSet const &ds, const Tag& tag)
       // FIXME:
       // gdcmDataExtra/gdcmSampleData/ImagesPapyrus/TestImages/wristb.pap
       // It's the contrary: root dataset does not have a Pixel Representation, but each SQ do...
-      assert( rootds.FindDataElement( pixelrep ) || ds.FindDataElement( pixelrep ) );
+      //assert( rootds.FindDataElement( pixelrep ) || ds.FindDataElement( pixelrep ) );
       if( ds.FindDataElement( pixelrep ) )
         {
         at.SetFromDataElement( ds.GetDataElement( pixelrep ) );
@@ -135,16 +134,17 @@ VR DataSetHelper::ComputeVR(File const &file, DataSet const &ds, const Tag& tag)
       else
         {
         //throw Exception( "Unhandled" );
-        gdcmWarningMacro( "Unhandled" );
+        gdcmWarningMacro( "Unhandled" ); // Typical to PDF encapsulated with some illegal attribute
         vr = VR::INVALID;
         }
       //assert( at.GetValue() == 0 || at.GetValue() == 1 );
-      if( at.GetValue() )
+      if( at.GetValue() == 1 )
         {
         vr = VR::SS;
         }
-      else
+      else /*if( at.GetValue() == 0 )*/
         {
+        // default to VR:US always (even when attribute is missing, or impossible to compute). This is much better than VR:UN anyway
         vr = VR::US;
         }
       }
@@ -181,10 +181,12 @@ VR DataSetHelper::ComputeVR(File const &file, DataSet const &ds, const Tag& tag)
     else // ( pixeldata == t  )
       {
       // For Pixel Data:
-      assert( ds.FindDataElement( bitsallocated ) );
+      if( !ds.FindDataElement( bitsallocated ) )
+        return VR::UN;
       Attribute<0x0028,0x0100> at;
       at.SetFromDataElement( ds.GetDataElement( bitsallocated ) );
       }
+    (void)v;
 
     if( pixeldata == t || t.IsGroupXX(overlaydata) )
       {
@@ -217,7 +219,7 @@ VR DataSetHelper::ComputeVR(File const &file, DataSet const &ds, const Tag& tag)
       vr = VR::INVALID;
       }
     }
-  else if( vr == VR::US_SS_OW )
+  else if( vr == VR::US_SS_OW || vr == VR::US_OW )
     {
     vr = VR::OW;
     }
