@@ -117,7 +117,7 @@ Retrieved from: http://en.literateprograms.org/Median_cut_algorithm_(C_Plus_Plus
     Point* points;
     int pointsLength;
   public:
-    Block(Point* points, int pointsLength);
+    Block(Point* points, std::ptrdiff_t pointsLength);
     Point * getPoints();
     int numPoints() const;
     int longestSideIndex() const;
@@ -138,10 +138,11 @@ Retrieved from: http://en.literateprograms.org/Median_cut_algorithm_(C_Plus_Plus
 
   //std::list<Point> medianCut(Point* image, int numPoints, unsigned int desiredSize);
 
-  Block::Block(Point* pts, int ptslen)
+  Block::Block(Point* pts, std::ptrdiff_t ptslen)
     {
+    assert( ptslen > 0 );
     this->points = pts;
-    this->pointsLength = ptslen;
+    this->pointsLength = (int)ptslen;
     for(int i=0; i < NUM_DIMENSIONS; i++)
       {
       minCorner.x[i] = std::numeric_limits<unsigned char>::min();
@@ -206,15 +207,25 @@ Retrieved from: http://en.literateprograms.org/Median_cut_algorithm_(C_Plus_Plus
   std::list<Point> medianCut(DataElement const &PixelData, int numPoints, unsigned int desiredSize,
     std::vector<unsigned char> & outputimage )
     {
-    //Point* points = (Point*)malloc(sizeof(Point) * numPoints);
+    assert( numPoints > 0 );
+    //Point* Points = (Point*)malloc(sizeof(Point) * numPoints);
     Point* Points = new Point[numPoints];
+    assert( Points );
     const ByteValue *bv = PixelData.GetByteValue();
-    const char *inbuffer = bv->GetPointer();
+    assert( bv );
+    const unsigned char *inbuffer = (const unsigned char*)bv->GetPointer();
+    assert( inbuffer );
     size_t bvlen = bv->GetLength(); (void)bvlen;
     assert( bvlen == (size_t) numPoints * 3 ); // only 8bits RGB please
     for(int i = 0; i < numPoints; i++)
       {
+#if 0
       memcpy(&Points[i], inbuffer + 3 * i, 3);
+#else
+      Points[i].x[0] = inbuffer[ 3 * i + 0 ];
+      Points[i].x[1] = inbuffer[ 3 * i + 1 ];
+      Points[i].x[2] = inbuffer[ 3 * i + 2 ];
+#endif
       }
     Point* image = Points;
 
@@ -224,7 +235,7 @@ Retrieved from: http://en.literateprograms.org/Median_cut_algorithm_(C_Plus_Plus
     initialBlock.shrink();
 
     blockQueue.push(initialBlock);
-    while (blockQueue.size() < desiredSize && blockQueue.top().numPoints() > 1)
+    while (blockQueue.size() < desiredSize /*&& blockQueue.top().numPoints() > 1*/ )
       {
       Block longestBlock = blockQueue.top();
 
@@ -259,7 +270,7 @@ Retrieved from: http://en.literateprograms.org/Median_cut_algorithm_(C_Plus_Plus
       blockQueue.pop();
       Point * points = block.getPoints();
 
-      int sum[NUM_DIMENSIONS] = {0};
+      int sum[NUM_DIMENSIONS] = {0,0,0};
       for(int i=0; i < block.numPoints(); i++)
         {
         for(int j=0; j < NUM_DIMENSIONS; j++)
@@ -277,13 +288,13 @@ Retrieved from: http://en.literateprograms.org/Median_cut_algorithm_(C_Plus_Plus
       result.push_back(averagePoint);
 
       //int index = std::distance(s.begin(), it.first);
-      int index = result.size();
+      size_t index = result.size();
       assert( index <= 256 );
 
       for(int i = 0; i < numPoints; i++)
         {
-        const char *currentcolor = inbuffer + 3 * i;
-        for(int j = 0; j < block.numPoints(); j++)
+        const unsigned char *currentcolor = inbuffer + 3 * i;
+        for(size_t j = 0; j < (size_t)block.numPoints(); j++)
           {
           assert( currentcolor < inbuffer + bvlen );
           assert( currentcolor + 3 <= inbuffer + bvlen );
@@ -291,7 +302,7 @@ Retrieved from: http://en.literateprograms.org/Median_cut_algorithm_(C_Plus_Plus
             {
             //assert( outputimage[i] == 0 );
             assert( index > 0 );
-            outputimage[i] = index - 1;
+            outputimage[i] = (unsigned char)(index - 1);
             }
           }
         }
@@ -315,7 +326,7 @@ void IconImageGenerator::BuildLUT( Bitmap & bitmap, unsigned int maxcolor )
   std::list<Point> palette =
     medianCut(bitmap.GetDataElement(), numPoints, maxcolor, indeximage);
 
-  int ncolors = palette.size();
+  size_t ncolors = palette.size();
   LookupTable & lut = bitmap.GetLUT();
   lut.Clear();
   lut.Allocate( 8 );
@@ -333,11 +344,11 @@ void IconImageGenerator::BuildLUT( Bitmap & bitmap, unsigned int maxcolor )
 
   for( int i = 0; i < 3; ++i )
     {
-    lut.InitializeLUT( LookupTable::LookupTableType(i), ncolors, 0, 8 );
-    lut.SetLUT( LookupTable::LookupTableType(i), &buffer[i][0], ncolors );
+    lut.InitializeLUT( LookupTable::LookupTableType(i), (unsigned short)ncolors, 0, 8 );
+    lut.SetLUT( LookupTable::LookupTableType(i), &buffer[i][0], (unsigned short)ncolors );
     }
 
-  bitmap.GetDataElement().SetByteValue( (char*)&indeximage[0], indeximage.size() );
+  bitmap.GetDataElement().SetByteValue( (char*)&indeximage[0], (uint32_t)indeximage.size() );
   assert( lut.Initialized() );
 }
 
@@ -393,7 +404,7 @@ void ComputeMinMax( const TPixelType *p, size_t npixels , double & min, double &
   // let's fake a slightly different min/max found:
   if( lmin == lmax )
     {
-    if( lmax + 1 < lmax )
+    if( lmax == std::numeric_limits<TPixelType>::max() )
       {
       lmin--;
       assert( lmin + 1 > lmin );
@@ -432,7 +443,7 @@ void ComputeMinMax( const TPixelType *p, size_t npixels , double & min, double &
   // let's fake a slightly different min/max found:
   if( lmin == lmax )
     {
-    if( lmax + 1 < lmax )
+    if( lmax == std::numeric_limits<TPixelType>::max() )
       {
       lmin--;
       assert( lmin + 1 > lmin );
@@ -577,9 +588,9 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
 
     if( I->GetPixelFormat().GetBitsAllocated() == 16 )
       {
-      assert( I->GetPixelFormat().GetPixelRepresentation() == 0 );
+      //assert( I->GetPixelFormat().GetPixelRepresentation() == 0 );
       std::string s = ss.str();
-      gdcm::Rescaler r;
+      Rescaler r;
       r.SetPixelFormat( I->GetPixelFormat() );
       //r.SetPixelFormat( PixelFormat::UINT16 );
 
@@ -620,7 +631,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         std::string s2 = ss2.str();
         // As per standard, we only support 8bits icon
         I->SetPixelFormat( PixelFormat::UINT8 );
-        pixeldata.SetByteValue( &s2[0], s2.size() );
+        pixeldata.SetByteValue( &s2[0], (uint32_t)s2.size() );
 
         BuildLUT( *I, 256 );
         }
@@ -628,7 +639,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         {
         I->SetPixelFormat( PixelFormat::UINT8 );
         I->GetPixelFormat().SetSamplesPerPixel( 3 );
-        pixeldata.SetByteValue( &v8[0], v8.size() );
+        pixeldata.SetByteValue( &v8[0], (uint32_t)v8.size() );
         }
       }
     else
@@ -642,7 +653,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
 
         // As per standard, we only support 8bits icon
         I->SetPixelFormat( PixelFormat::UINT8 );
-        pixeldata.SetByteValue( &s[0], s.size() );
+        pixeldata.SetByteValue( &s[0], (uint32_t)s.size() );
 
         BuildLUT(*I, 256 );
         }
@@ -650,7 +661,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         {
         I->SetPixelFormat( PixelFormat::UINT8 );
         I->GetPixelFormat().SetSamplesPerPixel( 3 );
-        pixeldata.SetByteValue( &s[0], s.size() );
+        pixeldata.SetByteValue( &s[0], (uint32_t)s.size() );
         }
       }
     }
@@ -692,12 +703,12 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
           if (G > 255) G = 255;
           if (B > 255) B = 255;
 
-          *ybr_out = R; ++ybr_out;
-          *ybr_out = G; ++ybr_out;
-          *ybr_out = B; ++ybr_out;
+          *ybr_out = (unsigned char)R; ++ybr_out;
+          *ybr_out = (unsigned char)G; ++ybr_out;
+          *ybr_out = (unsigned char)B; ++ybr_out;
           }
 #if 0
-    std::ofstream d( "/tmp/d.rgb" );
+    std::ofstream d( "/tmp/d.rgb", std::ios::binary );
     d.write( &tempvbuf[0], tempvbuf.size() );
     d.close();
 #endif
@@ -739,9 +750,9 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
           if (G > 255) G = 255;
           if (B > 255) B = 255;
 
-          *ybr_out = R; ++ybr_out;
-          *ybr_out = G; ++ybr_out;
-          *ybr_out = B; ++ybr_out;
+          *ybr_out = (unsigned char)R; ++ybr_out;
+          *ybr_out = (unsigned char)G; ++ybr_out;
+          *ybr_out = (unsigned char)B; ++ybr_out;
           }
         assert( ybra + 2 * ybrl == ybr_end ); (void)ybr_end;
         assert( ybrb + 1 * ybrl == ybr_end );
@@ -789,7 +800,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         {
         // As per standard, we only support 8bits icon
         I->SetPixelFormat( PixelFormat::UINT8 );
-        pixeldata.SetByteValue( &s[0], s.size() );
+        pixeldata.SetByteValue( &s[0], (uint32_t)s.size() );
 
         BuildLUT(*I, 256 );
         }
@@ -797,7 +808,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         {
         I->SetPixelFormat( PixelFormat::UINT8 );
         I->GetPixelFormat().SetSamplesPerPixel( 3 );
-        pixeldata.SetByteValue( &s[0], s.size() );
+        pixeldata.SetByteValue( &s[0], (uint32_t)s.size() );
         }
       }
     else
@@ -805,7 +816,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
       assert( I->GetPixelFormat() == PixelFormat::UINT16 );
       assert( I->GetPixelFormat().GetPixelRepresentation() == 0 );
       std::string s = is.str();
-      gdcm::Rescaler r;
+      Rescaler r;
       r.SetPixelFormat( I->GetPixelFormat() );
       //r.SetPixelFormat( PixelFormat::UINT16 );
 
@@ -841,7 +852,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         lut.Allocate();
 
         I->SetPixelFormat( PixelFormat::UINT8 );
-        pixeldata.SetByteValue( &v8[0], v8.size() );
+        pixeldata.SetByteValue( &v8[0], (uint32_t)v8.size() );
 
         BuildLUT(*I, 256 );
         }
@@ -849,7 +860,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         {
         I->SetPixelFormat( PixelFormat::UINT8 );
         I->GetPixelFormat().SetSamplesPerPixel( 3 );
-        pixeldata.SetByteValue( &v8[0], v8.size() );
+        pixeldata.SetByteValue( &v8[0], (uint32_t)v8.size() );
         }
       }
     }
@@ -857,17 +868,17 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
     {
     // MONOCHROME1 / MONOCHROME2 ...
     char *buffer2 = &vbuffer2[0];
-    pixeldata.SetByteValue( buffer2, vbuffer2.size() );
+    pixeldata.SetByteValue( buffer2, (uint32_t)vbuffer2.size() );
 
-    gdcm::Rescaler r;
+    Rescaler r;
     r.SetPixelFormat( I->GetPixelFormat() );
 
     // FIXME: This is not accurate. We should either:
     // - Read the value from window/level to get better min,max value
     // - iterate over all possible value to find the min,max as we are looping
     // over all values anyway
-    double min = oripf.GetMin();
-    double max = oripf.GetMax();
+    double min = (double)oripf.GetMin();
+    double max = (double)oripf.GetMax();
     if( Internals->UseMinMax )
       {
       min = Internals->Min;
@@ -875,7 +886,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
       }
     if( Internals->AutoMinMax )
       {
-      const char *p = &vbuffer2[0];
+      const void *p = &vbuffer2[0];
       size_t len = vbuffer2.size();
       const PixelFormat &pf = I->GetPixelFormat();
       assert( pf.GetSamplesPerPixel() == 1 );
@@ -885,16 +896,16 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         switch ( pf.GetScalarType() )
           {
         case PixelFormat::UINT8:
-          ComputeMinMax<uint8_t>( (uint8_t*)p, len / sizeof( uint8_t ), min, max, d);
+          ComputeMinMax<uint8_t>( (const uint8_t*)p, len / sizeof( uint8_t ), min, max, d);
           break;
         case PixelFormat::INT8:
-          ComputeMinMax<int8_t>( (int8_t*)p, len / sizeof( int8_t ), min, max, d);
+          ComputeMinMax<int8_t>( (const int8_t*)p, len / sizeof( int8_t ), min, max, d);
           break;
         case PixelFormat::UINT16:
-          ComputeMinMax<uint16_t>( (uint16_t*)p, len / sizeof( uint16_t ), min, max, d);
+          ComputeMinMax<uint16_t>( (const uint16_t*)p, len / sizeof( uint16_t ), min, max, d);
           break;
         case PixelFormat::INT16:
-          ComputeMinMax<int16_t>( (int16_t*)p, len / sizeof( int16_t ), min, max, d);
+          ComputeMinMax<int16_t>( (const int16_t*)p, len / sizeof( int16_t ), min, max, d);
           break;
         default:
           assert( 0 ); // should not happen
@@ -904,16 +915,16 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
         switch ( pf.GetScalarType() )
           {
         case PixelFormat::UINT8:
-          std::replace( (uint8_t*)p, (uint8_t*)p + len / sizeof( uint8_t ), d, min);
+          std::replace( (uint8_t*)p, (uint8_t*)p + len / sizeof( uint8_t ), (uint8_t)d, (uint8_t)min);
           break;
         case PixelFormat::INT8:
-          std::replace( (int8_t*)p, (int8_t*)p + len / sizeof( int8_t ), d, min);
+          std::replace( (int8_t*)p, (int8_t*)p + len / sizeof( int8_t ), (int8_t)d, (int8_t)min);
           break;
         case PixelFormat::UINT16:
-          std::replace( (uint16_t*)p, (uint16_t*)p + len / sizeof( uint16_t ), d, min);
+          std::replace( (uint16_t*)p, (uint16_t*)p + len / sizeof( uint16_t ), (uint16_t)d, (uint16_t)min);
           break;
         case PixelFormat::INT16:
-          std::replace( (int16_t*)p, (int16_t*)p + len / sizeof( int16_t ), d, min);
+          std::replace( (int16_t*)p, (int16_t*)p + len / sizeof( int16_t ), (int16_t)d, (int16_t)min);
           break;
         default:
           assert( 0 ); // should not happen
@@ -923,16 +934,16 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
       switch ( pf.GetScalarType() )
         {
       case PixelFormat::UINT8:
-        ComputeMinMax<uint8_t>( (uint8_t*)p, len / sizeof( uint8_t ), min, max);
+        ComputeMinMax<uint8_t>( (const uint8_t*)p, len / sizeof( uint8_t ), min, max);
         break;
       case PixelFormat::INT8:
-        ComputeMinMax<int8_t>( (int8_t*)p, len / sizeof( int8_t ), min, max);
+        ComputeMinMax<int8_t>( (const int8_t*)p, len / sizeof( int8_t ), min, max);
         break;
       case PixelFormat::UINT16:
-        ComputeMinMax<uint16_t>( (uint16_t*)p, len / sizeof( uint16_t ), min, max);
+        ComputeMinMax<uint16_t>( (const uint16_t*)p, len / sizeof( uint16_t ), min, max);
         break;
       case PixelFormat::INT16:
-        ComputeMinMax<int16_t>( (int16_t*)p, len / sizeof( int16_t ), min, max);
+        ComputeMinMax<int16_t>( (const int16_t*)p, len / sizeof( int16_t ), min, max);
         break;
       default:
         assert( 0 ); // should not happen
@@ -961,7 +972,7 @@ f. If a Palette Color lookup Table is used, an 8 Bit Allocated (0028,0100) shall
 
     // As per standard, we only support 8bits icon
     I->SetPixelFormat( PixelFormat::UINT8 );
-    pixeldata.SetByteValue( &v8[0], v8.size() );
+    pixeldata.SetByteValue( &v8[0], (uint32_t)v8.size() );
     }
 
   // \postcondition
