@@ -12,8 +12,11 @@
 
 =========================================================================*/
 #include "gdcmLookupTable.h"
+#include "gdcmSwapper.h"
 #include <vector>
 #include <set>
+#include <iomanip>
+#include <algorithm>
 
 #include <string.h>
 
@@ -134,26 +137,42 @@ void LookupTable::SetLUT(LookupTableType type, const unsigned char *array,
   // goes like this: 0, 1, 2, 3, 4, 5, 6, 7 ...
   if( BitSample == 8 )
     {
-    const unsigned int mult = Internal->BitSize[type]/8;
-    assert( Internal->Length[type]*mult == length
-      || Internal->Length[type]*mult + 1 == length );
-    unsigned int offset = 0;
-    if( mult == 2 )
+      const unsigned int mult = Internal->BitSize[type]/8;
+      const unsigned int mult2 = length / Internal->Length[type];
+      assert( Internal->Length[type] * mult2 == length );
+      if( Internal->Length[type]*mult == length
+          || Internal->Length[type]*mult + 1 == length )
       {
-      offset = 1;
+        assert( mult2 == 1 || mult2 == 2 );
+        unsigned int offset = 0;
+        if( mult == 2 )
+        {
+          offset = 1;
+        }
+        for( unsigned int i = 0; i < Internal->Length[type]; ++i)
+        {
+          assert( i*mult+offset < length );
+          assert( 3*i+type < Internal->RGB.size() );
+          Internal->RGB[3*i+type] = array[i*mult+offset];
+        }
       }
-    for( unsigned int i = 0; i < Internal->Length[type]; ++i)
+      else
       {
-      assert( i*mult+offset < length );
-      assert( 3*i+type < Internal->RGB.size() );
-      Internal->RGB[3*i+type] = array[i*mult+offset];
+        unsigned int offset = 0;
+        assert( mult2 == 2 );
+        for( unsigned int i = 0; i < Internal->Length[type]; ++i)
+        {
+          assert( i*mult2+offset < length );
+          assert( 3*i+type < Internal->RGB.size() );
+          Internal->RGB[3*i+type] = array[i*mult2+offset];
+        }
       }
     }
   else if( BitSample == 16 )
     {
     assert( Internal->Length[type]*(BitSample/8) == length );
-    uint16_t *uchar16 = (uint16_t*)&Internal->RGB[0];
-    const uint16_t *array16 = (uint16_t*)array;
+    uint16_t *uchar16 = (uint16_t*)(void*)&Internal->RGB[0];
+    const uint16_t *array16 = (const uint16_t*)(void*)array;
     for( unsigned int i = 0; i < Internal->Length[type]; ++i)
       {
       assert( 2*i < length );
@@ -184,8 +203,8 @@ void LookupTable::GetLUT(LookupTableType type, unsigned char *array, unsigned in
   else if( BitSample == 16 )
     {
     length = Internal->Length[type]*(BitSample/8);
-    uint16_t *uchar16 = (uint16_t*)&Internal->RGB[0];
-    uint16_t *array16 = (uint16_t*)array;
+    uint16_t *uchar16 = (uint16_t*)(void*)&Internal->RGB[0];
+    uint16_t *array16 = (uint16_t*)(void*)array;
     for( unsigned int i = 0; i < Internal->Length[type]; ++i)
       {
       assert( 2*i < length );
@@ -205,7 +224,7 @@ void LookupTable::GetLUTDescriptor(LookupTableType type, unsigned short &length,
     }
   else
     {
-    length = Internal->Length[type];
+    length = (unsigned short)Internal->Length[type];
     }
   subscript = Internal->Subscript[type];
   bitsize = Internal->BitSize[type];
@@ -356,7 +375,7 @@ void LookupTable::Encode(std::istream &is, std::ostream &os)
       assert( u.rgb[3] == 0 );
       int d = 0;
       assert( d < 256 && d >= 0 );
-      os.put( d );
+      os.put( (char)d );
       }
 #endif
     }
@@ -396,7 +415,7 @@ void LookupTable::Encode(std::istream &is, std::ostream &os)
       //std::cout << "Index: " << d << " -> "; printrgb( u.rgb ); std::cout << "\n";
       assert( d < s.size() );
       assert( d < 65536 && d >= 0 );
-      os.write( (char*)&d , 2 );
+      os.write( (const char*)&d , 2 );
       }
 
     // now setup the LUT itself:
@@ -425,7 +444,7 @@ void LookupTable::Encode(std::istream &is, std::ostream &os)
       assert( u.rgb[3] == 0 );
       int d = 0;
       assert( d < 65536 && d >= 0 );
-      os.write( (char*)&d , 2 );
+      os.write( (const char*)&d , 2 );
       }
 #endif
   }
@@ -440,13 +459,8 @@ void LookupTable::Decode(std::istream &is, std::ostream &os) const
     unsigned char rgb[3];
     while( !is.eof() )
       {
-      //is.Get(c);
       is.read( (char*)(&idx), 1);
-      // FIXME
-      if( is.eof() )
-        {
-        break;
-        }
+      if( is.eof() || !is.good() ) break;
       if( IncompleteLUT )
         {
         assert( idx < Internal->Length[RED] );
@@ -461,15 +475,14 @@ void LookupTable::Decode(std::istream &is, std::ostream &os) const
     }
   else if ( BitSample == 16 )
     {
-    const uint16_t *rgb16 = (uint16_t*)&Internal->RGB[0];
+    // gdcmData/NM-PAL-16-PixRep1.dcm
+    const uint16_t *rgb16 = (uint16_t*)(void*)&Internal->RGB[0];
     while( !is.eof() )
       {
       unsigned short idx;
       unsigned short rgb[3];
       is.read( (char*)(&idx), 2);
-      //is.Get(c);
-      // FIXME
-      if( is.eof() ) break;
+      if( is.eof() || !is.good() ) break;
       if( IncompleteLUT )
         {
         assert( idx < Internal->Length[RED] );
@@ -484,13 +497,125 @@ void LookupTable::Decode(std::istream &is, std::ostream &os) const
     }
 }
 
+bool LookupTable::Decode(char *output, size_t outlen, const char *input, size_t inlen ) const
+{
+  bool success = false;
+  if( outlen < 3 * inlen )
+    {
+    gdcmDebugMacro( "Out buffer too small" );
+    return false;
+    }
+  if( !Initialized() )
+    {
+    gdcmDebugMacro( "Not Initialized" );
+    return false;
+    }
+  if ( BitSample == 8 )
+    {
+    const unsigned char * end = (const unsigned char*)input + inlen;
+    unsigned char * rgb = (unsigned char*)output;
+    for( const unsigned char * idx = (const unsigned char*)input; idx != end; ++idx )
+      {
+      if( IncompleteLUT )
+        {
+        assert( *idx < Internal->Length[RED] );
+        assert( *idx < Internal->Length[GREEN] );
+        assert( *idx < Internal->Length[BLUE] );
+        }
+      rgb[RED]   = Internal->RGB[3 * *idx+RED];
+      rgb[GREEN] = Internal->RGB[3 * *idx+GREEN];
+      rgb[BLUE]  = Internal->RGB[3 * *idx+BLUE];
+      rgb += 3;
+      }
+    success = true;
+    }
+  else if ( BitSample == 16 )
+    {
+    const uint16_t *rgb16 = (const uint16_t*)(void*)&Internal->RGB[0];
+    assert( inlen % 2 == 0 );
+    const uint16_t * end = (const uint16_t*)(void*)(input + inlen);
+    uint16_t * rgb = (uint16_t*)(void*)output;
+    for( const uint16_t * idx = (const uint16_t*)(void*)input; idx != end; ++idx )
+      {
+      if( IncompleteLUT )
+        {
+        assert( *idx < Internal->Length[RED] );
+        assert( *idx < Internal->Length[GREEN] );
+        assert( *idx < Internal->Length[BLUE] );
+        }
+      rgb[RED]   = rgb16[3 * *idx+RED];
+      rgb[GREEN] = rgb16[3 * *idx+GREEN];
+      rgb[BLUE]  = rgb16[3 * *idx+BLUE];
+      rgb += 3;
+      }
+    success = true;
+    }
+  return success;
+}
+
+bool LookupTable::Decode8(char *output, size_t outlen, const char *input, size_t inlen ) const
+{
+  bool success = false;
+  if( outlen < 3 * inlen )
+    {
+    gdcmDebugMacro( "Out buffer too small" );
+    return false;
+    }
+  if( !Initialized() )
+    {
+    gdcmDebugMacro( "Not Initialized" );
+    return false;
+    }
+  if ( BitSample == 8 )
+    {
+    const unsigned char * end = (const unsigned char*)input + inlen;
+    unsigned char * rgb = (unsigned char*)output;
+    for( const unsigned char * idx = (const unsigned char*)input; idx != end; ++idx )
+      {
+      if( IncompleteLUT )
+        {
+        assert( *idx < Internal->Length[RED] );
+        assert( *idx < Internal->Length[GREEN] );
+        assert( *idx < Internal->Length[BLUE] );
+        }
+      rgb[RED]   = Internal->RGB[3 * *idx+RED];
+      rgb[GREEN] = Internal->RGB[3 * *idx+GREEN];
+      rgb[BLUE]  = Internal->RGB[3 * *idx+BLUE];
+      rgb += 3;
+      }
+    success = true;
+    }
+  else if ( BitSample == 16 )
+    {
+    const uint16_t *rgb16 = (const uint16_t*)(void*)&Internal->RGB[0];
+    assert( inlen % 2 == 0 );
+    const uint16_t * end = (const uint16_t*)(void*)(input + inlen);
+    uint8_t * rgb = (uint8_t*)output;
+    for( const uint16_t * idx = (const uint16_t*)(void*)input; idx != end; ++idx )
+      {
+      if( IncompleteLUT )
+        {
+        assert( *idx < Internal->Length[RED] );
+        assert( *idx < Internal->Length[GREEN] );
+        assert( *idx < Internal->Length[BLUE] );
+        }
+      rgb[RED]   = rgb16[3 * *idx+RED] >> 8;
+      rgb[GREEN] = rgb16[3 * *idx+GREEN] >> 8;
+      rgb[BLUE]  = rgb16[3 * *idx+BLUE] >> 8;
+      rgb += 3;
+      }
+    success = true;
+    }
+  return success;
+}
+
 const unsigned char *LookupTable::GetPointer() const
 {
   if ( BitSample == 8 )
     {
     return &Internal->RGB[0];
     }
-  return 0;
+  return nullptr;
 }
 
 bool LookupTable::GetBufferAsRGBA(unsigned char *rgba) const
@@ -529,8 +654,8 @@ bool LookupTable::GetBufferAsRGBA(unsigned char *rgba) const
     ret = true;
 */
     //std::vector<unsigned char>::const_iterator it = Internal->RGB.begin();
-    uint16_t *uchar16 = (uint16_t*)&Internal->RGB[0];
-    uint16_t *rgba16 = (uint16_t*)rgba;
+    uint16_t *uchar16 = (uint16_t*)(void*)&Internal->RGB[0];
+    uint16_t *rgba16 = (uint16_t*)(void*)rgba;
     size_t s = Internal->RGB.size();
     s /= 2;
     s /= 3;
@@ -574,8 +699,8 @@ bool LookupTable::WriteBufferAsRGBA(const unsigned char *rgba)
   else if ( BitSample == 16 )
     {
     //assert( Internal->Length[type]*(BitSample/8) == length );
-    uint16_t *uchar16 = (uint16_t*)&Internal->RGB[0];
-    const uint16_t *rgba16 = (uint16_t*)rgba;
+    uint16_t *uchar16 = (uint16_t*)(void*)&Internal->RGB[0];
+    const uint16_t *rgba16 = (const uint16_t*)(void*)rgba;
     size_t s = Internal->RGB.size();
     s /= 2;
     s /= 3;
@@ -603,4 +728,61 @@ bool LookupTable::WriteBufferAsRGBA(const unsigned char *rgba)
   return ret;
 }
 
+void LookupTable::Print(std::ostream &os) const
+{
+  if( BitSample == 16 )
+    {
+    uint16_t maxlut[3] = { 0 , 0 , 0};
+    uint16_t minlut[3] = { 0xffff, 0xffff, 0xffff };
+    uint16_t *uchar16 = (uint16_t*)(void*)&Internal->RGB[0];
+    if( Internal->Length[BLUE] != Internal->Length[RED]
+    || Internal->Length[RED] != Internal->Length[GREEN] ) return;
+    for( unsigned int i = 0; i < Internal->Length[BLUE]; ++i)
+      {
+      os << std::dec <<std::setw( 5 ) << std::setfill( '0' ) << i << " : ";
+      for(int type = RED; type <= BLUE; ++type )
+        {
+        assert( 2*(3*i+type) < Internal->RGB.size() );
+        const uint16_t val = SwapperDoOp::Swap(uchar16[3*i+type]);
+        minlut[type] = std::min( minlut[type], val );
+        maxlut[type] = std::max( maxlut[type], val );
+        os << std::hex << std::setw( 4 ) << std::setfill( '0' ) << val << ",";
+        }
+      os << std::endl;
+      }
+    os << "MINLUT: ";
+    for(int type = RED; type <= BLUE; ++type )
+      os << std::hex << std::setw( 4 ) << std::setfill( '0' ) << minlut[type] << ",";
+    os << std::endl;
+    os << "MAXLUT: ";
+    for(int type = RED; type <= BLUE; ++type )
+      os << std::hex << std::setw( 4 ) << std::setfill( '0' ) << maxlut[type] << ",";
+    os << std::endl;
+    }
+}
+
+bool LookupTable::IsRGB8() const
+{
+  if( BitSample == 16 )
+    {
+    uint16_t maxlut[3] = { 0 , 0 , 0};
+    uint16_t minlut[3] = { 0xffff, 0xffff, 0xffff };
+    uint16_t *uchar16 = (uint16_t*)(void*)&Internal->RGB[0];
+    if( Internal->Length[BLUE] != Internal->Length[RED]
+    || Internal->Length[RED] != Internal->Length[GREEN] ) return false;
+    for( unsigned int i = 0; i < Internal->Length[BLUE]; ++i)
+      {
+      for(int type = RED; type <= BLUE; ++type )
+        {
+        assert( 2*(3*i+type) < Internal->RGB.size() );
+        const uint16_t val = SwapperDoOp::Swap(uchar16[3*i+type]);
+        minlut[type] = std::min( minlut[type], val );
+        maxlut[type] = std::max( maxlut[type], val );
+        if( maxlut[type] > 0xff ) return false;
+        }
+      }
+    }
+  return true;
+}
+
 } // end namespace gdcm
