@@ -34,8 +34,7 @@ PixmapReader::PixmapReader():PixelData(new Pixmap)
 }
 
 PixmapReader::~PixmapReader()
-{
-}
+= default;
 
 const Pixmap& PixmapReader::GetPixmap() const
 {
@@ -85,7 +84,6 @@ bool PixmapReader::Read()
     //assert( ds.FindDataElement( Tag(0x7fe0,0x0010 ) ) );
     assert( ts != TransferSyntax::TS_END && ms != MediaStorage::MS_END );
     // Good it's the easy case. It's declared as an Image:
-    gdcmDebugMacro( "Sweet ! Finally a good DICOM file !" );
     //PixelData->SetCompressionFromTransferSyntax( ts );
     res = ReadImage(ms);
     }
@@ -95,8 +93,15 @@ bool PixmapReader::Read()
   //  }
   else
     {
+    MediaStorage ms2 = ds.GetMediaStorage();
     //assert( !ds.FindDataElement( Tag(0x7fe0,0x0010 ) ) );
-    if( ms != MediaStorage::MS_END )
+    if( ms == MediaStorage::MediaStorageDirectoryStorage && ms2 == MediaStorage::MS_END )
+      {
+      gdcmDebugMacro( "DICOM file is not an Image file but a : " <<
+        MediaStorage::GetMSString(ms) << " SOP Class UID" );
+      res = false;
+      }
+    else if( ms == ms2 && ms != MediaStorage::MS_END )
       {
       gdcmDebugMacro( "DICOM file is not an Image file but a : " <<
         MediaStorage::GetMSString(ms) << " SOP Class UID" );
@@ -104,27 +109,15 @@ bool PixmapReader::Read()
       }
     else
       {
-      // Too bad the media storage type was not recognized...
-      // what should we do ?
-      // Let's check 0008,0016:
-      // D 0008|0016 [UI] [SOP Class UID] [1.2.840.10008.5.1.4.1.1.7 ]
-      // ==> [Secondary Capture Image Storage]
-      const Tag tsopclassuid(0x0008, 0x0016);
-      if( ds.FindDataElement( tsopclassuid) && !ds.GetDataElement( tsopclassuid).IsEmpty() )
+      if( ms2 != MediaStorage::MS_END )
         {
-        const ByteValue *sopclassuid
-          = ImageHelper::GetPointerFromElement( tsopclassuid, *F );
-        std::string sopclassuid_str(
-          sopclassuid->GetPointer(),
-          sopclassuid->GetLength() );
-        MediaStorage ms2 = MediaStorage::GetMSType(sopclassuid_str.c_str());
         bool isImage2 = MediaStorage::IsImage( ms2 );
         if( isImage2 )
           {
           gdcmDebugMacro( "After all it might be a DICOM file "
             "(Mallinckrodt-like)" );
 
-    //PixelData->SetCompressionFromTransferSyntax( ts );
+          //PixelData->SetCompressionFromTransferSyntax( ts );
           //PixelData->SetCompressionType( Compression::RAW );
           res = ReadImage(ms2);
           }
@@ -187,7 +180,7 @@ void DoIconImage(const DataSet& rootds, Pixmap& image)
     //const SequenceOfItems* sq = iconimagesq.GetSequenceOfItems();
     SmartPointer<SequenceOfItems> sq = iconimagesq.GetValueAsSQ();
     // Is SQ empty ?
-    if( !sq ) return;
+    if( !sq || sq->IsEmpty() ) return;
     SequenceOfItems::ConstIterator it = sq->Begin();
     const DataSet &ds = it->GetNestedDataSet();
 
@@ -260,7 +253,7 @@ void DoIconImage(const DataSet& rootds, Pixmap& image)
       pi = PhotometricInterpretation::GetPIType(
         photometricinterpretation_str.c_str());
       }
-    assert( pi != PhotometricInterpretation::UNKNOW);
+    assert( pi != PhotometricInterpretation::UNKNOWN);
     pixeldata.SetPhotometricInterpretation( pi );
 
     //
@@ -307,13 +300,13 @@ void DoIconImage(const DataSet& rootds, Pixmap& image)
           assert( lut_raw );
           // LookupTableType::RED == 0
           lut->SetLUT( LookupTable::LookupTableType(i),
-            (unsigned char*)lut_raw->GetPointer(), lut_raw->GetLength() );
+            (const unsigned char*)lut_raw->GetPointer(), lut_raw->GetLength() );
           //assert( pf.GetBitsAllocated() == el_us3.GetValue(2) );
 
           unsigned long check =
             (el_us3.GetValue(0) ? el_us3.GetValue(0) : 65536)
             * el_us3.GetValue(2) / 8;
-          assert( check == lut_raw->GetLength()
+          assert( check == lut_raw->GetLength() || 2 * check == lut_raw->GetLength()
             || check + 1 == lut_raw->GetLength() ); (void)check;
           }
         else if( ds.FindDataElement( seglut ) )
@@ -321,7 +314,7 @@ void DoIconImage(const DataSet& rootds, Pixmap& image)
           const ByteValue *lut_raw = ds.GetDataElement( seglut ).GetByteValue();
           assert( lut_raw );
           lut->SetLUT( LookupTable::LookupTableType(i),
-            (unsigned char*)lut_raw->GetPointer(), lut_raw->GetLength() );
+            (const unsigned char*)lut_raw->GetPointer(), lut_raw->GetLength() );
           //assert( pf.GetBitsAllocated() == el_us3.GetValue(2) );
 
           //unsigned long check =
@@ -500,11 +493,12 @@ unsigned int GetNumberOfOverlaysInternal(DataSet const & ds, std::vector<uint16_
 
 bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
 {
-  bool updateoverlayinfo = false;
   unsigned int numoverlays;
   std::vector<uint16_t> overlaylist;
+  std::vector<bool> updateoverlayinfo;
   if( (numoverlays = GetNumberOfOverlaysInternal( ds, overlaylist )) )
     {
+    updateoverlayinfo.resize(numoverlays, false);
     pixeldata.SetNumberOfOverlays( numoverlays );
 
     for( unsigned int idxoverlays = 0; idxoverlays < numoverlays; ++idxoverlays )
@@ -552,10 +546,10 @@ bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
 
         if( !ov.GrabOverlayFromPixelData(ds) )
           {
-          gdcmErrorMacro( "Could not extract Overlay from Pixel Data" );
+          gdcmWarningMacro( "Could not extract Overlay from Pixel Data" );
           //throw Exception("TODO: Could not extract Overlay Data");
           }
-        updateoverlayinfo = true;
+        updateoverlayinfo[idxoverlays] = true;
         }
       }
     //std::cout << "Num of Overlays: " << numoverlays << std::endl;
@@ -575,14 +569,15 @@ bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
       if( obp < pf.GetBitsStored() )
         {
         pixeldata.RemoveOverlay( ov );
-        gdcmWarningMacro( "Invalid BitPosition: " << obp << " for overlay #" << ov << " removing it." );
+        updateoverlayinfo.erase( updateoverlayinfo.begin() + ov );
+        gdcmWarningMacro( "Invalid BitPosition: " << obp << " for overlay #" <<
+          ov << " removing it." );
         }
       }
     }
 
-  if( updateoverlayinfo )
-    {
-    for( size_t ov = 0; ov < pixeldata.GetNumberOfOverlays(); ++ov )
+    for( size_t ov = 0;
+      ov < pixeldata.GetNumberOfOverlays() && updateoverlayinfo[ov] ; ++ov )
       {
       Overlay& o = pixeldata.GetOverlay(ov);
       // We need to update information
@@ -593,16 +588,20 @@ bool DoOverlays(const DataSet& ds, Pixmap& pixeldata)
         }
       else
         {
-        gdcmErrorMacro( "Overlay is not supported" );
+        gdcmErrorMacro( "Overlay #" << ov << " is not supported" );
         return false;
         }
       }
-    }
 
   return true;
 }
 
 bool PixmapReader::ReadImage(MediaStorage const &ms)
+{
+  return ReadImageInternal(ms);
+}
+
+bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldata )
 {
   const DataSet &ds = F->GetDataSet();
   std::string conversion;
@@ -714,15 +713,23 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
   const Tag tphotometricinterpretation(0x0028, 0x0004);
   const ByteValue *photometricinterpretation
     = ImageHelper::GetPointerFromElement( tphotometricinterpretation, *F );
-  PhotometricInterpretation pi = PhotometricInterpretation::UNKNOW;
+  PhotometricInterpretation pi = PhotometricInterpretation::UNKNOWN;
   if( photometricinterpretation )
     {
     std::string photometricinterpretation_str(
       photometricinterpretation->GetPointer(),
       photometricinterpretation->GetLength() );
     pi = PhotometricInterpretation::GetPIType( photometricinterpretation_str.c_str() );
+    // http://www.dominator.com/assets/003/5278.pdf
+    // JPEG 2000 lossless YUV_RCT
+    if( pi == PhotometricInterpretation::PI_END )
+      {
+      gdcmWarningMacro( "Discarding suspicious PhotometricInterpretation found: "
+        << photometricinterpretation_str );
+      }
     }
-  else
+  // try again harder:
+  if( !photometricinterpretation || pi == PhotometricInterpretation::PI_END )
     {
     if( pf.GetSamplesPerPixel() == 1 )
       {
@@ -736,27 +743,34 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
       }
     else if( pf.GetSamplesPerPixel() == 4 )
       {
-      gdcmWarningMacro( "No PhotometricInterpretation found, default to RGB" );
+      gdcmWarningMacro( "No PhotometricInterpretation found, default to ARGB" );
       pi = PhotometricInterpretation::ARGB;
       }
+    else
+      {
+      gdcmWarningMacro( "Impossible value for Samples Per Pixel: " << pf.GetSamplesPerPixel() );
+      return false;
+      }
     }
-
+  assert( pi != PhotometricInterpretation::PI_END );
   if( !pf.GetSamplesPerPixel() || (pi.GetSamplesPerPixel() != pf.GetSamplesPerPixel()) )
     {
-    if( pi != PhotometricInterpretation::UNKNOW )
+    if( pi != PhotometricInterpretation::UNKNOWN )
       {
       pf.SetSamplesPerPixel( pi.GetSamplesPerPixel() );
       }
     else if ( isacrnema )
       {
       assert ( pf.GetSamplesPerPixel() == 0 );
-      assert ( pi == PhotometricInterpretation::UNKNOW );
+      assert ( pi == PhotometricInterpretation::UNKNOWN );
       pf.SetSamplesPerPixel( 1 );
       pi = PhotometricInterpretation::MONOCHROME2;
       }
     else
       {
-      gdcmWarningMacro( "Cannot recognize image type. Does not looks like ACR-NEMA and is missing both Sample Per Pixel AND PhotometricInterpretation. Please report" );
+      gdcmWarningMacro( "Cannot recognize image type. Does not looks like"
+        "ACR-NEMA and is missing both Sample Per Pixel AND PhotometricInterpretation."
+        "Please report" );
       return false;
       }
     }
@@ -768,7 +782,7 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
     {
     return false;
     }
-  if( pi == PhotometricInterpretation::UNKNOW ) return false;
+  if( pi == PhotometricInterpretation::UNKNOWN ) return false;
   PixelData->SetPhotometricInterpretation( pi );
 
   // 4. Planar Configuration
@@ -893,7 +907,7 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
           {
           // LookupTableType::RED == 0
           lut->SetLUT( LookupTable::LookupTableType(i),
-            (unsigned char*)lut_raw->GetPointer(), lut_raw->GetLength() );
+            (const unsigned char*)lut_raw->GetPointer(), lut_raw->GetLength() );
           //assert( pf.GetBitsAllocated() == el_us3.GetValue(2) );
           }
         else
@@ -912,7 +926,7 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
         if( lut_raw )
           {
           lut->SetLUT( LookupTable::LookupTableType(i),
-            (unsigned char*)lut_raw->GetPointer(), lut_raw->GetLength() );
+            (const unsigned char*)lut_raw->GetPointer(), lut_raw->GetLength() );
           //assert( pf.GetBitsAllocated() == el_us3.GetValue(2) );
           }
         else
@@ -927,7 +941,7 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
         }
       else
         {
-        assert(0);
+        gdcmAssertAlwaysMacro(0);
         }
       }
     if( ! lut->Initialized() ) return false;
@@ -950,76 +964,94 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
     }
 
   // 8. Do the PixelData
-  if( ms == MediaStorage::MRSpectroscopyStorage )
+  if( handlepixeldata )
     {
-    const Tag spectdata = Tag(0x5600, 0x0020);
-    if( !ds.FindDataElement( spectdata ) )
+    if( ms == MediaStorage::MRSpectroscopyStorage )
       {
-      gdcmWarningMacro( "No Spectroscopy Data Found" );
-      return false;
-      }
-    const DataElement& xde = ds.GetDataElement( spectdata );
-    //bool need = PixelData->GetTransferSyntax() == TransferSyntax::ImplicitVRBigEndianPrivateGE;
-    //PixelData->SetNeedByteSwap( need );
-    PixelData->SetDataElement( xde );
-    }
-  else
-    {
-    const Tag pixeldata = Tag(0x7fe0, 0x0010);
-    if( !ds.FindDataElement( pixeldata ) )
-      {
-      gdcmWarningMacro( "No Pixel Data Found" );
-      return false;
-      }
-    const DataElement& xde = ds.GetDataElement( pixeldata );
-    bool need = PixelData->GetTransferSyntax() == TransferSyntax::ImplicitVRBigEndianPrivateGE;
-    PixelData->SetNeedByteSwap( need );
-    PixelData->SetDataElement( xde );
-
-    // FIXME:
-    // We should check that when PixelData is RAW that Col * Dim == PixelData->GetLength()
-    //PixelFormat guesspf = PixelFormat->GuessPixelFormat();
-
-    }
-
-  const unsigned int *dims = PixelData->GetDimensions();
-  if( dims[0] == 0 || dims[1] == 0 )
-    {
-    // Pseudo-declared JPEG SC image storage. Let's fix col/row/pf/pi
-    gdcm::JPEGCodec jpeg;
-    if( jpeg.CanDecode( PixelData->GetTransferSyntax() ) )
-      {
-      std::stringstream ss;
-      const DataElement &de = PixelData->GetDataElement();
-      //const ByteValue *bv = de.GetByteValue();
-      const SequenceOfFragments *sqf = de.GetSequenceOfFragments();
-      if( !sqf )
+      const Tag spectdata = Tag(0x5600, 0x0020);
+      if( !ds.FindDataElement( spectdata ) )
         {
-        // TODO: It would be nice to recognize file such as JPEGDefinedLengthSequenceOfFragments.dcm
-        gdcmDebugMacro( "File is declared as JPEG compressed but does not contains Fragmens explicitely." );
+        gdcmWarningMacro( "No Spectroscopy Data Found" );
         return false;
         }
-      sqf->WriteBuffer( ss );
-      //std::string s( bv->GetPointer(), bv->GetLength() );
-      //is.str( s );
-      gdcm::PixelFormat jpegpf ( gdcm::PixelFormat::UINT8 ); // usual guess...
-      jpeg.SetPixelFormat( jpegpf );
-      gdcm::TransferSyntax ts;
-      bool b = jpeg.GetHeaderInfo( ss, ts );
-      if( b )
+      const DataElement& xde = ds.GetDataElement( spectdata );
+      //bool need = PixelData->GetTransferSyntax() == TransferSyntax::ImplicitVRBigEndianPrivateGE;
+      //PixelData->SetNeedByteSwap( need );
+      PixelData->SetDataElement( xde );
+      }
+    else
+      {
+      const Tag pixeldata = Tag(0x7fe0, 0x0010);
+      if( !ds.FindDataElement( pixeldata ) )
         {
-        std::vector<unsigned int> v(3);
-        v[0] = PixelData->GetDimensions()[0];
-        v[1] = PixelData->GetDimensions()[1];
-        v[2] = PixelData->GetDimensions()[2];
-        assert( jpeg.GetDimensions()[0] );
-        assert( jpeg.GetDimensions()[1] );
-        v[0] = jpeg.GetDimensions()[0];
-        v[1] = jpeg.GetDimensions()[1];
-        PixelData->SetDimensions( &v[0] );
-        //PixelData->SetPixelFormat( jpeg.GetPixelFormat() );
-        //PixelData->SetPhotometricInterpretation( jpeg.GetPhotometricInterpretation() );
-        assert( PixelData->IsTransferSyntaxCompatible( ts ) );
+        gdcmWarningMacro( "No Pixel Data Found" );
+        return false;
+        }
+      const DataElement& xde = ds.GetDataElement( pixeldata );
+      bool need = PixelData->GetTransferSyntax() == TransferSyntax::ImplicitVRBigEndianPrivateGE;
+      PixelData->SetNeedByteSwap( need );
+      PixelData->SetDataElement( xde );
+
+      // FIXME:
+      // We should check that when PixelData is RAW that Col * Dim == PixelData->GetLength()
+      //PixelFormat guesspf = PixelFormat->GuessPixelFormat();
+
+      }
+
+    const unsigned int *dims = PixelData->GetDimensions();
+    if( dims[0] == 0 || dims[1] == 0 )
+      {
+      // Pseudo-declared JPEG SC image storage. Let's fix col/row/pf/pi
+      JPEGCodec jpeg;
+      if( jpeg.CanDecode( PixelData->GetTransferSyntax() ) )
+        {
+        std::stringstream ss;
+        const DataElement &de = PixelData->GetDataElement();
+        //const ByteValue *bv = de.GetByteValue();
+        const SequenceOfFragments *sqf = de.GetSequenceOfFragments();
+        if( !sqf )
+          {
+          // TODO: It would be nice to recognize file such as JPEGDefinedLengthSequenceOfFragments.dcm
+          gdcmDebugMacro( "File is declared as JPEG compressed but does not contains Fragments explicitly." );
+          return false;
+          }
+        sqf->WriteBuffer( ss );
+        //std::string s( bv->GetPointer(), bv->GetLength() );
+        //is.str( s );
+        PixelFormat jpegpf ( PixelFormat::UINT8 ); // usual guess...
+        jpeg.SetPixelFormat( jpegpf );
+        TransferSyntax ts;
+        bool b = jpeg.GetHeaderInfo( ss, ts );
+        if( b )
+          {
+          std::vector<unsigned int> v(3);
+          v[0] = PixelData->GetDimensions()[0];
+          v[1] = PixelData->GetDimensions()[1];
+          v[2] = PixelData->GetDimensions()[2];
+          assert( jpeg.GetDimensions()[0] );
+          assert( jpeg.GetDimensions()[1] );
+          v[0] = jpeg.GetDimensions()[0];
+          v[1] = jpeg.GetDimensions()[1];
+          PixelData->SetDimensions( &v[0] );
+          //PixelData->SetPixelFormat( jpeg.GetPixelFormat() ); // need to handle carefully
+          if( PixelData->GetPixelFormat().GetSamplesPerPixel() != jpeg.GetPixelFormat().GetSamplesPerPixel() )
+          {
+          gdcmDebugMacro( "Fix samples per pixel." );
+          PixelData->GetPixelFormat().SetSamplesPerPixel( jpeg.GetPixelFormat().GetSamplesPerPixel() );
+          }
+          //PixelData->SetPhotometricInterpretation( jpeg.GetPhotometricInterpretation() );
+          if( PixelData->GetPhotometricInterpretation().GetSamplesPerPixel() != jpeg.GetPhotometricInterpretation().GetSamplesPerPixel() )
+          {
+          gdcmDebugMacro( "Fix photometric interpretation." );
+          PixelData->SetPhotometricInterpretation( jpeg.GetPhotometricInterpretation() );
+          }
+          assert( PixelData->IsTransferSyntaxCompatible( ts ) );
+          }
+        else
+          {
+          gdcmDebugMacro( "Columns or Row was found to be 0. Cannot compute dimension." );
+          return false;
+          }
         }
       else
         {
@@ -1027,11 +1059,6 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
         return false;
         }
       }
-    else
-      {
-      gdcmDebugMacro( "Columns or Row was found to be 0. Cannot compute dimension." );
-      return false;
-      }
     }
 
   // Let's be smart when computing the lossyflag (0028,2110) 
@@ -1045,12 +1072,15 @@ bool PixmapReader::ReadImage(MediaStorage const &ms)
     licat.SetFromDataSet( ds ); // could be empty
     const CSComp & v = licat.GetValue();
     lossyflag = atoi( v.c_str() ) == 1;
+    // Note: technically one can decompress into uncompressed form (eg.
+    // Implicit Little Endian) an input JPEG Lossy. So we need to check
+    // the attribute LossyImageCompression value:
     PixelData->SetLossyFlag(lossyflag);
     }
 
   // Two cases:
   // - DataSet did not specify the lossyflag
-  // - DataSet specify it to be 0, but there is still a change it could be wrong:
+  // - DataSet specify it to be 0, but there is still a chance it could be wrong:
   if( !haslossyflag || !lossyflag )
     {
     PixelData->ComputeLossyFlag();
