@@ -12,6 +12,7 @@
 
 =========================================================================*/
 #include "gdcmPixmapWriter.h"
+#include "gdcmImageHelper.h"
 #include "gdcmTrace.h"
 #include "gdcmDataSet.h"
 #include "gdcmDataElement.h"
@@ -31,8 +32,7 @@ PixmapWriter::PixmapWriter():PixelData(new Pixmap)
 }
 
 PixmapWriter::~PixmapWriter()
-{
-}
+= default;
 
 void PixmapWriter::SetPixmap(Pixmap const &img)
 {
@@ -62,11 +62,11 @@ void PixmapWriter::DoIconImage(DataSet & rootds, Pixmap const & image)
 
     // col & rows:
     Attribute<0x0028, 0x0011> columns;
-    columns.SetValue( icon.GetDimension(0) );
+    columns.SetValue( (uint16_t)icon.GetDimension(0) );
     ds.Insert( columns.GetAsDataElement() );
 
     Attribute<0x0028, 0x0010> rows;
-    rows.SetValue( icon.GetDimension(1) );
+    rows.SetValue( (uint16_t)icon.GetDimension(1) );
     ds.Insert( rows.GetAsDataElement() );
 
   PixelFormat pf = icon.GetPixelFormat();
@@ -93,17 +93,19 @@ void PixmapWriter::DoIconImage(DataSet & rootds, Pixmap const & image)
   if( pf.GetSamplesPerPixel() != 1 )
     {
     Attribute<0x0028, 0x0006> planarconf;
-    planarconf.SetValue( icon.GetPlanarConfiguration() );
+    planarconf.SetValue( (uint16_t)icon.GetPlanarConfiguration() );
     ds.Replace( planarconf.GetAsDataElement() );
     }
   PhotometricInterpretation pi = icon.GetPhotometricInterpretation();
 Attribute<0x0028,0x0004> piat;
     const char *pistr = PhotometricInterpretation::GetPIString(pi);
+{
     DataElement de( Tag(0x0028, 0x0004 ) );
     VL::Type strlenPistr = (VL::Type)strlen(pistr);
     de.SetByteValue( pistr, strlenPistr );
     de.SetVR( piat.GetVR() );
     ds.Replace( de );
+}
 
     if ( pi == PhotometricInterpretation::PALETTE_COLOR )
       {
@@ -172,10 +174,6 @@ Attribute<0x0028,0x0004> piat;
       ds.Replace( bluedesc.GetAsDataElement() );
       }
 
-    //ds.Remove( Tag(0x0028, 0x1221) );
-    //ds.Remove( Tag(0x0028, 0x1222) );
-    //ds.Remove( Tag(0x0028, 0x1223) );
-
 
  {
   // Pixel Data
@@ -231,7 +229,7 @@ Attribute<0x0028,0x0004> piat;
     }
 }
 
-bool PixmapWriter::PrepareWrite()
+bool PixmapWriter::PrepareWrite( MediaStorage const & ref_ms )
 {
   File& file = GetFile();
   DataSet& ds = file.GetDataSet();
@@ -240,12 +238,13 @@ bool PixmapWriter::PrepareWrite()
   const TransferSyntax &ts_orig = fmi_orig.GetDataSetTransferSyntax();
 
   // col & rows:
+#if 0
   Attribute<0x0028, 0x0011> columns;
-  columns.SetValue( PixelData->GetDimension(0) );
+  columns.SetValue( (uint16_t)PixelData->GetDimension(0) );
   ds.Replace( columns.GetAsDataElement() );
 
   Attribute<0x0028, 0x0010> rows;
-  rows.SetValue( PixelData->GetDimension(1) );
+  rows.SetValue( (uint16_t)PixelData->GetDimension(1) );
   ds.Replace( rows.GetAsDataElement() );
 
   // (0028,0008) IS [12]                                     #   2, 1 NumberOfFrames
@@ -263,6 +262,7 @@ bool PixmapWriter::PrepareWrite()
     assert( PixelData->GetDimension(2) == 1 );
     ds.Remove( tnumberofframes );
     }
+#endif
 
   PixelFormat pf = PixelData->GetPixelFormat();
   if ( !pf.IsValid() )
@@ -279,7 +279,7 @@ bool PixmapWriter::PrepareWrite()
     }
 
     {
-    assert( pi != PhotometricInterpretation::UNKNOW );
+    assert( pi != PhotometricInterpretation::UNKNOWN );
     const char *pistr = PhotometricInterpretation::GetPIString(pi);
     DataElement de( Tag(0x0028, 0x0004 ) );
     VL::Type strlenPistr = (VL::Type)strlen(pistr);
@@ -316,13 +316,116 @@ bool PixmapWriter::PrepareWrite()
   if( pf.GetSamplesPerPixel() != 1 )
     {
     Attribute<0x0028, 0x0006> planarconf;
-    planarconf.SetValue( PixelData->GetPlanarConfiguration() );
+    planarconf.SetValue( (uint16_t)PixelData->GetPlanarConfiguration() );
     ds.Replace( planarconf.GetAsDataElement() );
     }
+  // PhotometricInterpretation
+  // const Tag tphotometricinterpretation(0x0028, 0x0004);
+  //if( !ds.FindDataElement( Tag(0x0028, 0x0004) ) )
+    {
+    //if( pi == PhotometricInterpretation::RGB
+    //  || pi == PhotometricInterpretation::YBR_FULL ) // FIXME
+    //  {
+    //  Attribute<0x0028, 0x0006> planarconfiguration;
+    //  planarconfiguration.SetValue( PixelData->GetPlanarConfiguration() );
+    //  ds.Replace( planarconfiguration.GetAsDataElement() );
+    //  }
+    //else
+    if ( pi == PhotometricInterpretation::PALETTE_COLOR )
+      {
+      const LookupTable &lut = PixelData->GetLUT();
+      assert( lut.Initialized() );
+//      assert( (pf.GetBitsAllocated() == 8  && pf.GetPixelRepresentation() == 0)
+//           || (pf.GetBitsAllocated() == 16 && pf.GetPixelRepresentation() == 0) );
+      // lut descriptor:
+      // (0028,1101) US 256\0\16                                 #   6, 3 RedPaletteColorLookupTableDescriptor
+      // (0028,1102) US 256\0\16                                 #   6, 3 GreenPaletteColorLookupTableDescriptor
+      // (0028,1103) US 256\0\16                                 #   6, 3 BluePaletteColorLookupTableDescriptor
+      // lut data:
+      unsigned short length, subscript, bitsize;
+      unsigned short rawlut8[256];
+      unsigned short rawlut16[65536];
+      unsigned short *rawlut = rawlut8;
+      unsigned int lutlen = 256;
+      if( pf.GetBitsAllocated() == 16 )
+        {
+        rawlut = rawlut16;
+        lutlen = 65536;
+        }
+      unsigned int l;
+
+      // FIXME: should I really clear rawlut each time ?
+      // RED
+      memset(rawlut,0,lutlen*2);
+      lut.GetLUT(LookupTable::RED, (unsigned char*)rawlut, l);
+      DataElement redde( Tag(0x0028, 0x1201) );
+      redde.SetVR( VR::OW );
+      redde.SetByteValue( (char*)rawlut, l);
+      ds.Replace( redde );
+      // descriptor:
+      Attribute<0x0028, 0x1101, VR::US, VM::VM3> reddesc;
+      lut.GetLUTDescriptor(LookupTable::RED, length, subscript, bitsize);
+      reddesc.SetValue(length,0); reddesc.SetValue(subscript,1); reddesc.SetValue(bitsize,2);
+      ds.Replace( reddesc.GetAsDataElement() );
+
+      // GREEN
+      memset(rawlut,0,lutlen*2);
+      lut.GetLUT(LookupTable::GREEN, (unsigned char*)rawlut, l);
+      DataElement greende( Tag(0x0028, 0x1202) );
+      greende.SetVR( VR::OW );
+      greende.SetByteValue( (char*)rawlut, l);
+      ds.Replace( greende );
+      // descriptor:
+      Attribute<0x0028, 0x1102, VR::US, VM::VM3> greendesc;
+      lut.GetLUTDescriptor(LookupTable::GREEN, length, subscript, bitsize);
+      greendesc.SetValue(length,0); greendesc.SetValue(subscript,1); greendesc.SetValue(bitsize,2);
+      ds.Replace( greendesc.GetAsDataElement() );
+
+      // BLUE
+      memset(rawlut,0,lutlen*2);
+      lut.GetLUT(LookupTable::BLUE, (unsigned char*)rawlut, l);
+      DataElement bluede( Tag(0x0028, 0x1203) );
+      bluede.SetVR( VR::OW );
+      bluede.SetByteValue( (char*)rawlut, l);
+      ds.Replace( bluede );
+      // descriptor:
+      Attribute<0x0028, 0x1103, VR::US, VM::VM3> bluedesc;
+      lut.GetLUTDescriptor(LookupTable::BLUE, length, subscript, bitsize);
+      bluedesc.SetValue(length,0); bluedesc.SetValue(subscript,1); bluedesc.SetValue(bitsize,2);
+      ds.Replace( bluedesc.GetAsDataElement() );
+      }
+
+    ds.Remove( Tag(0x0028, 0x1221) );
+    ds.Remove( Tag(0x0028, 0x1222) );
+    ds.Remove( Tag(0x0028, 0x1223) );
+
+    }
+
+
+  // Cleanup LUT here since cant be done within gdcm::ImageApplyLookupTable
+  if( pi == PhotometricInterpretation::RGB )
+    {
+    // usual tags:
+    ds.Remove( Tag(0x0028, 0x1101) );
+    ds.Remove( Tag(0x0028, 0x1102) );
+    ds.Remove( Tag(0x0028, 0x1103) );
+
+    ds.Remove( Tag(0x0028, 0x1201) );
+    ds.Remove( Tag(0x0028, 0x1202) );
+    ds.Remove( Tag(0x0028, 0x1203) );
+
+    // Don't forget the segmented one:
+    ds.Remove( Tag(0x0028, 0x1221) );
+    ds.Remove( Tag(0x0028, 0x1222) );
+    ds.Remove( Tag(0x0028, 0x1223) );
+
+    // PaletteColorLookupTableUID ??
+    ds.Remove( Tag(0x0028, 0x1199) );
+    }
 
   // Overlay Data 60xx
-  gdcm::SequenceOfItems::SizeType nOv = PixelData->GetNumberOfOverlays();
-  for(gdcm::SequenceOfItems::SizeType ovidx = 0; ovidx < nOv; ++ovidx )
+  SequenceOfItems::SizeType nOv = PixelData->GetNumberOfOverlays();
+  for(SequenceOfItems::SizeType ovidx = 0; ovidx < nOv; ++ovidx )
     {
     // (6000,0010) US 484                                      #   2, 1 OverlayRows
     // (6000,0011) US 484                                      #   2, 1 OverlayColumns
@@ -388,10 +491,16 @@ bool PixmapWriter::PrepareWrite()
     }
 
   // Pixel Data
-  DataElement de( Tag(0x7fe0,0x0010) );
-  const Value &v = PixelData->GetDataElement().GetValue();
-  de.SetValue( v );
-  const ByteValue *bv = de.GetByteValue();
+  DataElement depixdata( Tag(0x7fe0,0x0010) );
+  DataElement & pde = PixelData->GetDataElement();
+  const ByteValue *bvpixdata = nullptr;
+  // Sometime advanced user may use a gdcm::ImageRegionReader to feed an empty gdcm::Image
+  if( !pde.IsEmpty() )
+    {
+    const Value &v = PixelData->GetDataElement().GetValue();
+    depixdata.SetValue( v );
+    bvpixdata = depixdata.GetByteValue();
+    }
   const TransferSyntax &ts = PixelData->GetTransferSyntax();
   assert( ts.IsExplicit() || ts.IsImplicit() );
 
@@ -415,22 +524,27 @@ bool PixmapWriter::PrepareWrite()
 
   if( /*ts.IsLossy() &&*/ PixelData->IsLossy() )
     {
-    // Add the Lossy stuff:
     Attribute<0x0028,0x2110> at1;
-    at1.SetValue( "01" );
-    ds.Replace( at1.GetAsDataElement() );
-    /*
-    The Defined Terms for Lossy Image Compression Method (0028,2114) ar e :
-    ISO_10918_1 = JPEG Lossy Compression
-    ISO_14495_1 = JPEG-LS Near-lossless Compression
-    ISO_15444_1 = JPEG 2000 Irreversible Compression
-    ISO_13818_2 = MPEG2 Compression
-     */
-
-    if( ts_orig != TransferSyntax::TS_END )
+    Attribute<0x0028,0x2114> at3;
+    if( ts_orig == TransferSyntax::TS_END )
       {
-      assert( ts_orig.IsLossy() );
-      Attribute<0x0028,0x2114> at3;
+      // Add the Lossy stuff:
+      at1.SetValue( "01" );
+      ds.Replace( at1.GetAsDataElement() );
+      }
+    else if( ts_orig.IsLossy() )
+      {
+      // Add the Lossy stuff:
+      at1.SetValue( "01" );
+      ds.Replace( at1.GetAsDataElement() );
+      /*
+      The Defined Terms for Lossy Image Compression Method (0028,2114) are :
+      ISO_10918_1 = JPEG Lossy Compression
+      ISO_14495_1 = JPEG-LS Near-lossless Compression
+      ISO_15444_1 = JPEG 2000 Irreversible Compression
+      ISO_13818_2 = MPEG2 Compression
+       */
+
       if( ts_orig == TransferSyntax::JPEG2000 )
         {
         static const CSComp newvalues2[] = {"ISO_15444_1"};
@@ -453,18 +567,31 @@ bool PixmapWriter::PrepareWrite()
         }
       else
         {
-        gdcmErrorMacro( "Pixel Data is lossy but I cannot find the original transfer syntax" );
+        gdcmErrorMacro(
+          "Pixel Data is lossy but I cannot find the original transfer syntax" );
         return false;
         }
       ds.Replace( at3.GetAsDataElement() );
       }
+    else
+      {
+      if( ds.FindDataElement( at1.GetTag() ) ) {
+            //assert( ds.FindDataElement( at3.GetTag() ) );
+            at1.Set( ds );
+            if( atoi(at1.GetValue().c_str()) != 1 ) {
+               gdcmWarningMacro( "Invalid value for LossyImageCompression" );
+            }
+      } else {
+               gdcmWarningMacro( "Missing attribute for LossyImageCompression" );
+      }
+      }
     }
 
   VL vl;
-  if( bv )
+  if( bvpixdata )
     {
     // if ts is explicit -> set VR
-    vl = bv->GetLength();
+    vl = bvpixdata->GetLength();
     }
   else
     {
@@ -477,12 +604,12 @@ bool PixmapWriter::PrepareWrite()
       {
       case 1:
       case 8:
-        de.SetVR( VR::OB );
+        depixdata.SetVR( VR::OB );
         break;
       case 12:
       case 16:
       case 32:
-        de.SetVR( VR::OW );
+        depixdata.SetVR( VR::OW );
         break;
       default:
         assert( 0 && "should not happen" );
@@ -491,29 +618,41 @@ bool PixmapWriter::PrepareWrite()
     }
   else
     {
-    de.SetVR( VR::OB );
+    depixdata.SetVR( VR::OB );
+    }
+  depixdata.SetVL( vl );
+  // Advanced user may have passed an empty image
+  if( !pde.IsEmpty() )
+    {
+    ds.Replace( depixdata );
     }
-  de.SetVL( vl );
-  ds.Replace( de );
 
   // Do Icon Image
   DoIconImage(ds, GetPixmap());
 
-  MediaStorage ms;
-  ms.SetFromFile( GetFile() );
-  assert( ms != MediaStorage::MS_END );
+  MediaStorage ms = ref_ms;
 
   // Most SOP Class support 2D, but let's make sure that 3D is ok:
   if( PixelData->GetNumberOfDimensions() > 2 )
-    {
+  {
     if( ms.GetModalityDimension() < PixelData->GetNumberOfDimensions() )
+    {
+      // input was specified with SC, but the Number of Frame is > 1. Fix that:
+      ms = ImageHelper::ComputeMediaStorageFromModality( ms.GetModality(),
+          PixelData->GetNumberOfDimensions(),
+          PixelData->GetPixelFormat(),
+          PixelData->GetPhotometricInterpretation(),
+          0, 1 );
+      if( ms.GetModalityDimension() < PixelData->GetNumberOfDimensions() )
       {
-      gdcmErrorMacro( "Problem with NumberOfDimensions and MediaStorage" );
-#if 0
-      return false;
-#endif
+        gdcmErrorMacro( "Problem with NumberOfDimensions and MediaStorage" );
+        return false;
       }
     }
+  }
+  // if we reach here somethnig went really wrong in previous step. Let's make
+  // it a hard failure
+  gdcmAssertAlwaysMacro( ms != MediaStorage::MS_END );
 
   const char* msstr = MediaStorage::GetMSString(ms);
   if( !ds.FindDataElement( Tag(0x0008, 0x0016) ) )
@@ -527,6 +666,11 @@ bool PixmapWriter::PrepareWrite()
   else
     {
     const ByteValue *bv = ds.GetDataElement( Tag(0x0008,0x0016) ).GetByteValue();
+    if( !bv )
+      {
+      gdcmErrorMacro( "Cant be empty" );
+      return false;
+      }
     if( strncmp( bv->GetPointer(), msstr, bv->GetLength() ) != 0 )
       {
       DataElement de = ds.GetDataElement( Tag(0x0008,0x0016) );
@@ -539,6 +683,7 @@ bool PixmapWriter::PrepareWrite()
       assert( bv->GetLength() == strlen( msstr ) || bv->GetLength() == strlen(msstr) + 1 );
       }
     }
+  ImageHelper::SetDimensionsValue(file, *PixelData);
 
   // UIDs:
   // (0008,0018) UI [1.3.6.1.4.1.5962.1.1.1.1.3.20040826185059.5457] #  46, 1 SOPInstanceUID
@@ -550,14 +695,11 @@ bool PixmapWriter::PrepareWrite()
   if( ds.FindDataElement( Tag(0x0008, 0x0018) ) && false )
     {
     // We are coming from a real DICOM image, we need to reference it...
-    //assert( 0 && "TODO FIXME" );
     const Tag tsourceImageSequence(0x0008,0x2112);
-    //assert( ds.FindDataElement( tsourceImageSequence ) == false );
     SmartPointer<SequenceOfItems> sq;
     if( ds.FindDataElement( tsourceImageSequence ) )
       {
       DataElement &de = (DataElement&)ds.GetDataElement( tsourceImageSequence );
-      //assert( de.IsUndefinedLength() );
       de.SetVLToUndefined(); // For now
       if( de.IsEmpty() )
         {
@@ -572,7 +714,6 @@ bool PixmapWriter::PrepareWrite()
       }
     sq->SetLengthToUndefined();
     Item item; //( /*Tag(0xfffe,0xe000)*/ );
-    de.SetVLToUndefined();
     //DataSet sourceimageds;
     // (0008,1150) UI =MRImageStorage                          #  26, 1 ReferencedSOPClassUID
     // (0008,1155) UI [1.3.6.1.4.17434.1.1.5.2.1160650698.1160650698.0] #  48, 1 ReferencedSOPInstanceUID
@@ -627,18 +768,33 @@ bool PixmapWriter::PrepareWrite()
     }
 
   FileMetaInformation &fmi = file.GetHeader();
-  fmi.Clear();
-  //assert( ts == TransferSyntax::ImplicitVRLittleEndian );
-    {
-    const char *tsuid = TransferSyntax::GetTSString( ts );
-    DataElement de( Tag(0x0002,0x0010) );
-    VL::Type strlenTSUID = (VL::Type)strlen(tsuid);
-    de.SetByteValue( tsuid, strlenTSUID );
-    de.SetVR( Attribute<0x0002, 0x0010>::GetVR() );
-    fmi.Replace( de );
-    fmi.SetDataSetTransferSyntax(ts);
-    }
-  fmi.FillFromDataSet( ds );
+  if( GetCheckFileMetaInformation() )
+  {
+    fmi.Clear();
+    //assert( ts == TransferSyntax::ImplicitVRLittleEndian );
+      {
+      const char *tsuid = TransferSyntax::GetTSString( ts );
+      DataElement de( Tag(0x0002,0x0010) );
+      VL::Type strlenTSUID = (VL::Type)strlen(tsuid);
+      de.SetByteValue( tsuid, strlenTSUID );
+      de.SetVR( Attribute<0x0002, 0x0010>::GetVR() );
+      fmi.Replace( de );
+      fmi.SetDataSetTransferSyntax(ts);
+      }
+    fmi.FillFromDataSet( ds );
+  }
+  else
+  {
+      Attribute<0x0002,0x0010> at;
+      at.SetFromDataSet( fmi );
+      const char *tsuid = TransferSyntax::GetTSString( ts );
+      UIComp tsui  = at.GetValue();
+      if( tsui != tsuid )
+      {
+        gdcmErrorMacro( "Incompatible TransferSyntax." );
+         return false;
+      }
+  }
 
 
   return true;
@@ -646,7 +802,17 @@ bool PixmapWriter::PrepareWrite()
 
 bool PixmapWriter::Write()
 {
-  if( !PrepareWrite() ) return false;
+  MediaStorage ms;
+  if( !ms.SetFromFile( GetFile() ) )
+  {
+    // Let's fix some old ACR-NAME stuff:
+    ms = ImageHelper::ComputeMediaStorageFromModality( ms.GetModality(),
+        PixelData->GetNumberOfDimensions(),
+        PixelData->GetPixelFormat(),
+        PixelData->GetPhotometricInterpretation(),
+        0, 1 );
+  }
+  if( !PrepareWrite( ms ) ) return false;
 
   assert( Stream );
   if( !Writer::Write() )
