@@ -21,9 +21,12 @@
 #include "gdcmSwapper.h"
 
 #include <vector>
+#include <algorithm> // req C++11
 #include <stddef.h> // ptrdiff_t fix
 #include <cstring>
 
+#include <gdcmrle/rle.h>
+
 namespace gdcm
 {
 
@@ -47,22 +50,20 @@ public:
 class RLEFrame
 {
 public:
-  void Read(std::istream &is)
+  bool Read(std::istream &is)
     {
     // read Header (64 bytes)
     is.read((char*)(&Header), sizeof(uint32_t)*16);
     assert( sizeof(uint32_t)*16 == 64 );
     assert( sizeof(RLEHeader) == 64 );
-    //ByteSwap<unsigned long>::SwapRangeFromSwapCodeIntoSystem(
-    //  (unsigned long*)&Header, is.GetSwapCode(), 16);
     SwapperNoOp::SwapArray((uint32_t*)&Header,16);
     uint32_t numSegments = Header.NumSegments;
     if( numSegments >= 1 )
       {
-      assert( Header.Offset[0] == 64 );
+      if( Header.Offset[0] != 64 ) return false;
       }
     // We just check that we are indeed at the proper position start+64
-    //char bytes[256*256];
+    return true;
     }
   void Print(std::ostream &os)
     {
@@ -122,7 +123,7 @@ a Literal Run, in which case it's best to merge the three runs into a Literal Ru
 Three-byte repeats shall be encoded as Replicate Runs. Each row of the image shall be encoded
 separately and not cross a row boundary.
 */
-inline int count_identical_bytes(const char *start, unsigned int len)
+inline int count_identical_bytes(const char *start, size_t len)
 {
   assert( len );
 #if 0
@@ -137,7 +138,7 @@ inline int count_identical_bytes(const char *start, unsigned int len)
 #else
   const char ref = start[0];
   unsigned int count = 1; // start at one; make unsigned for comparison
-  const unsigned int cmin = std::min(128u,len);
+  const size_t cmin = std::min((size_t)128,len);
   while( count < cmin && start[count] == ref )
     {
   //std::cerr << "count/len:" << count << "," << len << std::endl;
@@ -149,7 +150,7 @@ inline int count_identical_bytes(const char *start, unsigned int len)
 #endif
 }
 
-inline int count_nonrepetitive_bytes(const char *start, unsigned int len)
+inline int count_nonrepetitive_bytes(const char *start, size_t len)
 {
 /*
  * TODO:
@@ -171,7 +172,7 @@ a Literal Run, in which case it's best to merge the three runs into a Literal Ru
   return p - start;
 #else
   unsigned int count = 1;
-  const unsigned int cmin = std::min(128u,len);
+  const size_t cmin = std::min((size_t)128,len);
 #if 0
   // TODO: this version that handles the note still does not work...
   while( count < cmin )
@@ -216,11 +217,11 @@ a Literal Run, in which case it's best to merge the three runs into a Literal Ru
 }
 
 /* return output length */
-ptrdiff_t rle_encode(char *output, unsigned int outputlength, const char *input, unsigned int inputlength)
+ptrdiff_t rle_encode(char *output, size_t outputlength, const char *input, size_t inputlength)
 {
   char *pout = output;
   const char *pin = input;
-  unsigned int length = inputlength;
+  size_t length = inputlength;
   while( pin != input + inputlength )
     {
     assert( length <= inputlength );
@@ -232,7 +233,7 @@ ptrdiff_t rle_encode(char *output, unsigned int outputlength, const char *input,
       //
       // Test first we are allowed to write two bytes:
       if( pout + 1 + 1 > output + outputlength ) return -1;
-      *pout = -count + 1;
+      *pout = (char)(-count + 1);
       assert( /**pout != -128 &&*/ 1 - *pout == count );
       assert( *pout <= -1 && *pout >= -127 );
       ++pout;
@@ -246,7 +247,7 @@ ptrdiff_t rle_encode(char *output, unsigned int outputlength, const char *input,
       count = count_nonrepetitive_bytes(pin, length);
       // first test we are allowed to write 1 + count bytes in the output buffer:
       if( pout + count + 1 > output + outputlength ) return -1;
-      *pout = count - 1;
+      *pout = (char)(count - 1);
       assert( *pout != -128 && *pout+1 == count );
       assert( *pout >= 0 );
       ++pout;
@@ -296,7 +297,7 @@ bool DoInvertPlanarConfiguration(T *output, const T *input, uint32_t inputlength
     b += 3;
     }
   assert( b == input + length + 2);
-  assert ( pout = output + length );
+  assert ( pout == output + length );
   return true;
 }
 
@@ -326,9 +327,9 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
   unsigned long image_len = bvl / dims[2];
 
   // If 16bits, need to do the padded composite...
-  char *buffer = 0;
+  char *buffer = nullptr;
   // if rgb (3 comp) need to the planar configuration
-  char *bufferrgb = 0;
+  char *bufferrgb = nullptr;
   if( GetPixelFormat().GetBitsAllocated() > 8 )
     {
     //RequestPaddedCompositePixelCode = true;
@@ -343,7 +344,7 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
     bufferrgb = new char [ image_len ];
     }
 
-  int MaxNumSegments = 1;
+  unsigned int MaxNumSegments = 1;
   if( GetPixelFormat().GetBitsAllocated() == 8 )
     {
     MaxNumSegments *= 1;
@@ -378,7 +379,7 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
     assert( MaxNumSegments % 3 == 0 );
     }
 
-  RLEHeader header = { MaxNumSegments, { 64 } };
+  RLEHeader header = { static_cast<uint32_t> ( MaxNumSegments ), { 64 } };
   // there cannot be any space in between the end of the RLE header and the start
   // of the first RLE segment
   //
@@ -393,13 +394,13 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
       {
       if( GetPixelFormat().GetBitsAllocated() == 8 )
         {
-        DoInvertPlanarConfiguration<char>(bufferrgb, ptr_img, image_len / sizeof(char));
+        DoInvertPlanarConfiguration<char>(bufferrgb, ptr_img, (uint32_t)(image_len / sizeof(char)));
         }
       else /* ( GetPixelFormat().GetBitsAllocated() == 16 ) */
         {
         assert( GetPixelFormat().GetBitsAllocated() == 16 );
         // should not happen right ?
-        DoInvertPlanarConfiguration<short>((short*)bufferrgb, (short*)ptr_img, image_len / sizeof(short));
+        DoInvertPlanarConfiguration<short>((short*)(void*)bufferrgb, (const short*)(void*)ptr_img, (uint32_t)(image_len / sizeof(short)));
         }
       ptr_img = bufferrgb;
       }
@@ -480,17 +481,17 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
       ptr_img = buffer;
       }
     assert( image_len % MaxNumSegments == 0 );
-    const int input_seg_length = image_len / MaxNumSegments;
+    const size_t input_seg_length = image_len / MaxNumSegments;
     std::string datastr;
-    for(int seg = 0; seg < MaxNumSegments; ++seg )
+    for(unsigned int seg = 0; seg < MaxNumSegments; ++seg )
       {
-      int partition = input_seg_length;
+      size_t partition = input_seg_length;
       const char *ptr = ptr_img + seg * input_seg_length;
       assert( ptr < ptr_img + image_len );
       if( seg == MaxNumSegments - 1 )
         {
         partition += image_len % MaxNumSegments;
-        assert( (MaxNumSegments-1) * input_seg_length + partition == (int)image_len );
+        assert( (MaxNumSegments-1) * input_seg_length + partition == (size_t)image_len );
         }
       assert( partition == input_seg_length );
 
@@ -503,7 +504,9 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
         ptrdiff_t llength = rle_encode(outbuf, n, ptr + y*dims[0], partition / dims[1] /*image_len*/);
         if( llength < 0 )
           {
-          std::cerr << "RLE compressor error" << std::endl;
+          gdcmErrorMacro( "RLE compressor error" );
+          delete[] buffer;
+          delete[] bufferrgb;
           return false;
           }
         assert( llength );
@@ -511,7 +514,7 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
         length += llength;
         }
       // update header
-      header.Offset[1+seg] = header.Offset[seg] + length;
+      header.Offset[1+seg] = (uint32_t)(header.Offset[seg] + length);
 
       assert( data.str().size() == length );
       datastr += data.str();
@@ -528,18 +531,10 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
     frag.SetByteValue( &str[0], strSize );
     sq->AddFragment( frag );
     }
-
   out.SetValue( *sq );
 
-  if( buffer /*GetPixelFormat().GetBitsAllocated() > 8*/ )
-    {
-    //RequestPaddedCompositePixelCode = true;
-    delete[] buffer;
-    }
-  if ( bufferrgb /*GetPhotometricInterpretation() == PhotometricInterpretation::RGB*/ )
-    {
-    delete[] bufferrgb;
-    }
+  delete[] buffer;
+  delete[] bufferrgb;
 
   return true;
 }
@@ -557,6 +552,49 @@ bool RLECodec::Code(DataElement const &in, DataElement &out)
 // Endif
 // Endloop
 
+size_t RLECodec::DecodeFragment(Fragment const & frag, char *buffer, size_t llen)
+{
+
+  std::stringstream is;
+  const ByteValue &bv = dynamic_cast<const ByteValue&>(frag.GetValue());
+  size_t bv_len = bv.GetLength();
+  char *mybuffer = new char[bv_len];
+  bv.GetBuffer(mybuffer, bv.GetLength());
+  is.write(mybuffer, bv.GetLength());
+  delete[] mybuffer;
+  std::stringstream os;
+  SetLength( (unsigned long)llen );
+#if !defined(NDEBUG)
+  const unsigned int * const dimensions = this->GetDimensions();
+  const PixelFormat & pf = this->GetPixelFormat();
+  assert( llen == dimensions[0] * dimensions[1] * pf.GetPixelSize() );
+#endif
+  bool r = DecodeByStreams(is, os);
+  if( !r ) return 0;
+  (void)r; //warning removal
+  std::streampos p = is.tellg();
+  // http://groups.google.com/group/microsoft.public.vc.stl/browse_thread/thread/96740930d0e4e6b8
+  if( !!is )
+    {
+    // Indeed the length of the RLE stream has been padded with a \0
+    // which is discarded
+    std::streamoff check = bv.GetLength() - p;
+    // check == 2 for gdcmDataExtra/gdcmSampleData/US_DataSet/GE_US/2929J686-breaker
+    //assert( check == 0 || check == 1 || check == 2 );
+    if( check ) gdcmDebugMacro( "tiny offset detected in between RLE segments: " << check );
+    }
+  else
+    {
+    // ALOKA_SSD-8-MONO2-RLE-SQ.dcm
+    gdcmWarningMacro( "Bad RLE stream" );
+    }
+  std::string::size_type check = os.str().size();
+  // If the following assert fail expect big troubles:
+  memcpy(buffer, os.str().c_str(), check);
+//  pos += check;
+  return check;
+}
+
 bool RLECodec::Decode(DataElement const &in, DataElement &out)
 {
   if( NumberOfDimensions == 2 )
@@ -565,14 +603,16 @@ bool RLECodec::Decode(DataElement const &in, DataElement &out)
     const SequenceOfFragments *sf = in.GetSequenceOfFragments();
     if( !sf ) return false;
     unsigned long len = GetBufferLength();
-    //char *buffer = new char[len];
     std::stringstream is;
     sf->WriteBuffer( is );
     SetLength( len );
-      std::stringstream os;
-      bool r = Decode(is, os);
-      assert( r == true );
-      (void)r; //warning removal
+    std::stringstream os;
+    bool r = DecodeByStreams(is, os);
+    if( !r )
+    {
+      gdcmErrorMacro( "DecodeByStreams failure."  );
+      return false;
+    }
     std::string str = os.str();
     std::string::size_type check = str.size();
     assert( check == len );
@@ -585,77 +625,149 @@ bool RLECodec::Decode(DataElement const &in, DataElement &out)
     out = in;
     const SequenceOfFragments *sf = in.GetSequenceOfFragments();
     if( !sf ) return false;
-    unsigned long len = GetBufferLength();
-    char *buffer = new char[len];
+    const unsigned long len = GetBufferLength();
     unsigned long pos = 0;
     // Each RLE Frame store a 2D frame. len is the 3d length
-    unsigned long llen = len / sf->GetNumberOfFragments();
+    const size_t nframes = sf->GetNumberOfFragments();
+    const size_t zdim = Dimensions[2];
+    if( nframes != zdim )
+    {
+      gdcmErrorMacro( "Invalid number of fragments: " << nframes << " should be: " << zdim  );
+      return false;
+    }
+    char *buffer = new char[len];
+    const std::size_t llen = len / nframes;
     // assert( GetNumberOfDimensions() == 2
     //      || GetDimension(2) == sf->GetNumberOfFragments() );
-    for(unsigned int i = 0; i < sf->GetNumberOfFragments(); ++i)
+    bool corruption = false;
+    for(unsigned int i = 0; i < nframes; ++i)
       {
-      std::stringstream is;
       const Fragment &frag = sf->GetFragment(i);
-      const ByteValue &bv = dynamic_cast<const ByteValue&>(frag.GetValue());
-      size_t bv_len = bv.GetLength();
-      char *mybuffer = new char[bv_len];
-      bv.GetBuffer(mybuffer, bv.GetLength());
-      is.write(mybuffer, bv.GetLength());
-      delete[] mybuffer;
-      std::stringstream os;
-      SetLength( llen );
-      bool r = Decode(is, os);
-      assert( r == true );
-      (void)r; //warning removal
-      std::streampos p = is.tellg();
-      // http://groups.google.com/group/microsoft.public.vc.stl/browse_thread/thread/96740930d0e4e6b8
-      if( !!is )
-        {
-        // Indeed the length of the RLE stream has been padded with a \0
-        // which is discarded
-          std::streamoff check = bv.GetLength() - p;
-        // check == 2 for gdcmDataExtra/gdcmSampleData/US_DataSet/GE_US/2929J686-breaker
-        assert( check == 0 || check == 1 || check == 2 );
-        if( check ) gdcmWarningMacro( "tiny offset detected in between RLE segments" );
-        }
-      else
-        {
-        // ALOKA_SSD-8-MONO2-RLE-SQ.dcm
-        gdcmWarningMacro( "Bad RLE stream" );
-        }
-      std::string::size_type check = os.str().size();
-      // If the following assert fail expect big troubles:
-      assert( check == llen );
-      memcpy(buffer+pos, os.str().c_str(), check);
-      pos += check;
+      const size_t check = DecodeFragment(frag, buffer + pos, llen); (void)check;
+      if( check != llen )
+      {
+        gdcmDebugMacro( "RLE pb with frag: " << i );
+        corruption = true;
       }
-    assert( pos == len );
-    out.SetByteValue( buffer, len );
+      pos += (unsigned long)llen;
+      }
+    if( !corruption )
+      assert( pos == len );
+    out.SetByteValue( buffer, (uint32_t)len );
     delete[] buffer;
-    return true;
+    return !corruption;
     }
   return false;
 }
 
-bool RLECodec::Decode(std::istream &is, std::ostream &os)
+bool RLECodec::DecodeExtent(
+  char *buffer,
+  unsigned int xmin, unsigned int xmax,
+  unsigned int ymin, unsigned int ymax,
+  unsigned int zmin, unsigned int zmax,
+  std::istream & is
+)
 {
+  std::stringstream tmpos;
+  BasicOffsetTable bot;
+  bot.Read<SwapperNoOp>( is );
+  //std::cout << bot << std::endl;
+
+  const unsigned int * dimensions = this->GetDimensions();
+  const PixelFormat & pf = this->GetPixelFormat();
+  assert( pf.GetBitsAllocated() % 8 == 0 );
+  assert( pf != PixelFormat::SINGLEBIT );
+  assert( pf != PixelFormat::UINT12 && pf != PixelFormat::INT12 );
+
+  // skip
+  std::stringstream os;
+  Fragment frag;
+  for( unsigned int z = 0; z < zmin; ++z )
+    {
+    frag.ReadPreValue<SwapperNoOp>(is);
+    std::streamoff off = frag.GetVL();
+    is.seekg( off, std::ios::cur );
+    }
+  for( unsigned int z = zmin; z <= zmax; ++z )
+    {
+    frag.ReadPreValue<SwapperNoOp>(is);
+    std::streampos start = is.tellg();
+
+    SetLength( dimensions[0] * dimensions[1] * pf.GetPixelSize() );
+    const bool r = DecodeByStreams(is, os); (void)r;
+    if( !r ) return false;
+    assert( r );
+
+    // handle DICOM padding
+    std::streampos end = is.tellg();
+    size_t numberOfReadBytes = (size_t)(end - start);
+    if( numberOfReadBytes > frag.GetVL() )
+      {
+      // Special handling for ALOKA_SSD-8-MONO2-RLE-SQ.dcm
+      size_t diff = numberOfReadBytes - frag.GetVL();
+      assert( diff == 1 );
+      os.seekp( 0 - (int)diff, std::ios::cur );
+      os.put( 0 );
+      end = (size_t)end - 1;
+      }
+    assert( end - start == frag.GetVL() || (size_t)(end - start) + 1 == frag.GetVL() );
+    // sync is (rle16loo.dcm)
+    if( (end - start) % 2 == 1 )
+      {
+      is.get();
+      }
+    } // for each z
+
+  os.seekg(0, std::ios::beg );
+  assert( os.good() );
+  std::istream *theStream = &os;
+
+  unsigned int rowsize = xmax - xmin + 1;
+  unsigned int colsize = ymax - ymin + 1;
+  unsigned int bytesPerPixel = pf.GetPixelSize();
+
+  std::vector<char> buffer1;
+  buffer1.resize( rowsize*bytesPerPixel );
+  char *tmpBuffer1 = &buffer1[0];
+  unsigned int y, z;
+  std::streamoff theOffset;
+  for (z = zmin; z <= zmax; ++z)
+    {
+    for (y = ymin; y <= ymax; ++y)
+      {
+      theStream->seekg(std::ios::beg);
+      theOffset = 0 + ((z-zmin)*dimensions[1]*dimensions[0] + y*dimensions[0] + xmin)*bytesPerPixel;
+      theStream->seekg(theOffset);
+      theStream->read(tmpBuffer1, rowsize*bytesPerPixel);
+      memcpy(&(buffer[((z-zmin)*rowsize*colsize +
+            (y-ymin)*rowsize)*bytesPerPixel]),
+        tmpBuffer1, rowsize*bytesPerPixel);
+      }
+    }
+  return true;
+}
+
+bool RLECodec::DecodeByStreamsCommon(std::istream &, std::ostream &)
+{
+  return false;
+}
+
+bool RLECodec::DecodeByStreams(std::istream &is, std::ostream &os)
+{
+  std::streampos start = is.tellg();
   // FIXME: Do some stupid work:
   char dummy_buffer[256];
   std::stringstream tmpos;
-  // DEBUG
-  is.seekg( 0, std::ios::end);
-  std::streampos buf_size = is.tellg();
-  is.seekg(0, std::ios::beg);
-  // END DEBUG
 
   RLEFrame &frame = Internals->Frame;
-  frame.Read(is);
-  //frame.Print(std::cout);
+  if( !frame.Read(is) )
+     return false;
   unsigned long numSegments = frame.Header.NumSegments;
 
   unsigned long numberOfReadBytes = 0;
 
   unsigned long length = Length;
+  assert( length );
   // Special case:
   assert( GetPixelFormat().GetBitsAllocated() == 32 ||
           GetPixelFormat().GetBitsAllocated() == 16 ||
@@ -667,12 +779,14 @@ bool RLECodec::Decode(std::istream &is, std::ostream &os)
 
   assert( GetPixelFormat().GetSamplesPerPixel() == 3 || GetPixelFormat().GetSamplesPerPixel() == 1 );
   // A footnote:
-  // RLE *by definition* with more than one component will have applied the Planar Configuration
-  // because it simply does not make sense to do it otherwise. So implicitely
-  // RLE is indeed PlanarConfiguration == 1. However when the image says: "hey I am PlanarConfiguration = 0
-  // AND RLE", then apply the PlanarConfiguration internally so that people don't get lost
-  // Because GDCM internally set PlanarConfiguration == 0 by default, even if the Attribute is not
-  // sent, it will still default to 0 and we will be consistant with ourselves...
+  // RLE *by definition* with more than one component will have applied the
+  // Planar Configuration because it simply does not make sense to do it
+  // otherwise. So implicitely RLE is indeed PlanarConfiguration == 1. However
+  // when the image says: "hey I am PlanarConfiguration = 0 AND RLE", then
+  // apply the PlanarConfiguration internally so that people don't get lost
+  // Because GDCM internally set PlanarConfiguration == 0 by default, even if
+  // the Attribute is not sent, it will still default to 0 and we will be
+  // consistent with ourselves...
   if( GetPixelFormat().GetSamplesPerPixel() == 3 && GetPlanarConfiguration() == 0 )
     {
     RequestPlanarConfiguration = true;
@@ -681,7 +795,7 @@ bool RLECodec::Decode(std::istream &is, std::ostream &os)
   for(unsigned long i = 0; i<numSegments; ++i)
     {
     numberOfReadBytes = 0;
-    std::streampos pos = is.tellg();
+    std::streampos pos = is.tellg() - start;
     if ( frame.Header.Offset[i] - pos != 0 )
       {
       // ACUSON-24-YBR_FULL-RLE.dcm
@@ -691,27 +805,29 @@ bool RLECodec::Decode(std::istream &is, std::ostream &os)
       //   " when it should says: " << pos << std::endl );
       std::streamoff check = frame.Header.Offset[i] - pos;//should it be a streampos or a uint32? mmr
       // check == 2 for gdcmDataExtra/gdcmSampleData/US_DataSet/GE_US/2929J686-breaker
-      assert( check == 1 || check == 2);
+      //assert( check == 1 || check == 2);
       (void)check; //warning removal
-      is.seekg( frame.Header.Offset[i], std::ios::beg );
+      is.seekg( frame.Header.Offset[i] + start, std::ios::beg );
       }
 
     unsigned long numOutBytes = 0;
     signed char byte;
-    //std::cerr << "Length: " << Length << "\n";
-    //assert( (uint32_t)is.Tellg() == frame.Header.Offset[i] );
 
-    // FIXME: ALOKA_SSD-8-MONO2-RLE-SQ.dcm
-    // I think the RLE decoder is off by one, we are reading in 128001 byte, while only 128000
-    // are present
+    // FIXME: ALOKA_SSD-8-MONO2-RLE-SQ.dcm I think the RLE decoder is off by
+    // one, we are reading in 128001 byte, while only 128000 are present
     while( numOutBytes < length )
       {
       is.read((char*)&byte, 1);
-      assert( is.good() );
+      if( !is.good() )
+      {
+        gdcmErrorMacro( "Could not decode" );
+        return false;
+      }
       numberOfReadBytes++;
       if( byte >= 0 /*&& byte <= 127*/ ) /* 2nd is always true */
         {
         is.read( dummy_buffer, byte+1 );
+        //assert( is.good() ); // impossible because ALOKA_SSD-8-MONO2-RLE-SQ.dc
         numberOfReadBytes += byte+1;
         numOutBytes += byte+ 1;
         tmpos.write( dummy_buffer, byte+1 );
@@ -722,10 +838,6 @@ bool RLECodec::Decode(std::istream &is, std::ostream &os)
         is.read( &nextByte, 1);
         numberOfReadBytes += 1;
         memset(dummy_buffer, nextByte, -byte + 1);
-        //for( int c = 0; c < -byte + 1; ++c )
-        //  {
-        //  dummy_buffer[c] = nextByte;
-        //  }
         numOutBytes += -byte + 1;
         tmpos.write( dummy_buffer, -byte+1 );
         }
@@ -733,22 +845,170 @@ bool RLECodec::Decode(std::istream &is, std::ostream &os)
         {
         assert( byte == -128 );
         }
-        assert( is.eof()
-        || numberOfReadBytes + frame.Header.Offset[i] - is.tellg() == 0);
-      //std::cerr << "numOutBytes: " << numOutBytes << " / " << length << "\n";
-      //std::cerr << "numberOfReadBytes: " << numberOfReadBytes << "\n";
+      //assert( numberOfReadBytes + frame.Header.Offset[i] - is.tellg() + start == 0);
       }
-    assert( numOutBytes == length );
+    if( numOutBytes != length ) return false;
     }
 
-  return ImageCodec::Decode(tmpos,os);
+  return ImageCodec::DecodeByStreams(tmpos,os);
 }
 
 bool RLECodec::GetHeaderInfo(std::istream &is, TransferSyntax &ts)
 {
-  (void)is;
+  RLEFrame frame;
+  if( !frame.Read(is) )
+     return false;
+  // numsegments = num_comp * bpp / 8;
+  // numsegments >0 && numsegments <= 12
+  uint32_t bytespercomp = frame.Header.NumSegments;
+  if( frame.Header.NumSegments % 3 == 0 )
+  {
+    PI = PhotometricInterpretation::RGB;
+    PlanarConfiguration = 1;
+    this->PF.SetSamplesPerPixel( 3 );
+    bytespercomp /= 3;
+  }
+  else
+  {
+    PI = PhotometricInterpretation::MONOCHROME2;
+    this->PF.SetSamplesPerPixel( 1 );
+  }
+  this->PF.SetBitsAllocated( bytespercomp * 8 );
   ts = TransferSyntax::RLELossless;
   return true;
 }
 
+ImageCodec * RLECodec::Clone() const
+{
+  return new RLECodec;
+}
+
+bool RLECodec::StartEncode( std::ostream & )
+{
+  return true;
+}
+bool RLECodec::IsRowEncoder()
+{
+  return false;
+}
+
+bool RLECodec::IsFrameEncoder()
+{
+  return true;
+}
+
+class memsrc : public ::rle::source
+{
+public:
+  memsrc( const char * data, size_t datalen ):ptr(data),cur(data),len(datalen)
+    {
+    }
+  int read( char * out, int l ) override
+    {
+    memcpy( out, cur, l );
+    cur += l;
+    assert( cur <= ptr + len );
+    return l;
+    }
+  streampos_t tell() override
+    {
+    assert( cur <= ptr + len );
+    return (streampos_t)(cur - ptr);
+    }
+  bool seek(streampos_t pos) override
+    {
+    cur = ptr + pos;
+    assert( cur <= ptr + len && cur >= ptr );
+    return true;
+    }
+  bool eof() override
+    {
+    assert( cur <= ptr + len );
+    return cur == ptr + len;
+    }
+  memsrc * clone() override
+    {
+    memsrc * ret = new memsrc( ptr, len );
+    return ret;
+    }
+private:
+  const char * ptr;
+  const char * cur;
+  size_t len;
+};
+
+bool RLECodec::AppendRowEncode( std::ostream & os, const char * data, size_t datalen)
+{
+  (void)os; (void)data; (void)datalen;
+  assert(0);
+  return false;
+}
+
+class streamdest : public rle::dest
+{
+public:
+  streamdest( std::ostream & os ):stream(os)
+  {
+  start = os.tellp();
+  }
+  int write( const char * in, int len ) override
+    {
+    stream.write(in, len );
+    return len;
+    }
+  bool seek( streampos_t abs_pos ) override
+    {
+    stream.seekp( abs_pos + start );
+    return true;
+    }
+private:
+  std::ostream & stream;
+  std::streampos start;
+};
+
+bool RLECodec::AppendFrameEncode( std::ostream & out, const char * data, size_t datalen )
+{
+  try {
+  const PixelFormat & pf = this->GetPixelFormat();
+  unsigned int pc = this->GetPlanarConfiguration();
+  bool isLittleEndian = !this->GetNeedByteSwap();
+  rle::pixel_info pi((unsigned char)pf.GetSamplesPerPixel(), (unsigned char)(pf.GetBitsAllocated()));
+
+  const unsigned int * dimensions = this->GetDimensions();
+  rle::image_info ii(dimensions[0], dimensions[1], pi, pc ? true : false, isLittleEndian);
+  const int h = dimensions[1];
+
+  memsrc src( data, datalen );
+  rle::rle_encoder re(src, ii);
+
+  streamdest fd( out );
+
+  if( !re.write_header( fd ) )
+    {
+    gdcmErrorMacro( "could not write header" );
+    return false;
+    }
+
+  for( int y = 0; y < h; ++y )
+    {
+    const int ret = re.encode_row( fd );
+    if( ret < 0 )
+      {
+      gdcmErrorMacro( "problem at row: " << y );
+      return false;
+      }
+    }
+  } catch( std::exception &  ) {
+    gdcmErrorMacro( "invalid compression params (not supported for now)." );
+    return false;
+  }
+
+  return true;
+}
+
+bool RLECodec::StopEncode( std::ostream & )
+{
+  return true;
+}
+
 } // end namespace gdcm
