@@ -39,11 +39,11 @@ static const char * const gdcmMD5DataImages[][2] = {
 /* gdcm 384 288 1 8 3 */
 //{ "2a922d5c606354612bfdbece1421d863" , "ACUSON-24-YBR_FULL-RLE-b.dcm" },
 //{ "22b32f23beb118f7b64c13bf04bc2809" , "ACUSON-24-YBR_FULL-RLE-b.dcm" },
-{ "2d7a28cae6c3b3183284d1b4ae08307f" , "ACUSON-24-YBR_FULL-RLE-b.dcm" },
+{ "22b32f23beb118f7b64c13bf04bc2809" , "ACUSON-24-YBR_FULL-RLE-b.dcm" },
 /* gdcm 768 576 1 8 3 */
 //{ "f9bb8a37acabdf8b0cfa4fd1b471e6aa" , "ACUSON-24-YBR_FULL-RLE.dcm" },
 //{ "435c66f7e113d11d226d500294aae865" , "ACUSON-24-YBR_FULL-RLE.dcm" },
-{ "429f31f0b70bd515b3feeda5dea5eac0" , "ACUSON-24-YBR_FULL-RLE.dcm" },
+{ "435c66f7e113d11d226d500294aae865" , "ACUSON-24-YBR_FULL-RLE.dcm" },
 /* gdcm 768 576 25 8 3 */
 { "e36350b0711fd34eb86c386164554679" , "ACUSON-8-YBR_FULL-JPEG-TrailingInfo.dcm" },
 /* gdcm 608 420 1 8 1 */
@@ -650,7 +650,7 @@ static const char * const gdcmMD5DataImages[][2] = {
 
 { "6bf95a48f366bdf8af3a198c7b723c77" , "SinglePrecisionSC.dcm" },
 
-{ "a870a7a7cab8c17646d118ae146be588" , "MR16BitsAllocated_8BitsStored.dcm" },
+{ "42cb593979fcc4630f965c31caaf4ec1" , "MR16BitsAllocated_8BitsStored.dcm" },
 
 { "da153c2f438d6dd4277e0c6ad2aeae74" , "OsirixFake16BitsStoredFakeSpacing.dcm" },
 { "1c485e1ac2b2bdbeeba14391b8c1e0c8" , "JPEGDefinedLengthSequenceOfFragments.dcm" },
@@ -661,7 +661,7 @@ static const char * const gdcmMD5DataImages[][2] = {
 { "5e9085c66976d2f5f9989d88bf7a90c4" , "D_CLUNIE_CT1_JLSN.dcm" },
 
 { "9eb513314b2fcf25d895e18ffb2ead0b" , "UnexpectedSequenceDelimiterInFixedLengthSequence.dcm" },
-{ "12d1567ed81236cf3b01dc12766581a0" , "IM-0001-0066.CommandTag00.dcm" },
+{ "fb8119f30ec32ff575789caa969180ca" , "IM-0001-0066.CommandTag00.dcm" },
 { "6f26e552a1b71d386483118779d192ad" , "NM_Kakadu44_SOTmarkerincons.dcm" },
 { "56238d3665ebdb0251d1161fb7f4edc6" , "GDCMJ2K_TextGBR.dcm" },
 { "f8a1f4ce85b51527267e670a8aa0c308" , "PhilipsInteraSeqTermInvLen.dcm" },
@@ -679,6 +679,33 @@ static const char * const gdcmMD5DataImages[][2] = {
 { "d40e2d27988f0c546b0daeb67fcbfba8" , "i32.XADC.7.215MegaBytes.dcm" },
 { "c383b244fd43cb0a9db764b71cb59741" , "1.3.46.670589.7.5.10.80002138018.20001204.181556.9.1.1.dcm" },
 
+{ "f111ed4eea73f535261039d3f7b112e9" , "JPEGInvalidSecondFrag.dcm" },
+
+// Computed with kakadu:
+{ "a218fcee00e16d430f30ec7ebd4937dc" , "lena512_rot90.j2k.dcm" },
+
+{ "dbf83984984741c98205d77a947b442c" , "SC16BitsAllocated_8BitsStoredJ2K.dcm" },
+{ "c164a73ba18ab4e88977921ffc7c3a65" , "SC16BitsAllocated_8BitsStoredJPEG.dcm" },
+
+{ "4a5d8549791432b29948dd8ff63e2534" , "SIEMENS_SYNGO-12-MONO2-LUTWrongLength512.dcm" },
+
+{ "2c8630ae1bff925b258d98e742dfc632" , "JPEGLS_CharLS_10742.dcm" },
+{ "f666a95dde9ddfdec0cf9574a7985ae3" , "IllegalGroup2ImplicitTS.dcm" },
+
+{ "b85686dd3169b99725149abb00181254" , "RLEDebianBug816607Orig.dcm" },
+{ "464960d948dff0ded72b0380080f18a3" , "JPEGNote_empty.dcm" },
+{ "cdfb5dd0c999d88badeb00a16f96554f" , "JPEGLosslessYBR_FULL_422.dcm" },
+{ "464960d948dff0ded72b0380080f18a3" , "JPEGNote_missing.dcm" },
+{ "161e9fd986ea87adf2596e8e16641f11" , "JPEGNote_bogus.dcm" },
+
+{ "ad8f1164f0517ee150bf510d5dba5cc6" , "GE_MR_0025xx1bProtocolDataBlockXML.dcm" },
+
+{ "a0e0effa8d5638721b19e592acc11a4c" , "EmptyIcon_Bug417.dcm" },
+
+{ "2b64303085d747956bfdda2ee8cc33bc" , "EnhancedWithIPPPerFrameIOPShared.dcm" },
+{ "4ca2f3a88d21d941f6485a8d91f96e29" , "FUJI-ffff-MONO1-J2K.dcm" },
+
+
 /* Stopping condition */
-{ 0 ,0 }
+{ nullptr ,nullptr }
 };
