@@ -24,7 +24,6 @@ struct StreamOffset
 
 static const StreamOffset gdcmStreamOffsetDataFiles[] = {
 { "00191113.dcm",934 },
-{ "00191113.dcm",934 },
 { "012345.002.050.dcm",9604 },
 { "05115014-mr-siemens-avanto-syngo-with-palette-icone.dcm",41948 },
 { "05148044-mr-siemens-avanto-syngo.dcm",55884 },
@@ -45,9 +44,9 @@ static const StreamOffset gdcmStreamOffsetDataFiles[] = {
 { "CT-SIEMENS-MissingPixelDataInIconSQ.dcm",20504 },
 { "CT_16b_signed-UsedBits13.dcm",2582 },
 { "DCMTK_JPEGExt_12Bits.dcm",796 },
-{ "DICOMDIR",-1 },
-{ "DICOMDIR-Philips-EasyVision-4200-Entries",-1 },
-{ "DICOMDIR_MR_B_VA12A",-1 },
+{ "DICOMDIR",1112434 },
+{ "DICOMDIR-Philips-EasyVision-4200-Entries",1297194 },
+{ "DICOMDIR_MR_B_VA12A", 14853880},
 { "DMCPACS_ExplicitImplicit_BogusIOP.dcm",66618 },
 { "DX_GE_FALCON_SNOWY-VOI.dcm",104014 },
 { "DX_J2K_0Padding.dcm",1494 },
@@ -95,7 +94,7 @@ static const StreamOffset gdcmStreamOffsetDataFiles[] = {
 { "D_CLUNIE_XA1_RLE.dcm",1128 },
 { "DermaColorLossLess.dcm",940 },
 { "ELSCINT1_JP2vsJ2K.dcm",1820 },
-{ "ELSCINT1_PMSCT_RLE1.dcm",-1 },
+{ "ELSCINT1_PMSCT_RLE1.dcm",67912 },
 { "ExplicitVRforPublicElementsImplicitVRforShadowElements.dcm",9546 },
 { "FUJI-10-MONO1-ACR_NEMA_2.dcm",862 },
 { "GE_CT_With_Private_compressed-icon.dcm",10810 },
@@ -146,7 +145,7 @@ static const StreamOffset gdcmStreamOffsetDataFiles[] = {
 { "MR_Philips_Intera_PrivateSequenceImplicitVR.dcm",7026 },
 { "MR_Philips_Intera_SwitchIndianess_noLgtSQItem_in_trueLgtSeq.dcm",9276 },
 { "MR_SIEMENS_forceLoad29-1010_29-1020.dcm",60564 },
-{ "MR_Spectroscopy_SIEMENS_OF.dcm",-1 },
+{ "MR_Spectroscopy_SIEMENS_OF.dcm",2102344 },
 { "NM-MONO2-16-13x-heart.dcm",1234 },
 { "OT-MONO2-8-a7.dcm",438 },
 { "OT-PAL-8-face.dcm",1654 },
@@ -190,10 +189,10 @@ static const StreamOffset gdcmStreamOffsetDataFiles[] = {
 { "US-RGB-8-epicard.dcm",1012 },
 { "US-RGB-8-esopecho.dcm",904 },
 { "XA-MONO2-8-12x-catheter.dcm",1006 },
-{ "dicomdir_Acusson_WithPrivate_WithSR",-1 },
-{ "dicomdir_Pms_WithVisit_WithPrivate_WithStudyComponents",-1 },
-{ "dicomdir_Pms_With_heavy_embedded_sequence",-1 },
-{ "dicomdir_With_embedded_icons",-1 },
+{ "dicomdir_Acusson_WithPrivate_WithSR",3826 },
+{ "dicomdir_Pms_WithVisit_WithPrivate_WithStudyComponents",37218 },
+{ "dicomdir_Pms_With_heavy_embedded_sequence",3003414 },
+{ "dicomdir_With_embedded_icons", 7125212 },
 { "fffc0000UN.dcm",1154 },
 { "gdcm-ACR-LibIDO.acr",182 },
 { "gdcm-CR-DCMTK-16-NonSamplePerPix.dcm",740 },
@@ -222,8 +221,27 @@ static const StreamOffset gdcmStreamOffsetDataFiles[] = {
 { "TOSHIBA_J2K_SIZ0_PixRep1.dcm", 2346  },
 { "NM-PAL-16-PixRep1.dcm", 4444 },
 { "MEDILABInvalidCP246_EVRLESQasUN.dcm", 19174 },
-
+{ "JPEGInvalidSecondFrag.dcm", 4800 },
+{ "lena512_rot90.j2k.dcm", 768 },
+{ "ELSCINT1_PMSCT_RLE1_priv.dcm", 298002 },
+{ "unreadable.dcm", 249230 },
+{ "SC16BitsAllocated_8BitsStoredJ2K.dcm", 764 },
+{ "SC16BitsAllocated_8BitsStoredJPEG.dcm", 958 },
+{ "SIEMENS_SYNGO-12-MONO2-LUTWrongLength512.dcm", 48726 },
+{ "JPEGNote_empty.dcm", 716 },
+{ "JPEGLS_CharLS_10742.dcm", 760 },
+{ "JPEGLosslessYBR_FULL_422.dcm", 814 },
+{ "JPEGNote_missing.dcm", 608 },
+{ "JPEGNote_bogus.dcm", 734 },
+{ "RLEDebianBug816607Orig.dcm", 780 },
+{ "IllegalGroup2ImplicitTS.dcm", 2272 },
+{ "JPEG_LS_InvalidEscapeSequence_COM_padding.dcm", 764 },
+{ "LengthOfItemLarger.dcm", 12748 },
+{ "GE_MR_0025xx1bProtocolDataBlockXML.dcm",6474 },
+{ "EmptyIcon_Bug417.dcm",4482 },
+{ "EnhancedWithIPPPerFrameIOPShared.dcm",23224 },
+{ "FUJI-ffff-MONO1-J2K.dcm",2052 },
 
 /* Stopping condition */
-{ 0 ,0 },
+{ nullptr ,0 },
 };
