@@ -19,6 +19,7 @@
 #include "vtkGDCMImageReader.h"
 
 #include "vtkVersion.h"
+#include "vtkErrorCode.h"
 #include "vtkMedicalImageProperties.h"
 #include "vtkXMLImageDataWriter.h"
 #include "vtkInteractorStyleImage.h"
@@ -30,7 +31,7 @@
 #include "vtkPolyDataMapper2D.h"
 #include "vtkImageReslice.h"
 #include "vtkRenderWindowInteractor.h"
-#include "vtkImageViewer.h"
+#include "vtkRenderWindow.h"
 #include "vtkPointData.h"
 #include "vtkImageMapToColors.h"
 #include "vtkLookupTable.h"
@@ -38,7 +39,7 @@
 #include "vtkImageMapToWindowLevelColors.h"
 #include "vtkImageActor.h"
 #include "vtkWindowToImageFilter.h"
-#if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
+#if VTK_MAJOR_VERSION >= 6 || (VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0)
 #include "vtkImageMapToColors16.h"
 #include "vtkBalloonWidget.h"
 #include "vtkBalloonRepresentation.h"
@@ -87,45 +88,17 @@ class vtkAngleWidget;
 
 #include <getopt.h>
 #include <assert.h>
-//----------------------------------------------------------------------------
-// vtkImageViewer2 new interface wants SetSlice, but vtkImageViewer does not have
-// this new interface (what a pain), so let's fake a new interface to
-// vtkImageViewer without patching VTK
-class vtkGDCMImageViewer : public vtkImageViewer
-{
-public:
-  vtkTypeRevisionMacro(vtkGDCMImageViewer,vtkImageViewer);
 
-  static vtkGDCMImageViewer *New()
-    {
-#ifdef VTK_DEBUG_LEAKS
-    vtkDebugLeaks::ConstructClass("vtkGDCMImageViewer");
+#ifndef vtkFloatingPointType
+#define vtkFloatingPointType double
 #endif
-    return new vtkGDCMImageViewer;
-    }
-  int GetSlice() { return this->GetZSlice(); }
-  void SetSlice(int s) { this->SetZSlice(s); }
-
-  int GetSliceMin() { return this->GetWholeZMin(); }
-  int GetSliceMax() { return this->GetWholeZMax(); }
-#if VTK_MAJOR_VERSION >= 5
-  // TODO:
-  void AddInputConnection(vtkAlgorithmOutput* input) {(void)input;}
-#else
-  void AddInput(vtkImageData * input) {(void)input;}
-#endif
-  double GetOverlayVisibility() { return 0; }
-  void SetOverlayVisibility(double vis) {(void)vis;}
-};
-vtkCxxRevisionMacro(vtkGDCMImageViewer, "$Revision: 1.30 $")
-vtkInstantiatorNewMacro(vtkGDCMImageViewer)
 
 #if VTK_MAJOR_VERSION >= 5
 #else
 class vtkImageColorViewer : public vtkImageViewer2
 {
 public:
-  vtkTypeRevisionMacro(vtkImageColorViewer,vtkImageViewer2);
+  vtkTypeMacro(vtkImageColorViewer,vtkImageViewer2);
 
   static vtkImageColorViewer *New()
     {
@@ -163,7 +136,7 @@ public:
 private:
   vtkImageActor                   *OverlayImageActor;
 };
-vtkCxxRevisionMacro(vtkImageColorViewer, "$Revision: 1.30 $")
+//vtkCxxRevisionMacro(vtkImageColorViewer, "$Revision: 1.30 $")
 vtkInstantiatorNewMacro(vtkImageColorViewer)
 #endif
 
@@ -212,14 +185,18 @@ public:
           vtkPNGWriter * writer = vtkPNGWriter::New();
           vtkWindowToImageFilter * w2i = vtkWindowToImageFilter::New();
           w2i->SetInput( rwi->GetRenderWindow() );
+#if VTK_MAJOR_VERSION >= 6
+          writer->SetInputConnection( w2i->GetOutputPort() );
+#else
           writer->SetInput( w2i->GetOutput() );
+#endif
           writer->SetFileName( "snapshot.png" );
           writer->Write();
           writer->Delete();
           w2i->Delete();
           //std::cerr << "Screenshort saved to snapshot.png" << std::endl;
           }
-#if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
+#if VTK_MAJOR_VERSION >= 6 || (VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0)
         else if ( keycode == 'l' )
           {
           IconWidget->Off();
@@ -246,7 +223,7 @@ public:
 #endif
         else
           {
-#if (VTK_MAJOR_VERSION >= 5) || ( VTK_MAJOR_VERSION == 4 && VTK_MINOR_VERSION > 5 )
+#if VTK_MAJOR_VERSION >= 5 || (VTK_MAJOR_VERSION == 4 && VTK_MINOR_VERSION > 5)
           int max = ImageViewer->GetSliceMax();
           int slice = (ImageViewer->GetSlice() + 1) % ++max;
           ImageViewer->SetSlice( slice );
@@ -287,7 +264,7 @@ public:
 
 int verbose;
 
-#if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
+#if VTK_MAJOR_VERSION >= 6 || (VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0)
 class vtkBalloonCallback : public vtkCommand
 {
 public:
@@ -307,20 +284,31 @@ public:
 void FillCornerFromMedProp( vtkCornerAnnotation *ca, vtkMedicalImageProperties *medprop )
 {
   std::string bottomleft = "S: ";
-  bottomleft += medprop->GetSeriesNumber();
+  if( medprop->GetSeriesNumber() )
+    bottomleft += medprop->GetSeriesNumber();
   bottomleft += "\nI: ";
-  bottomleft += medprop->GetImageNumber();
+  if( medprop->GetImageNumber() )
+    bottomleft += medprop->GetImageNumber();
   bottomleft += "\n<window_level>";
   ca->SetText(0, bottomleft.c_str());
-  std::string bottomright = medprop->GetStationName();
-  ca->SetText(1, bottomright.c_str());
-  std::string topleft = medprop->GetInstitutionName();
+  if( medprop->GetStationName() )
+    {
+    const std::string bottomright = medprop->GetStationName();
+    ca->SetText(1, bottomright.c_str());
+    }
+  std::string topleft;
+  if( medprop->GetInstitutionName() )
+    topleft += medprop->GetInstitutionName();
   topleft += "\n";
-  topleft += medprop->GetPatientName();
+  if( medprop->GetPatientName() )
+    topleft += medprop->GetPatientName();
   ca->SetText(2, topleft.c_str());
-  std::string topright = medprop->GetStudyDate();
+  std::string topright;
+  if( medprop->GetStudyDate() )
+    topright += medprop->GetStudyDate();
   topright += "\n";
-  topright += medprop->GetAcquisitionTime();
+  if( medprop->GetAcquisitionTime() )
+    topright += medprop->GetAcquisitionTime();
   ca->SetText(3, topright.c_str());
 }
 
@@ -348,6 +336,13 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
 
   //reader->FileLowerLeftOn();
   reader->Update();
+  if( reader->GetErrorCode() )
+    {
+    std::cerr << "There was an error: " << vtkErrorCode::
+      GetStringFromErrorCode(reader->GetErrorCode()) << std::endl;
+    return;
+    }
+
   //reader->Print( cout );
   if( verbose )
     reader->GetOutput()->Print( cout );
@@ -367,7 +362,7 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
     viewer->AddInputConnection ( reader->GetOverlayPort(0) );
     }
   // TODO: Icon can be added using the vtkLogoWidget
-#if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
+#if VTK_MAJOR_VERSION >= 6 || (VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0)
   vtkPointHandleRepresentation2D *handle = vtkPointHandleRepresentation2D::New();
   handle->GetProperty()->SetColor(1,0,0);
   vtkDistanceRepresentation2D *drep = vtkDistanceRepresentation2D::New();
@@ -421,7 +416,11 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
       {
       vtkLookupTable *lut = reader->GetIconImage()->GetPointData()->GetScalars()->GetLookupTable();
       vtkImageMapToColors *map = vtkImageMapToColors::New ();
+#if (VTK_MAJOR_VERSION >= 6)
+      map->SetInputData(reader->GetIconImage());
+#else
       map->SetInput (reader->GetIconImage());
+#endif
       map->SetLookupTable ( lut );
       //FIXME there is no way to know the type of LUT the icon is using:
       //if( reader->GetImageFormat() == VTK_LOOKUP_TABLE )
@@ -487,7 +486,7 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
   // MONOCHROME1 is also implemented with a lookup table
   if( reader->GetImageFormat() == VTK_LOOKUP_TABLE || reader->GetImageFormat() == VTK_INVERSE_LUMINANCE )
     {
-#if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
+#if VTK_MAJOR_VERSION >= 6 || (VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0)
     assert( reader->GetOutput()->GetPointData()->GetScalars()
       && reader->GetOutput()->GetPointData()->GetScalars()->GetLookupTable() );
 #endif
@@ -498,11 +497,15 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
       // This must be a Segmented Palette and on VTK 4.4 this is not supported
       std::cerr << "Not implemented. You will not see the Color LUT" << std::endl;
       }
-#if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
+#if VTK_MAJOR_VERSION >= 6 || (VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0)
     if( lut->IsA( "vtkLookupTable16" ) )
       {
       vtkImageMapToColors16 *map = vtkImageMapToColors16::New ();
+#if VTK_MAJOR_VERSION >= 6
+      map->SetInputConnection (reader->GetOutputPort());
+#else
       map->SetInput (reader->GetOutput());
+#endif
       map->SetLookupTable (reader->GetOutput()->GetPointData()->GetScalars()->GetLookupTable());
       if( reader->GetImageFormat() == VTK_LOOKUP_TABLE )
         {
@@ -514,14 +517,22 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
         }
       map->Update();
       map->GetOutput()->GetScalarRange(range);
+#if VTK_MAJOR_VERSION >= 6
+      viewer->SetInputConnection( map->GetOutputPort() );
+#else
       viewer->SetInput( map->GetOutput() );
+#endif
       map->Delete();
       }
     else
 #endif
       {
       vtkImageMapToColors *map = vtkImageMapToColors::New ();
+#if VTK_MAJOR_VERSION >= 6
+      map->SetInputConnection (reader->GetOutputPort());
+#else
       map->SetInput (reader->GetOutput());
+#endif
       map->SetLookupTable (reader->GetOutput()->GetPointData()->GetScalars()->GetLookupTable());
       if( reader->GetImageFormat() == VTK_LOOKUP_TABLE )
         {
@@ -533,7 +544,11 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
         }
       map->Update();
       map->GetOutput()->GetScalarRange(range);
+#if VTK_MAJOR_VERSION >= 6
+      viewer->SetInputConnection( map->GetOutputPort() );
+#else
       viewer->SetInput( map->GetOutput() );
+#endif
       map->Delete();
       }
     }
@@ -541,10 +556,18 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
     {
 #if VTK_MAJOR_VERSION >= 5
     vtkImageYBRToRGB *filter = vtkImageYBRToRGB::New();
+#if VTK_MAJOR_VERSION >= 6
+    filter->SetInputConnection( reader->GetOutputPort() );
+#else
     filter->SetInput( reader->GetOutput() );
+#endif
     filter->Update();
     filter->GetOutput()->GetScalarRange(range);
+#if VTK_MAJOR_VERSION >= 6
+    viewer->SetInputConnection( filter->GetOutputPort() );
+#else
     viewer->SetInput( filter->GetOutput() );
+#endif
     filter->Delete();
 #else
     std::cerr << "Not implemented" << std::endl;
@@ -567,7 +590,11 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
   if( reader->GetCurve() )
     {
     vtkPolyDataMapper2D * rectMapper = vtkPolyDataMapper2D::New();
+#if VTK_MAJOR_VERSION >= 6
+    rectMapper->SetInputData( reader->GetCurve() );
+#else
     rectMapper->SetInput( reader->GetCurve() );
+#endif
 
     vtkActor2D * rectActor = vtkActor2D::New();
     rectActor->SetMapper( rectMapper );
@@ -644,7 +671,7 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
   // Here is where we setup the observer,
   vtkGDCMObserver<TViewer> *obs = vtkGDCMObserver<TViewer>::New();
   obs->ImageViewer = viewer;
-#if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
+#if VTK_MAJOR_VERSION >= 6 || (VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0)
   if(iconwidget) iconwidget->On();
   obs->IconWidget = iconwidget;
   obs->DistanceWidget = dwidget;
@@ -687,7 +714,7 @@ void ExecuteViewer(TViewer *viewer, vtkStringArray *filenames)
 #endif
 
   reader->Delete();
-#if VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0
+#if VTK_MAJOR_VERSION >= 6 || (VTK_MAJOR_VERSION >= 5 && VTK_MINOR_VERSION > 0)
   cbk->Delete();
   dwidget->Off();
   balloonwidget->Off();
@@ -785,7 +812,7 @@ int main(int argc, char *argv[])
       {
     case 0:
         {
-        const char *s = long_options[option_index].name;
+        const char *s = long_options[option_index].name; (void)s;
         //printf ("option %s", s);
         if (optarg)
           {
