@@ -204,8 +204,8 @@ extern "C" {
 
 // Do not include anything below that define. That should in no case change any forward decls in
 // system headers ...
-#if (defined(__APPLE__)&&(__GNUC__<4)) || (defined(WIN32)&&!defined(__CYGWIN__)) || \
- (!defined(__APPLE__) && !defined(WIN32) && !defined(_XOPEN_SOURCE_EXTENDED) && !defined(__FreeBSD__))
+#if (defined(__APPLE__)&&(__GNUC__<3)) || (defined(WIN32)&&!defined(__CYGWIN__)) || \
+ (!defined(__APPLE__) && !defined(WIN32) && !defined(__GNUC__) && !defined(_XOPEN_SOURCE_EXTENDED) && !defined(__FreeBSD__))
 #define socklen_t int
 #endif
 
@@ -272,7 +272,7 @@ bool sockerr::op () const
   case EHOSTDOWN:
   case EHOSTUNREACH:
   case ENOTEMPTY:
-#   if !defined(__linux__) && !defined(__sun) && !defined(__hpux)// LN
+#   if !defined(__linux__) && !defined(__sun) && !defined(__hpux) && !defined(__EMSCRIPTEN__) // LN
   case EPROCLIM:
 #   endif
   case EUSERS:
@@ -336,7 +336,7 @@ sockbuf::sockbuf (const sockbuf::sockdesc& thesd)
 }
 
 sockbuf::sockbuf (int domain, sockbuf::type st, int proto)
-  : rep (0)
+  : rep (nullptr)
 {
 #if defined(WIN32) && !defined(__CYGWIN__)
   WORD version = MAKEWORD(1,1);
@@ -364,7 +364,7 @@ sockbuf::sockbuf (int domain, sockbuf::type st, int proto)
 
 sockbuf::sockbuf (const sockbuf& sb)
 :
-streambuf(),
+std::streambuf(),
 //streambuf (sb),
 rep (sb.rep)
 {
@@ -429,8 +429,8 @@ int sockbuf::sync ()
         catch (int wlen)
         {
             // write was not completely successful
-            stringstream sb;
-            string err ("sockbuf::sync");
+            std::stringstream sb;
+            std::string err ("sockbuf::sync");
             err += "(" + sockname.text + ")";
             if (wlen) {
                 // reposition unwritten chars
@@ -455,7 +455,7 @@ int sockbuf::sync ()
     return 0;
 }
 
-streamsize sockbuf::showmanyc ()
+std::streamsize sockbuf::showmanyc ()
 // return the number of chars in the input sequence
 {
     if (gptr () && gptr () < egptr ())
@@ -465,7 +465,7 @@ streamsize sockbuf::showmanyc ()
 
 sockbuf::int_type sockbuf::underflow ()
 {
-  if (gptr () == 0)
+  if (gptr () == nullptr)
     return eof; // input stream has been disabled
 
   if (gptr () < egptr ())
@@ -491,7 +491,7 @@ sockbuf::int_type sockbuf::uflow ()
   return ret;
 }
 
-streamsize sockbuf::xsgetn (char_type* s, streamsize n)
+std::streamsize sockbuf::xsgetn (char_type* s, std::streamsize n)
 {
   std::streamsize rval = showmanyc ();
   if (rval >= n) {
@@ -521,7 +521,7 @@ sockbuf::int_type sockbuf::overflow (sockbuf::int_type c)
 //                         insert c into buffer, and return c.
 // In all cases, if error happens, throw exception.
 {
-  if (pbase () == 0)
+  if (pbase () == nullptr)
     return eof;
 
   if (c == eof)
@@ -534,7 +534,7 @@ sockbuf::int_type sockbuf::overflow (sockbuf::int_type c)
   return c;
 }
 
-streamsize sockbuf::xsputn (const char_type* s, streamsize n)
+std::streamsize sockbuf::xsputn (const char_type* s, std::streamsize n)
 {
   std::streamsize wval = epptr () - pptr ();
   if (n <= wval) {
@@ -577,15 +577,15 @@ sockbuf::sockdesc sockbuf::accept (sockAddr& sa)
   if ((int)(soc = ::accept (rep->sock, sa.addr (),
                        &len)) == -1)
     throw sockerr (errno, "sockbuf::sockdesc", sockname.text.c_str());
-  return sockdesc (soc);
+  return {soc};
 }
 
 sockbuf::sockdesc sockbuf::accept ()
 {
   int soc = -1;
-  if ((int)(soc = ::accept (rep->sock, 0, 0)) == -1)
+  if ((int)(soc = ::accept (rep->sock, nullptr, nullptr)) == -1)
     throw sockerr (errno, "sockbuf::sockdesc", sockname.text.c_str());
-  return sockdesc (soc);
+  return {soc};
 }
 
 int sockbuf::read (void* buf, int len)
@@ -742,7 +742,7 @@ int sockbuf::is_readready (int wp_sec, int wp_usec) const
   tv.tv_sec  = wp_sec;
   tv.tv_usec = wp_usec;
 
-  int ret = select ((int)(rep->sock)+1, &fds, 0, 0, (wp_sec == -1) ? 0: &tv);
+  int ret = select ((int)(rep->sock)+1, &fds, nullptr, nullptr, (wp_sec == -1) ? nullptr: &tv);
   if (ret == -1) throw sockerr (errno, "sockbuf::is_readready", sockname.text.c_str());
   return ret;
 }
@@ -757,7 +757,7 @@ int sockbuf::is_writeready (int wp_sec, int wp_usec) const
   tv.tv_sec  = wp_sec;
   tv.tv_usec = wp_usec;
 
-  int ret = select ((int)(rep->sock)+1, 0, &fds, 0, (wp_sec == -1) ? 0: &tv);
+  int ret = select ((int)(rep->sock)+1, nullptr, &fds, nullptr, (wp_sec == -1) ? nullptr: &tv);
   if (ret == -1) throw sockerr (errno, "sockbuf::is_writeready", sockname.text.c_str());
   return ret;
 }
@@ -772,7 +772,7 @@ int sockbuf::is_exceptionpending (int wp_sec, int wp_usec) const
   tv.tv_sec = wp_sec;
   tv.tv_usec = wp_usec;
 
-  int ret = select ((int)(rep->sock)+1, 0, 0, &fds, (wp_sec == -1) ? 0: &tv);
+  int ret = select ((int)(rep->sock)+1, nullptr, nullptr, &fds, (wp_sec == -1) ? nullptr: &tv);
   if (ret == -1) throw sockerr (errno, "sockbuf::is_exceptionpending", sockname.text.c_str());
   return ret;
 }
@@ -782,11 +782,11 @@ void sockbuf::shutdown (shuthow sh)
   switch (sh) {
   case shut_read:
     delete [] eback ();
-    setg (0, 0, 0);
+    setg (nullptr, nullptr, nullptr);
     break;
   case shut_write:
     delete [] pbase ();
-    setp (0, 0);
+    setp (nullptr, nullptr);
     break;
   case shut_readwrite:
     shutdown (shut_read);
